package mario;

public class Bowser extends WorldObject {

  public static final float SPEED = 0.5f;

  public int minX;
  public int maxX;
  public int targetX;
  public int jumpDelay;
  public int spitFireDelay;
  public int hits = 5;
  public boolean throwsHammers;
  public int throwDelay;
  public int availableHammers = 8;

  public Bowser(int x, int y, boolean mapCoordinates) {
    if (mapCoordinates) {
      x <<= 5;
      y <<= 5;
    }

    this.fx = this.x = x;
    this.fy = this.y = y;

    maxX = x;
    minX = x - 352;
    vx = -SPEED;

    targetX = minX + random.nextInt(x - minX);

    bx1 = mx1 = 0;
    by1 = my1 = 0;
    bx2 = mx2 = 63;
    by2 = my2 = 63;

    G = 0.1f;
    supports = false;
    jumpDelay = random.nextInt(120);

    playingMode.bowser = this;

    throwsHammers = playingMode.world >= 6;
  }

  @Override
  public void update() {
    
    if (playingMode.axeState != PlayingMode.AXE_STATE_NONE) {
      return;
    }

    if (applyGravity(true)) {
      if (jumpDelay == 120) {
        vy = -3.4f;
        jumpDelay = 0;
        if (random.nextBoolean()) {
          jumpDelay = 60;
        }
      } else {
        jumpDelay++;
      }
    }

    if (playingMode.mario.x < x) {
      if (spitFireDelay <= 0) {
        spitFireDelay = 240;
        if (random.nextInt(playingMode.mario.hasFlower ? 3 : 6) == 0) {
          spitFireDelay >>= 1;
        }
        playingMode.enemies.add(new SpitFire(x, 
            ((y + random.nextInt(48)) & 0xFFFFFFE0) + 8));
      } else {
        spitFireDelay--;
      }
    }

    if (throwsHammers) {
      if (throwDelay > 0) {
        throwDelay--;
      } else {
        availableHammers--;
        playingMode.items.add(new Hammer(x + 16, y,
            playingMode.mario.x < x ? -3f : 3f, -7f));
        if (availableHammers == 0) {
          availableHammers = 8;
          throwDelay = 60 + random.nextInt(60);
        } else {
          throwDelay = 2 + random.nextInt(10);
        }
      }
    }

    translateX(vx, true);

    if (vx < 0) {
      if (x <= targetX || x <= minX) {
        vx = SPEED;
        if (playingMode.mario.x > x + 32) {
          targetX = Math.min(maxX, playingMode.mario.x + 64);
        } else {
          targetX = x + random.nextInt(maxX - x);
        }
      }
    } else {
      if (x >= targetX || x >= maxX) {
        vx = -SPEED;
        targetX = minX + random.nextInt(x - minX);
      }
    }
  }

  @Override
  public boolean blast(int px, int py, boolean left) {
    if (withinMargin(px, py)) {
      if (--hits == 0) {
        switch(playingMode.world) {
          case 1:
            playingMode.others.add(new UpsideDownDeadGoomba(
                x + 16, y + 16, left ? -4 : 4));
            break;
          case 2:
            playingMode.others.add(new UpsideDownDeadKoopaShell(
                x + 16, y + 16, left ? -4 : 4, KoopaTroopaTypes.GREEN));
            break;
          case 3:
            playingMode.others.add(new UpsideDownDeadKoopaShell(
                x + 16, y + 16, left ? -4 : 4, KoopaTroopaTypes.BUZZY_BEETLE));
            break;
          case 4:
            playingMode.others.add(new UpsideDownDeadSpiny(
                x + 16, y + 16, left ? -4 : 4, -8));
            break;
          case 5:
            playingMode.others.add(new UpsideDownDeadLakitu(
                x + 16, y + 16, left ? -4 : 4));
            break;
          case 6:
            playingMode.others.add(new UpsideDownDeadBloober(
                x + 16, y + 16, left ? -4 : 4));
            break;
          case 7:
            playingMode.others.add(new UpsideDownDeadHammerBrother(
                x + 16, y + 16, left ? -4 : 4));
            break;
          case 8:
            playingMode.others.add(new UpsideDownDeadBowser(
                x + 32, y + 32, left ? -4 : 4, -8));
            break;
        }
        playingMode.addPoints(this, 5000);
        remove = true;
      }
      return true;
    }
    return false;
  }

  @Override
  public boolean hurtsMario() {
    return !remove && intersects(playingMode.mario, true);
  }

  @Override
  public void render() {
    g.setColor(Palette.BOWSER);
    g.translate(x - playingMode.cameraX, y);
    g.drawOval(2, 0, 59, 23);
    g.drawLine(31, 24, 31, 44);
    g.drawLine(32, 24, 32, 44);
    g.drawLine(31, 44, 0, 63);
    g.drawLine(32, 44, 63, 63);
    g.drawLine(31, 27, 0, 41);
    g.drawLine(32, 27, 63, 41);
    g.translate(playingMode.cameraX - x, -y);
  }

}
