package mario;

public class BrickPlatform extends AbstractPlatform {

  public static final int STEPS = 16;
  public static final float INCREMENT = (float)(Math.PI / STEPS);
  public static final float RADIUS = 8;

  public int mapX;
  public int mapY;
  public float t;
  public int steps = STEPS;
  public int originY;
  public boolean hidden;

  public BrickPlatform(int px, int py) {
    this(px, py, false);
  }

  public BrickPlatform(int px, int py, boolean hidden) {
    this.mapX = px >> 5;
    this.mapY = py >> 5;
    playingMode.physicsMap[mapY][mapX] = BlockTypes.EMPTY;
    playingMode.displayMap[mapY][mapX] = BlockTypes.EMPTY;
    x = mapX << 5;
    originY = y = mapY << 5;
    deadly = true;
    this.hidden = hidden;

    if (getPhysicsMap(mapX, mapY - 1) == BlockTypes.COIN) {
      playingMode.physicsMap[mapY - 1][mapX] = BlockTypes.EMPTY;
      playingMode.displayMap[mapY - 1][mapX] = BlockTypes.EMPTY;
      playingMode.others.add(new FlyingCoin(x, y - 32));
    }
  }

  @Override
  public void move() {
    if (--steps == 0) {
      if (!hidden) {
        playingMode.physicsMap[mapY][mapX] = BlockTypes.BRICK;
        playingMode.displayMap[mapY][mapX] = BlockTypes.BRICK;
      }
      remove = true;
    } else {
      y = (int)(originY - RADIUS * Math.sin(t));
      t += INCREMENT;
    }
  }

  @Override
  public void render() {
    if (!hidden) {
      g.setColor(Palette.STONES[playingMode.stoneIndex]);
      g.translate((int)x - playingMode.cameraX, (int)y);
      g.drawRect(0, 0, 31, 31);
      g.translate(playingMode.cameraX - (int)x, -(int)y);
    }
  }
}
