
package mario;

import java.awt.*;

public class CheepCheep extends WorldObject {

  public static final float ANGLE_SPEED = (float)(2.0 * Math.PI / (12 * 60));

  public float speed = 0.5f;
  public CheepCheepSpawner cheepCheepSpawner;
  public Color color;
  public boolean oscillates;
  public float angle;
  public float centerY;

  public CheepCheep(CheepCheepSpawner cheepCheepSpawner) {

    this.cheepCheepSpawner = cheepCheepSpawner;

    y = (4 + random.nextInt(8)) << 5;
    fy = y;
    fx = x = playingMode.cameraX + 512;

    if (random.nextBoolean()) {
      speed = 1f;
      color = Palette.RED_CHEEP_CHEEP;
    } else {
      speed = 0.5f;
      color = Palette.GRAY_CHEEP_CHEEP;
    }

    oscillates = random.nextBoolean();
    if (oscillates) {
      angle = random.nextFloat() * (float)(2.0 * Math.PI);
      centerY = y;
    }

    supports = false;
  }

  @Override
  public void update() {

    fx -= speed;
    x = (int)fx;

    if (x + 32 < playingMode.cameraX) {
      remove = true;
      cheepCheepSpawner.killed(false);
    }

    if (oscillates) {
      angle += ANGLE_SPEED;
      fy = centerY + 32f * (float)Math.sin(angle);
      y = (int)fy;
    }
  }

  @Override
  public boolean blast(int px, int py, boolean left) {
    if (remove) {
      return false;
    }
    if (withinMargin(px, py)) {
      playingMode.others.add(
          new UpsideDownDeadCheepCheep(x, y, left ? -4 : 4, color));
      playingMode.addPoints(this, 200);
      remove = true;
      cheepCheepSpawner.killed(true);
      return true;
    }
    return false;
  }

  @Override
  public void starred(boolean left) {
    if (remove) {
      return;
    }
    shelled(left);
    playingMode.addPoints(this, 200);
  }

  @Override
  public void shelled(boolean left) {
    if (remove) {
      return;
    }
    playingMode.others.add(
        new UpsideDownDeadCheepCheep(x, y, left ? -4 : 4, color));
    remove = true;
    cheepCheepSpawner.killed(true);
  }

  @Override
  public boolean hurtsMario() {
    return !remove && intersects(playingMode.mario, true);
  }

  @Override
  public void render() {
    g.translate(x - playingMode.cameraX, y);
    g.setColor(color);
    g.drawOval(0, 0, 21, 31);
    g.drawLine(22, 15, 31, 4);
    g.drawLine(22, 16, 31, 27);
    g.drawLine(31, 4, 31, 27);
    g.translate(playingMode.cameraX - x, -y);
  }
}
