package mario;

import java.awt.*;

public class EndingMode implements IMode {

  public static final int STATE_CREDITS = 0;
  public static final int STATE_WORLD_OBJECTS = 1;
  public static final int STATE_THE_END = 2;

  public static final int CAST_CAST = 0;
  public static final int CAST_INIT = 1;
  public static final int CAST_ENTERING = 2;
  public static final int CAST_PAUSED = 3;
  public static final int CAST_EXITING = 4;

  public static final float CREDITS_VY = -0.5f;
  public static final int CAST_VX = -3;

  public static final String[] TITLES = {
    "MAIN DIRECTOR",
    "MAP DIRECTOR",
    "AREA DIRECTOR",
    "PROGRAM DIRECTOR",
    "MARIO AND SYSTEM PROGRAMMER",
    "OBJECT PROGRAMMER",
    "BACKGROUND PROGRAMMER",
    "MAP PROGRAMMER",
    "AREA DATA INPUT",
    "CHARACTER GRAPHIC DESIGNER",    
    "PRODUCER",
    "EXECUTIVE PRODUCER",
    "CODE GUY",
    "INSPIRED BY",
    "VERY SPECIAL THANKS TO",
  };
  public static final Color[] TITLE_COLORS = new Color[TITLES.length];

  public static final String[] CAST_NAMES = {
    "MARIO",
    "LUIGI",
    "PRINCESS TOADSTOOL", 
    "MUSHROOM RETAINER",       
    "GOOMBA",
    "GREEN KOOPA TROOPA",
    "PIRANHA PLANT",
    "RED KOOPA TROOPA",
    "RED KOOPA PARATROOPA",
    "FIREBAR",
    "GREEN KOOPA PARATROOPA",
    "BLOOBER",
    "SWIMMING CHEEP CHEEP",
    "FLYING CHEEP CHEEP",
    "PODOBOO",
    "HAMMER BROTHER",
    "LAKITU",
    "SPINY",
    "BUZZY BEETLE",
    "BULLET BILL",
    "BOWSER",
  };

  public static final int[] CAST_NAMES_X = new int[CAST_NAMES.length];
  public WorldObject[] worldObjects = new WorldObject[21];

  public Main main;
  public Graphics g;
  public float creditsFy = 480;
  public int creditsY;
  public int state = STATE_CREDITS;
  public int castIndex;
  public int castDelay = 2 * 60;
  public int castX = 512;
  public int castTargetX;
  public int castState = CAST_CAST;
  public int theEndDelay;
  public IInput input;

  public EndingMode() {

    main = Main.main;
    g = main.g;
    Main.playingMode = new PlayingMode(main.worlds[0][0], true, 0, false);
    Main.playingMode.mario = new Mario();
    input = new KeyboardInput();

    for(int i = 0; i < TITLE_COLORS.length; i++) {
      TITLE_COLORS[i] = Palette.CREDITS[i % 3];
    }

    Mario mario = new Mario();
    mario.big = true;
    mario.my2 = 63;
    mario.color = Palette.MARIO;
    worldObjects[0] = mario;

    Mario luigi = new Mario();
    luigi.big = true;
    luigi.my2 = 63;
    luigi.color = Palette.LUIGI;
    worldObjects[1] = luigi;

    Toad peach = new Toad(0, 0, false);
    peach.color = Palette.PEACH;
    worldObjects[2] = peach;

    Toad toad = new Toad(0, 0, false);
    toad.color = Palette.TOAD;
    worldObjects[3] = toad;

    worldObjects[4] = new Goomba(0, 0, false);

    worldObjects[5] = new KoopaTroopa(0, 0, false, KoopaTroopaTypes.GREEN);

    PiranhaPlant piranhaPlant = new PiranhaPlant(0, 0, false);
    piranhaPlant.clipY = piranhaPlant.y;
    piranhaPlant.state = PiranhaPlant.STATE_ABOVE;
    worldObjects[6] = piranhaPlant;

    worldObjects[7] = new KoopaTroopa(0, 0, false, KoopaTroopaTypes.RED);

    worldObjects[8] = new RedKoopaParatroopa(0, 0, false);

    Firebar firebar = new Firebar(0, 0, false, true, 6);
    firebar.angle = (float)(Math.PI * 0.25);
    firebar.update();
    worldObjects[9] = firebar;

    worldObjects[10] = new GreenKoopaParatroopa(0, 0, false);

    worldObjects[11] = new Bloober(0, 0, false);

    CheepCheep cheepCheep = new CheepCheep(new CheepCheepSpawner());
    cheepCheep.color = Palette.GRAY_CHEEP_CHEEP;
    cheepCheep.x = 0;
    cheepCheep.y = 0;
    worldObjects[12] = cheepCheep;

    CheepCheep cheepCheep2 = new CheepCheep(new CheepCheepSpawner());
    cheepCheep2.color = Palette.RED_CHEEP_CHEEP;
    cheepCheep2.x = 0;
    cheepCheep2.y = 0;
    worldObjects[13] = cheepCheep2;

    Podoboo podoboo = new Podoboo(0);
    podoboo.x = 0;
    podoboo.y = 0;
    worldObjects[14] = podoboo;

    HammerBrother hammerBrother = new HammerBrother(0, 0, false);
    worldObjects[15] = hammerBrother;

    Lakitu lakitu = new Lakitu();
    lakitu.x = 0;
    lakitu.y = 0;
    worldObjects[16] = lakitu;

    worldObjects[17] = new Spiny(0, 0, false, lakitu);

    worldObjects[18] = new KoopaTroopa(0, 0, false,
        KoopaTroopaTypes.BUZZY_BEETLE);

    worldObjects[19] = new BulletBill(0, 0, false, -1, false, 0, 0);

    worldObjects[20] = new Bowser(0, 0, false);

    for(int i = 0; i < CAST_NAMES.length; i++) {
      CAST_NAMES_X[i] = (512 - (CAST_NAMES[i].length() << 4)) >> 1;
    }
  }

  @Override
  public void init() {
  }

  @Override
  public void update() {
    switch(state) {
      case STATE_CREDITS:
        updateCredits();
        break;
      case STATE_WORLD_OBJECTS:
        updateWorldObjects();
        break;
      case STATE_THE_END:
        updateTheEnd();
        break;
    }
  }

  private void updateCredits() {
    creditsFy += CREDITS_VY;
    creditsY = (int)creditsFy;
    if (creditsY < -1909) {
      state = STATE_WORLD_OBJECTS;
    }
  }

  private void updateWorldObjects() {
    switch(castState) {
      case CAST_CAST:
        if (--castDelay == 0) {
          castState = CAST_INIT;
        }
        break;
      case CAST_INIT:
        castX = 512;
        castTargetX = (512 - worldObjects[castIndex].mx2) >> 1;
        castState = CAST_ENTERING;
        break;
      case CAST_ENTERING:
        castX += CAST_VX;
        if (castX <= castTargetX) {
          castX = castTargetX;
          castState = CAST_PAUSED;
          castDelay = 2 * 60;
        }
        break;
      case CAST_PAUSED:
        if (--castDelay == 0) {
          castState = CAST_EXITING;
        }
        break;
      case CAST_EXITING:
        castX += CAST_VX;
        if (castX <= -70) {
          castIndex++;
          if (castIndex == worldObjects.length) {
            state = STATE_THE_END;
          } else {
            castState = CAST_INIT;
          }
        }
        break;
    }
  }

  private void updateTheEnd() {
    if (++theEndDelay >= 10 * 64 && !input.isStart()) {
      main.finishedCredits();
    }
  }

  @Override
  public void render() {

    g.setColor(Color.BLACK);
    g.fillRect(0, 0, 512, 480);

    switch(state) {
      case STATE_CREDITS:
        renderCredits();
        break;
      case STATE_WORLD_OBJECTS:
        renderWorldObjects();
        break;
      case STATE_THE_END:
        renderTheEnd();
        break;
    }
  }

  private void renderCredits() {
    for(int i = 0, y = creditsY; i < TITLES.length; i++, y += 128) {
      if (y < 480 && y > -128) {
        g.setColor(TITLE_COLORS[i]);
        Text.drawString(32, y, TITLES[i]);
        g.setColor(Color.WHITE);
        if (i == TITLES.length - 2) {
          Text.drawString(64, y + 32, "THE BRILLIANT WORKS");
          Text.drawString(128, y + 64, "OF NINTENDO");
        } else if (i == TITLES.length - 1) {
          Text.drawString(64, y + 32, "SHIGERU MIYAMOTO");
          Text.drawString(128, y + 64, "FOR INVENTING MARIO");
        } else {
          Text.drawString(64, y + 32, "MICHAEL BIRKEN");
        }
      }
    }
  }

  private void renderWorldObjects() {
    if (castState == CAST_CAST) {
      g.setColor(Color.YELLOW);
      Text.drawString(224, 180, "CAST");
    } else {
      g.translate(castX, 144);
      worldObjects[castIndex].render();
      g.translate(-castX, -144);

      if (castState == CAST_PAUSED) {
        g.setColor(Color.WHITE);
        Text.drawString(CAST_NAMES_X[castIndex], 240, CAST_NAMES[castIndex]);
      }
    }
  }

  private void renderTheEnd() {
    g.setColor(Color.WHITE);
    Text.drawString(200, 232, "THE END");
  }
}
