package mario;

import java.awt.*;

public class Firework extends WorldObject {

  public static final int MAX_RADIUS = 48;
  public static final int RADIUS_SPEED = 16;
  public static final float STREAMER_SPEED = 0.75f;
  public static final int STREAMERS = 64;
  public static final int YELLOWS = 32;
  public static final int REDS = 32;
  public static final int BLACKS = 32;
  public static final int[] FLASH = { 0xFE, 0xFD, 0xFF };
  public static final int[] YELLOW = { 0xFF, 0xE3, 0x74 };
  public static final int[] RED = { 0xFF, 0x2D, 0x35 };
  public static final Color[] PALETTE = new Color[YELLOWS + REDS + BLACKS];
  public static final float STREAMER_INC = (float)(2.0 * Math.PI / STREAMERS);

  public int radius;

  public Firework(int x, int y) {
    this.fx = this.x = x;
    this.fy = this.y = y;
  }

  static {
    int[] color = new int[3];
    for(int i = 0; i < YELLOWS; i++) {
      double fraction = i / (double)YELLOWS;
      for(int j = 0; j < 3; j++) {
        color[j] = (int)(FLASH[j] + (YELLOW[j] - FLASH[j]) * fraction);
      }
      PALETTE[i] = new Color((color[0] << 16) | (color[1] << 8) | color[2]);
    }
    for(int i = 0; i < REDS; i++) {
      double fraction = i / (double)REDS;
      for(int j = 0; j < 3; j++) {
        color[j] = (int)(YELLOW[j] + (RED[j] - YELLOW[j]) * fraction);
      }
      PALETTE[i + YELLOWS]
          = new Color((color[0] << 16) | (color[1] << 8) | color[2]);
    }
    for(int i = 0; i < BLACKS; i++) {
      double fraction = i / (double)BLACKS;
      for(int j = 0; j < 3; j++) {
        color[j] = (int)(RED[j] - RED[j] * fraction);
      }
      PALETTE[i + YELLOWS + REDS]
          = new Color((color[0] << 16) | (color[1] << 8) | color[2]);
    }
  }

  public void update() {
    if (radius < MAX_RADIUS) {
      radius += RADIUS_SPEED;
    } else {
      remove = true;
      float angle = 0;
      for(int i = 0; i < STREAMERS; i++, angle += STREAMER_INC) {
        playingMode.others.add(new FireworkStreamer(x, y,
            (float)(STREAMER_SPEED * Math.cos(angle)),
            (float)(STREAMER_SPEED * Math.sin(angle))));
      }
    }
  }

  public void render() {
    g.setColor(PALETTE[0]);
    g.fillOval(x - playingMode.cameraX - radius, y - radius,
        radius << 1, radius << 1);
  }
}
