package mario;

public class FlyingCheepCheep extends WorldObject {

  public static final float GRAVITY = 0.1f;
  public static final int[] STEPS = new int[32];
  public static final float[] VELOCITIES = new float[32];

  static {
    float y = 96;
    float vy = 0;
    int index = -1;
    int count = 0;
    while(true) {
      vy += GRAVITY;
      y += vy;
      int index2 = ((int)y) >> 5;
      if (index2 > index) {
        if (index2 >= STEPS.length) {
          break;
        }
        index = index2;
        STEPS[index] = count;
        VELOCITIES[index] = vy;
      }
      count++;
    }
  }

  public FlyingCheepCheep() {
    fy = y = 480;
    vy = -VELOCITIES[15];

    float targetX = 0;

    switch(random.nextInt(5)) {
      case 0:
      case 1:
      case 2:
        if (playingMode.mario.facingLeft) {
          fx = x = playingMode.mario.x - 192;
          targetX = playingMode.mario.x - 288;
        } else {
          fx = x = playingMode.mario.x + 224;
          targetX = playingMode.mario.x + 320;
        }
        break;
      case 3:
        if (playingMode.mario.facingLeft) {
          fx = x = playingMode.cameraX + 480;
          targetX = playingMode.cameraX;
        } else {
          fx = x = playingMode.cameraX;
          targetX = playingMode.cameraX + 480;
        }
        break;
      case 4:
        if (playingMode.mario.facingLeft) {
          fx = x = playingMode.mario.x + 128 + random.nextInt(128);
        } else {
          fx = x = playingMode.mario.x - 96 - random.nextInt(128);
        }
        targetX = playingMode.mario.x;
        break;
    }

    int marioIndex = (playingMode.mario.y + 16) >> 5;
    if (marioIndex >= STEPS.length) {
      marioIndex = STEPS.length - 1;
    }
    vx = (targetX - fx) / (STEPS[15] + STEPS[marioIndex]);
  }

  @Override
  public void update() {
    vy += GRAVITY;
    
    if (!moveY(vy)) {
      return;
    }

    fx += vx;
    x = (int)fx;

    if (vy > 0 && y >= 480) {
      remove = true;
      playingMode.flyingCheepCheepDied();
    }
  }

  @Override
  public void crush() {
    remove = true;
    playingMode.others.add(
          new UpsideDownDeadCheepCheep(x, y,
              playingMode.mario.x < x ? 4 : -4,
              Palette.RED_CHEEP_CHEEP));
    playingMode.addStompPoints(this);
    playingMode.flyingCheepCheepDied();
  }

  @Override
  public boolean blast(int px, int py, boolean left) {
    if (remove) {
      return false;
    }
    if (withinMargin(px, py)) {
      playingMode.others.add(
          new UpsideDownDeadCheepCheep(x, y, left ? -4 : 4,
              Palette.RED_CHEEP_CHEEP));
      playingMode.addPoints(this, 200);
      remove = true;
      playingMode.flyingCheepCheepDied();
      return true;
    }
    return false;
  }

  @Override
  public void starred(boolean left) {
    if (remove) {
      return;
    }
    shelled(left);
    playingMode.addPoints(this, 200);
  }

  @Override
  public void shelled(boolean left) {
    if (remove) {
      return;
    }
    playingMode.others.add(
        new UpsideDownDeadCheepCheep(x, y, left ? -4 : 4,
            Palette.RED_CHEEP_CHEEP));
    remove = true;
    playingMode.flyingCheepCheepDied();
  }

  @Override
  public boolean hurtsMario() {
    return !remove && playingMode.mario.intersects(this, true);
  }

  @Override
  public void render() {
    g.translate(x - playingMode.cameraX, y);
    g.setColor(Palette.RED_CHEEP_CHEEP);
    if (vx > 0) {
      g.drawOval(10, 0, 21, 31);
      g.drawLine(9, 15, 0, 4);
      g.drawLine(9, 16, 0, 27);
      g.drawLine(0, 4, 0, 27);
    } else {
      g.drawOval(0, 0, 21, 31);
      g.drawLine(22, 15, 31, 4);
      g.drawLine(22, 16, 31, 27);
      g.drawLine(31, 4, 31, 27);
    }
    g.translate(playingMode.cameraX - x, -y);
  }

}
