package mario;

public class Goomba extends WorldObject {

  public Goomba(int x, int y, boolean mapCoordinates) {
    bx1 = 6;
    by1 = 0;
    bx2 = 17;
    by2 = 31;

    if (mapCoordinates) {
      x <<= 5;
      y <<= 5;
    }
    fx = this.x = x;
    fy = this.y = y;
    vx = -1;
  }

  public void update() {

    for(int i = playingMode.platforms.size() - 1; i >= 0; i--) {
      AbstractPlatform platform = playingMode.platforms.get(i);
      if (platform.deadly && platform.intersects(this)) {
        remove = true;
        playingMode.others.add(new UpsideDownDeadGoomba(x, y,
            platform.x < x ? 4 : -4));
        playingMode.addPoints(this, 100);
        return;
      }
    }

    applyGravity(true);
    if (vx < 0) {
      if (playingMode.withEnemyMargin(this, (int)(fx + vx), y + 15)
          != null || !translateX(vx, false)) {
        vx = Math.abs(vx);
      }
    } else {
      if (playingMode.withEnemyMargin(this, 31 + (int)(fx + vx), y + 15)
          != null || !translateX(vx, false)) {
        vx = -Math.abs(vx);
      }
    }

    if (x < playingMode.cameraX - 512 || y > 480) {
      remove = true;
      return;
    }
  }

  @Override
  public boolean blast(int px, int py, boolean left) {
    if (withinMargin(px, py)) {
      playingMode.others.add(new UpsideDownDeadGoomba(x, y, left ? -4 : 4));
      playingMode.addPoints(this, 100);
      remove = true;
      return true;
    }
    return false;
  }

  @Override
  public void starred(boolean left) {
    shelled(left);
    playingMode.addPoints(this, 100);    
  }

  @Override
  public void shelled(boolean left) {
    playingMode.others.add(new UpsideDownDeadGoomba(x, y, left ? -4 : 4));
    remove = true;
  }

  @Override
  public void crush() {
    remove = true;
    playingMode.others.add(new CrushedDeadGoomba(x, y));
    playingMode.addStompPoints(this);
  }

  @Override
  public boolean hurtsMario() {
    return !remove && intersects(playingMode.mario, true);
  }

  public void render() {
    g.translate(x - playingMode.cameraX, y);
    g.setColor(Palette.GOOMBA_BOTTOM);
    g.drawRect(8, 21, 15, 10);
    g.setColor(Palette.GOOMBA_TOP);
    g.drawArc(0, 0, 31, 25, 0, 225);
    g.drawArc(0, 0, 31, 25, 0, -45);
    g.drawLine(5, 21, 26, 21);
    g.translate(playingMode.cameraX - x, -y);
  }
  
}
