package mario;

import java.awt.*;

public class GreenKoopaParatroopa extends WorldObject {

  public GreenKoopaParatroopa(
      int x, int y, boolean mapCoordinates) {

    if (mapCoordinates) {
      x <<= 5;
      y <<= 5;
    }

    this.fx = this.x = x;
    this.fy = this.y = y;
    vx = -1;
    G = 0.15f;
  }

  public void update() {

    if (vx < 0) {
      if (playingMode.withEnemyMargin(this, (int)(fx + vx), y + 15)
          != null || !translateX(vx, false)) {
        vx = Math.abs(vx);
      }
    } else {
      if (playingMode.withEnemyMargin(this, 31 + (int)(fx + vx), y + 15)
          != null || !translateX(vx, false)) {
        vx = -Math.abs(vx);
      }
    }

    if (x < playingMode.cameraX - 512 || y > 480) {
      remove = true;
      return;
    }

    if (!playingMode.mario.supportedBy(this) && applyGravity(true)) {
      vy = -4.4f;
    }
  }

  @Override
  public boolean isSolid(int rx1, int ry1, int rx2, int ry2, int direction) {
    for(int i = playingMode.platforms.size() - 1; i >= 0; i--) {
      AbstractPlatform platform = playingMode.platforms.get(i);
      if (platform.intersects(rx1, ry1, rx2, ry2, false)) {
        return true;
      }
    }
    if (direction == DirectionTypes.UP) {
      if (playingMode.mario.intersects(rx1, ry1, rx2, ry2, false)) {
        return true;
      }
    }
    return false;
  }

  @Override
  public boolean blast(int px, int py, boolean left) {
    if (withinMargin(px, py)) {
      playingMode.others.add(new UpsideDownDeadKoopaTroopa(
          x, y, left ? -4 : 4, KoopaTroopaTypes.GREEN));
      playingMode.addPoints(this, 200);
      remove = true;
      return true;
    }
    return false;
  }

  @Override
  public void starred(boolean left) {
    shelled(left);
    playingMode.addPoints(this, 200);
  }

  @Override
  public void shelled(boolean left) {
    playingMode.others.add(new UpsideDownDeadKoopaTroopa(
        x, y, left ? -4 : 4, KoopaTroopaTypes.GREEN));
    playingMode.addPoints(this, 200);
    remove = true;
  }

  @Override
  public void crush() {
    remove = true;
    playingMode.enemies.add(
        new KoopaTroopa(x, y, false, KoopaTroopaTypes.GREEN));
    playingMode.addStompPoints(this);
  }

  @Override
  public boolean hurtsMario() {
    return !remove && intersects(playingMode.mario, true);
  }

  public void render() {
    g.translate(x - playingMode.cameraX, y);
    g.setColor(Palette.KOOPA_TROOPA_LEG);
    g.drawLine(3, 17, 3, 31);
    g.drawLine(28, 17, 28, 31);
    g.drawLine(3, 31, 12, 31);
    g.drawLine(12, 27, 12, 31);
    g.drawLine(19, 27, 19, 31);
    g.drawLine(19, 31, 28, 31);
    g.setColor(Palette.GREEN_KOOPA_TROOPA);
    g.drawArc(2, 0, 27, 30, 0, 180);
    g.setColor(Color.WHITE);
    g.drawLine(0, 16, 5, 16);
    g.drawLine(26, 16, 31, 16);
    g.drawArc(5, 7, 21, 20, 0, -180);
    g.drawLine(0, 0, 8, 8);
    g.drawLine(31, 0, 23, 8);
    g.translate(playingMode.cameraX - x, -y);
  }
}
