package mario;

public class HammerBrother extends WorldObject {

  public static final float WALK_SPEED = 0.025f;

  public float walkAngle;
  public float centerX;
  public int jumpDelay = 3 * 60;
  public boolean jumping;
  public boolean climbing;
  public int jumpY;
  public int throwDelay;

  public HammerBrother(int x, int y, boolean mapCoordinates) {

    G = 0.45f;

    if (mapCoordinates) {
      x <<= 5;
      y <<= 5;
    }

    this.centerX = this.fx = this.x = x;
    this.fy = this.y = y;

    mx1 = bx1 = 0;
    my1 = by1 = 16;
    mx2 = bx2 = 31;
    my2 = by2 = 63;

    fx = centerX + 8f * (float)Math.sin(walkAngle);
    x = (int)fx;
  }

  @Override
  public void update() {

    if (x + 128 < playingMode.cameraX) {
      remove = true;
      return;
    }

    for(int i = playingMode.platforms.size() - 1; i >= 0; i--) {
      AbstractPlatform platform = playingMode.platforms.get(i);
      if (platform.deadly && platform.intersects(this)) {
        remove = true;
        playingMode.others.add(new UpsideDownDeadHammerBrother(x, y,
            platform.x < x ? 4 : -4));
        playingMode.addPoints(this, 1000);
        return;
      }
    }

    walkAngle += WALK_SPEED;
    fx = centerX + 16f * (float)Math.sin(walkAngle);
    x = (int)fx;

    if (jumping) {
      vy += G;
      if (!moveY(vy)) {
        jumping = false;
        jumpDelay = (random.nextInt(2) + 4) * 60;
      }
    } else {
      applyGravity(true);
      if (throwDelay > 0) {
        throwDelay--;
      } else {
        throwDelay = 45 + random.nextInt(60);
        playingMode.items.add(new Hammer(x, y,
            playingMode.mario.x < x ? -2f : 2f, -6f));
      }
      if (jumpDelay > 0) {
        jumpDelay--;
      } else {                
        jumping = true;
        climbing = random.nextBoolean();
        if (y >= 320) {
          climbing = true;
        } else if (y <= 96) {
          climbing = false;
        }
        jumpY = y + 64;
        if (climbing) {
          vy = -12f;
        } else {
          vy = -4f;
        }
      }
    }
  }

  @Override
  public boolean moveY(float dy) {

    if (playingMode.mario.supportedBy(this)) {
      return false;
    }

    float fy2 = fy + dy;
    int y2 = (int)fy2;

    if (y > y2) {
      for(y--; y >= y2; y--) {
        if (playingMode.mario.supportedBy(this)) {
          return false;
        }
      }
    } else {
      if (climbing || y > jumpY) {
        for(y++; y <= y2; y++) {
          int mapY = (y + 64) >> 5;
          if ((y & 31) == 0
              && (isMapSolid(x >> 5, mapY, DirectionTypes.NONE)
                  || isMapSolid((x + 31) >> 5, mapY, DirectionTypes.NONE))) {
            return false;
          }
        }
      }
    }

    y = y2;
    fy = fy2;
    return true;
  }

  @Override
  public boolean blast(int px, int py, boolean left) {
    if (withinMargin(px, py)) {
      playingMode.others.add(
          new UpsideDownDeadHammerBrother(x, y, left ? -4 : 4));
      playingMode.addPoints(this, 1000);
      remove = true;
      return true;
    }
    return false;
  }

  @Override
  public void starred(boolean left) {
    shelled(left);
    playingMode.addPoints(this, 1000);
  }

  @Override
  public void shelled(boolean left) {
    playingMode.others.add(
        new UpsideDownDeadHammerBrother(x, y, left ? -4 : 4));
    remove = true;
  }

  @Override
  public void crush() {
    remove = true;
    playingMode.addStompPoints(this);
    playingMode.others.add(new UpsideDownDeadHammerBrother(
        x, y, playingMode.mario.x < x ? 4 : -4));
  }

  @Override
  public boolean hurtsMario() {
    return !remove && intersects(playingMode.mario, true);
  }

  @Override
  public void render() {
    g.setColor(Palette.HAMMER_BROTHER);
    g.translate(x - playingMode.cameraX, y + 16);
    g.drawOval(2, 0, 27, 17);
    g.drawLine(15, 18, 15, 33);
    g.drawLine(16, 18, 16, 33);
    g.drawLine(15, 32, 0, 47);
    g.drawLine(16, 32, 31, 47);
    g.drawLine(15, 20, 0, 30);
    g.drawLine(16, 20, 31, 30);
    g.translate(playingMode.cameraX - x, -(y + 16));
  }
}
