package mario;

import java.awt.*;

public class HoveringKoopaParatroopa extends WorldObject {

  public static final float Y_RADIUS = 10;
  public static final float X_RADIUS = 3 * 32;
  
  public static final float X_SPEED = 0.015f;
  public static final float Y_SPEED = 5 * X_SPEED;

  public float yAngle;
  public float xAngle;
  public float xCenter;
  public float yCenter;
  public float xLast;
  public float yLast;
  public boolean firstTime = true;

  public HoveringKoopaParatroopa(
      int x, int y, boolean mapCoordinates) {

    if (mapCoordinates) {
      x <<= 5;
      y <<= 5;
    }

    this.xCenter = x;
    this.yCenter = y;

    this.fx = xCenter - X_RADIUS * (float)Math.sin(xAngle);
    this.fy = yCenter + Y_RADIUS * (float)Math.cos(yAngle);
    this.x = (int)this.fx;
    this.y = (int)this.fy;
  }

  public void update() {

    yAngle += Y_SPEED;
    xAngle += X_SPEED;

    translateY(yCenter + Y_RADIUS * (float)Math.cos(yAngle) - fy, true);
    translateX(xCenter - X_RADIUS * (float)Math.sin(xAngle) - fx, true);

    if (x + 32 + X_RADIUS < playingMode.cameraX) {
      remove = true;
      return;
    }
  }

  @Override
  public boolean isSolid(int rx1, int ry1, int rx2, int ry2, int direction) {
    for(int i = playingMode.platforms.size() - 1; i >= 0; i--) {
      AbstractPlatform platform = playingMode.platforms.get(i);
      if (platform.intersects(rx1, ry1, rx2, ry2, false)) {
        return true;
      }
    }
    if (direction == DirectionTypes.UP) {
      if (playingMode.mario.intersects(rx1, ry1, rx2, ry2, false)) {
        return true;
      }
    }
    return false;
  }

  @Override
  public boolean blast(int px, int py, boolean left) {
    if (withinMargin(px, py)) {
      playingMode.others.add(new UpsideDownDeadKoopaTroopa(
          x, y, left ? -4 : 4, KoopaTroopaTypes.GREEN));
      playingMode.addPoints(this, 200);
      remove = true;
      return true;
    }
    return false;
  }

  @Override
  public void starred(boolean left) {
    shelled(left);
    playingMode.addPoints(this, 200);
  }

  @Override
  public void shelled(boolean left) {
    playingMode.others.add(new UpsideDownDeadKoopaTroopa(
        x, y, left ? -4 : 4, KoopaTroopaTypes.GREEN));
    playingMode.addPoints(this, 200);
    remove = true;
  }

  @Override
  public void crush() {
    remove = true;
    playingMode.enemies.add(
        new KoopaTroopa(x, y, false, KoopaTroopaTypes.GREEN));
    playingMode.addStompPoints(this);
  }

  @Override
  public boolean hurtsMario() {
    return !remove && intersects(playingMode.mario, true);
  }

  public void render() {
    g.translate(x - playingMode.cameraX, y);
    g.setColor(Palette.KOOPA_TROOPA_LEG);
    g.drawLine(3, 17, 3, 31);
    g.drawLine(28, 17, 28, 31);
    g.drawLine(3, 31, 12, 31);
    g.drawLine(12, 27, 12, 31);
    g.drawLine(19, 27, 19, 31);
    g.drawLine(19, 31, 28, 31);
    g.setColor(Palette.GREEN_KOOPA_TROOPA);
    g.drawArc(2, 0, 27, 30, 0, 180);
    g.setColor(Color.WHITE);
    g.drawLine(0, 16, 5, 16);
    g.drawLine(26, 16, 31, 16);
    g.drawArc(5, 7, 21, 20, 0, -180);
    g.drawLine(0, 0, 8, 8);
    g.drawLine(31, 0, 23, 8);
    g.translate(playingMode.cameraX - x, -y);
  }
}
