package mario;

import java.io.*;

public class InputPlayer implements IInput {

  public static final int RECORDING_LENGTH = InputRecorder.SECONDS * 60;

  public byte[][] recordings;
  public int fileIndex = -1;
  public int recordingIndex = -1;
  public int length;
  public boolean isUp;
  public boolean isDown;
  public boolean isLeft;
  public boolean isRight;
  public boolean isRun;
  public boolean isJump;
  public IInput input;

  public InputPlayer(String... fileNames) throws Throwable {
    recordings = new byte[fileNames.length][RECORDING_LENGTH];
    for(int i = 0; i < fileNames.length; i++) {
      ClassLoader classLoader = Main.class.getClassLoader();
      DataInputStream in = new DataInputStream(
          classLoader.getResourceAsStream("demos/" + fileNames[i] + ".dat"));
      in.readFully(recordings[i]);
      in.close();
    }
  }

  public void resetInput() {
    input = new KeyboardInput();
  }

  public int reset() {
    if (++fileIndex == recordings.length) {
      fileIndex = 0;
    }
    recordingIndex = -1;
    return fileIndex;
  }

  public boolean isUp() {
    return isUp;
  }

  public boolean isDown() {
    return isDown;
  }

  public boolean isLeft() {
    return isLeft;
  }

  public boolean isRight() {
    return isRight;
  }

  public boolean isJump() {
    return isJump;
  }

  public boolean isRun() {
    return isRun;
  }

  public boolean isPause() {
    return false;
  }

  public boolean isStart() {
    if (input == null) {
      return false;
    }
    return input.isStart();
  }

  public boolean hasNext() {
    if (++recordingIndex < RECORDING_LENGTH) {
      int value = recordings[fileIndex][recordingIndex];
      isUp    = (value &  1) != 0;
      isDown  = (value &  2) != 0;
      isLeft  = (value &  4) != 0;
      isRight = (value &  8) != 0;
      isRun   = (value & 16) != 0;
      isJump  = (value & 32) != 0;
      return true;
    } else {
      return false;
    }
  }

}
