package mario;

public class JumpingBoard extends AbstractPlatform {

  public static final float ANGLE_SPEED = 0.2f;
  public static final float CYCLE = (float)Math.PI;
  public static final float JUMP_ANGLE = (float)(0.1 * Math.PI);

  public float angle;
  public int centerY;

  public JumpingBoard(int x, int y, boolean mapCoordinates) {

    if (mapCoordinates) {
      x <<= 5;
      y <<= 5;
    }

    this.fx = this.x = x;
    this.fy = this.y = y;

    centerY = y;

    mx1 = bx1 = 0;
    my1 = by1 = 0;
    mx2 = bx2 = 31;
    my2 = by2 = 63;
  }

  @Override
  public void move() {
    if (supportsMario || angle > 0) {
      angle += ANGLE_SPEED;
      fy = centerY + 40f * (float)Math.sin(angle);
      y = (int)fy;
      my2 = by2 = 63 - (y - centerY);
      if (angle >= CYCLE) {
        angle = 0;
        fy = y = centerY;
        mx1 = bx1 = 0;
        my1 = by1 = 0;
        mx2 = bx2 = 31;
        my2 = by2 = 63;
        if (supportsMario) {
          Mario mario = playingMode.mario;
          mario.jumpState = Mario.JUMP_STATE_JUMP;
          mario.vy = -9f;
        }
      } else if (angle >= JUMP_ANGLE && supportsMario) {
        playingMode.mario.onJumpingBoard = true;
      }
    } else {
      angle = 0;
      fy = y = centerY;
      mx1 = bx1 = 0;
      my1 = by1 = 0;
      mx2 = bx2 = 31;
      my2 = by2 = 63;
    }
  }

  @Override
  public void render() {
    g.translate(x - playingMode.cameraX, centerY);
    g.setColor(Palette.JUMPING_BOARD_SPRING);
    int halfHeight = ((46 + (y - centerY)) >> 1) + 1;
    g.drawLine(10, y - centerY + 4, 4, halfHeight);
    g.drawLine(4, halfHeight, 10, 45);
    g.drawLine(21, y - centerY + 4, 27, halfHeight);
    g.drawLine(27, halfHeight, 21, 45);
    g.setColor(Palette.STONES[playingMode.stoneIndex]);
    g.drawRect(0, 50, 31, 13);
    g.setColor(Palette.JUMPING_BOARD_TOP);
    g.drawRect(0, y - centerY, 31, 3);
    g.drawRect(0, 46, 31, 3);
    g.translate(playingMode.cameraX - x, -centerY);
  }

}
