package mario;

import java.awt.*;

public class KoopaTroopa extends WorldObject {

  public int type;
  public Color color;

  public KoopaTroopa(int x, int y, boolean mapCoordinates, int type) {
    if (mapCoordinates) {
      x <<= 5;
      y <<= 5;
    }
    this.fx = this.x = x;
    this.fy = this.y = y;
    vx = -1;
    this.type = type;
    switch(type) {
      case KoopaTroopaTypes.GREEN:
        color = Palette.GREEN_KOOPA_TROOPA;
        break;
      case KoopaTroopaTypes.RED:
        color = Palette.RED_KOOPA_TROOPA;
        break;
      case KoopaTroopaTypes.BUZZY_BEETLE:
        color = Palette.BUZZY_BEETLE;
        break;
    }
  }

  public void update() {

    for(int i = playingMode.platforms.size() - 1; i >= 0; i--) {
      AbstractPlatform platform = playingMode.platforms.get(i);
      if (platform.deadly && platform.intersects(this)) {
        remove = true;
        playingMode.shells.add(new StaticKoopaShell(x, y,
            platform.x < x ? 2 : -2, -9, type));
        return;
      }
    }

    applyGravity(true);

    if (vx < 0) {
      if ((type == KoopaTroopaTypes.RED
              && !isPointSolid(x - 1, y + 32, DirectionTypes.DOWN))
          || playingMode.withEnemyMargin(this, (int)(fx + vx), y + 15)
              != null || !translateX(vx, false)) {
        vx = Math.abs(vx);
      }
    } else {
      if ((type == KoopaTroopaTypes.RED
              && !isPointSolid(x + 32, y + 32, DirectionTypes.DOWN))
          || playingMode.withEnemyMargin(this, 31 + (int)(fx + vx), y + 15)
              != null || !translateX(vx, false)) {
        vx = -Math.abs(vx);
      }
    }

    if (x < playingMode.cameraX - 512 || y > 480) {
      remove = true;
    }
  }

  @Override
  public boolean blast(int px, int py, boolean left) {
    if (withinMargin(px, py)) {
      if (type == KoopaTroopaTypes.BUZZY_BEETLE) {
        return true;
      }
      playingMode.others.add(
          new UpsideDownDeadKoopaTroopa(x, y, left ? -4 : 4, type));
      playingMode.addPoints(this, 200);
      remove = true;
      return true;
    }
    return false;
  }

  @Override
  public void starred(boolean left) {
    shelled(left);
    playingMode.addPoints(this, 200);
  }

  @Override
  public void shelled(boolean left) {
    playingMode.others.add(
        new UpsideDownDeadKoopaTroopa(x, y, left ? -4 : 4, type));
    playingMode.addPoints(this, 200);
    remove = true;
  }

  @Override
  public void crush() {
    remove = true;
    playingMode.shells.add(new StaticKoopaShell(x, y, 0, 0, type));
    playingMode.addStompPoints(this);
  }

  @Override
  public boolean hurtsMario() {
    return !remove && intersects(playingMode.mario, true);
  }

  public void render() {
    g.translate(x - playingMode.cameraX, y);
    g.setColor(Palette.KOOPA_TROOPA_LEG);
    g.drawLine(3, 17, 3, 31);
    g.drawLine(28, 17, 28, 31);
    g.drawLine(3, 31, 12, 31);
    g.drawLine(12, 27, 12, 31);
    g.drawLine(19, 27, 19, 31);
    g.drawLine(19, 31, 28, 31);
    g.setColor(color);
    g.drawArc(2, 0, 27, 30, 0, 180);
    g.setColor(Color.WHITE);
    g.drawLine(0, 16, 5, 16);
    g.drawLine(26, 16, 31, 16);
    g.drawArc(5, 7, 21, 20, 0, -180);
    g.translate(playingMode.cameraX - x, -y); 
  }

}
