package mario;

public class Lakitu extends WorldObject {
    
  public static final float ANGLE_SPEED = 0.015f;
  public static final float RADIUS = 4 * 32;
  public static final float SPEED = RADIUS * (float)Math.sin(ANGLE_SPEED);

  public boolean entering = true;
  public boolean died;
  public boolean sleeping;
  public boolean exiting;
  public float angle;
  public int sleepDelay;
  public int availableSpinies = 3;
  public int dropDelay;

  public Lakitu() {
    this.fx = this.x = playingMode.cameraX + 512;
    this.fy = this.y = 96;
  }

  @Override
  public void update() {
    if (exiting) {
      if (sleeping || x > playingMode.cameraX + 512
          || x + 32 < playingMode.cameraX || y > 480) {
        remove = true;
        return;
      } else if (died) {
        dying();
      } else {
        fx -= SPEED;
        x = (int)fx;
      } 
    } else if (sleeping) {
      if (sleepDelay > 0) {
        sleepDelay--;
      } else {
        died = false;
        sleeping = false;
        entering = true;
        supports = true;
        fx = x = playingMode.cameraX + 512;
        fy = y = 96;        
        angle = 0;
        dropDelay = 0;
      }
    } else if (died) {
      dying();
    } else if (entering) {
      if (fx > playingMode.cameraX + 240) {
        fx -= SPEED;
        x = (int)fx;
      } else {
        entering = false;
        angle = 0;
      }
      dropping();
    } else {
      angle += ANGLE_SPEED;
      fx = playingMode.cameraX + 240 - RADIUS * (float)Math.sin(angle);
      x = (int)fx;
      dropping();
    }
  }

  public void exit() {
    exiting = true;
  }

  private void dying() {
    vy += G;
    fy += vy;
    y = (int)fy;
    fx += vx;
    x = (int)fx;
    if (y > 480) {
      sleeping = true;
      sleepDelay = 15 * 60;
    }
  }

  private void dropping() {
    if (++dropDelay >= 2 * 60) {
      if (availableSpinies > 0) {
        dropDelay = 0;
        availableSpinies--;
        playingMode.enemies.add(new Spiny(x, y - 32, false, this));
      }
    }
  }

  @Override
  public boolean blast(int px, int py, boolean left) {
    if (died) {
      return false;
    }
    if (withinMargin(px, py)) {      
      playingMode.addPoints(this, 200);
      die(left ? -4 : 4);
      return true;
    }
    return false;
  }

  @Override
  public void starred(boolean left) {
    if (died) {
      return;
    }
    shelled(left);
    playingMode.addPoints(this, 200);
  }

  @Override
  public void shelled(boolean left) {
    if (died) {
      return;
    }
    die(left ? -4 : 4);
    died = true;
  }

  @Override
  public void crush() {
    if (died) {
      return;
    }
    die(0);
    playingMode.addStompPoints(this);
  }

  @Override
  public boolean hurtsMario() {
    return !died && intersects(playingMode.mario, true);
  }

  public void die(float vx) {
    this.vx = vx;
    this.vy = -4f;
    died = true;
    supports = false;
  }

  public void spinyDied() {
    availableSpinies++;
  }

  @Override
  public void render() {
    g.setColor(Palette.LAKITU);
    g.translate(x - playingMode.cameraX, y);
    g.drawOval(0, 0, 31, 31);
    g.drawOval(10, 10, 11, 11);
    g.drawRect(0, 0, 31, 31);
    g.translate(playingMode.cameraX - x, -y);
  }

}
