/*
 * Worst Mario Game Ever
 * Copyright (C) 2010 meatfighter.com
 *
 * This file is part of Worst Mario Game Ever
 *
 * Worst Mario Game Ever is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Worst Mario Game Ever is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package mario;

import java.applet.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

public class Main extends Applet {

  public static Main main;
  public static PlayingMode playingMode;

  private static final int FRAMES_PER_SECOND = 60;
  private static final long FRAME_PERIOD = 1000000000L / FRAMES_PER_SECOND;
  private static final int MAX_UPDATES = 16;
  
  public final boolean[] K = new boolean[65535];

  public Graphics g;
  public Segment[][] worlds = new Segment[8][4];
  public Segment[] bonusAreas = new Segment[14];
  public Random random = new Random();
  public InputPlayer inputPlayer;
  public boolean luigiMode;
  public boolean hardMode;
  public boolean hardMode2;
  public int lastKeyPressed;
  public int world;
  public int subworld;
  public int lives;
  public int coins;
  public int score;
  public boolean passedHalfwayPoint;

  public int upKeyCode = KeyEvent.VK_UP;
  public int downKeyCode = KeyEvent.VK_DOWN;
  public int leftKeyCode = KeyEvent.VK_LEFT;
  public int rightKeyCode = KeyEvent.VK_RIGHT;
  public int runKeyCode = KeyEvent.VK_D;
  public int jumpKeyCode = KeyEvent.VK_F;
  public int pauseKeyCode = KeyEvent.VK_P;
  public int startKeyCode = KeyEvent.VK_ENTER;

  private javax.swing.Timer focusTimer;
  private Image image;  
  private Graphics appletGraphics;
  private IMode gameMode;
  private IMode nextGameMode;

  @Override
  public void init() {
    try {
      loadSegments();
      inputPlayer = new InputPlayer("1-1", "1-2", "1-3");
    } catch(Throwable t) {
      t.printStackTrace();
      return;
    }
    startGameLoop();
  }

  private void executeGameLoop() throws Throwable {
    long nextFrameStart = System.nanoTime();
    while(true) {
      if (nextGameMode != null) {
        gameMode = nextGameMode;
        gameMode.init();
        nextGameMode = null;
        nextFrameStart = System.nanoTime();
      }
      int updates = 0;
      do {
        gameMode.update();
        nextFrameStart += FRAME_PERIOD;
        if (++updates >= MAX_UPDATES) {
          nextFrameStart = System.nanoTime();
          break;
        }
      } while(nextFrameStart < System.nanoTime());
      gameMode.render();
      appletGraphics.drawImage(image, 0, 0, null);
      
      long remaining = nextFrameStart - System.nanoTime();
      if (remaining > 0) {
        Thread.sleep(remaining / 1000000);
      }
    }
  }

  @Override
  public void paint(Graphics g) {
    if (image != null) {
      g.drawImage(image, 0, 0, null);
    }
  }

  private void startGameLoop() {
    EventQueue.invokeLater(new Runnable() {
      public void run() {
        focusTimer = new javax.swing.Timer(100, new ActionListener() {
          public void actionPerformed(ActionEvent e) {
            if (Main.this.isVisible()) {
              if (Main.this.hasFocus()) {
                addKeyListener(new KeyListener() {
                  public void keyTyped(KeyEvent e) {
                  }
                  public void keyPressed(KeyEvent e) {
                    lastKeyPressed = e.getKeyCode();
                    K[lastKeyPressed] = true;
                  }
                  public void keyReleased(KeyEvent e) {
                    K[e.getKeyCode()] = false;
                  }
                });
                focusTimer.stop();
                synchronized(Main.this) {
                  main = Main.this;
                  image = createImage(512, 480);
                  g = image.getGraphics();
                  appletGraphics = getGraphics(); 
                  setGameMode(new TitleScreenMode(false));
                }
                new Thread() {
                  @Override
                  public void run() {
                    try {
                      executeGameLoop();
                    } catch(Throwable t) {
                      t.printStackTrace();
                    }
                  }
                }.start();
              } else {
                Main.this.requestFocus();
              }
            }
          }
        });
        focusTimer.start();
      }
    });
  }

  public void setGameMode(IMode gameMode) {
    this.nextGameMode = gameMode;
  }

  private void loadSegments() throws Throwable {
    worlds[0][0] = loadSegment("1-1", 1, 1, false, 0);
    worlds[0][1] = loadSegment("1-2", 1, 2, false, 0);
    worlds[0][2] = loadSegment("1-3", 1, 3, false, 0);
    worlds[0][3] = loadSegment("1-4", 1, 4, false, 0);
    worlds[1][0] = loadSegment("2-1", 2, 1, false, 0);
    worlds[1][1] = loadSegment("2-2", 2, 2, false, 0);
    worlds[1][2] = loadSegment("2-3", 2, 3, false, 0);
    worlds[1][3] = loadSegment("2-4", 2, 4, false, 0);
    worlds[2][0] = loadSegment("3-1", 3, 1, false, 0);
    worlds[2][1] = loadSegment("3-2", 3, 2, false, 0);
    worlds[2][2] = loadSegment("3-3", 3, 3, false, 0);
    worlds[2][3] = loadSegment("3-4", 3, 4, false, 0);
    worlds[3][0] = loadSegment("4-1", 4, 1, false, 0);
    worlds[3][1] = loadSegment("4-2", 4, 2, false, 0);
    worlds[3][2] = loadSegment("4-3", 4, 3, false, 0);
    worlds[3][3] = loadSegment("4-4", 4, 4, false, 0);
    worlds[4][0] = loadSegment("5-1", 5, 1, false, 0);
    worlds[4][1] = loadSegment("5-2", 5, 2, false, 0);
    worlds[4][2] = loadSegment("5-3", 5, 3, false, 0);
    worlds[4][3] = loadSegment("5-4", 5, 4, false, 0);
    worlds[5][0] = loadSegment("6-1", 6, 1, false, 0);
    worlds[5][1] = loadSegment("6-2", 6, 2, false, 0);
    worlds[5][2] = loadSegment("6-3", 6, 3, false, 0);
    worlds[5][3] = loadSegment("6-4", 6, 4, false, 0);
    worlds[6][0] = loadSegment("7-1", 7, 1, false, 0);
    worlds[6][1] = loadSegment("7-2", 7, 2, false, 0);
    worlds[6][2] = loadSegment("7-3", 7, 3, false, 0);
    worlds[6][3] = loadSegment("7-4", 7, 4, false, 0);
    worlds[7][0] = loadSegment("8-1", 8, 1, false, 0);
    worlds[7][1] = loadSegment("8-2", 8, 2, false, 0);
    worlds[7][2] = loadSegment("8-3", 8, 3, false, 0);
    worlds[7][3] = loadSegment("8-4", 8, 4, false, 0);
    bonusAreas[0] = loadSegment("a", 1, 1, true, 0);
    bonusAreas[1] = loadSegment("b", 1, 1, true, 1);
    bonusAreas[2] = loadSegment("c", 1, 1, true, 2);
    bonusAreas[3] = loadSegment("d", 1, 1, true, 3);
    bonusAreas[4] = loadSegment("e", 1, 1, true, 4);
    bonusAreas[5] = loadSegment("f", 1, 1, true, 5);
    bonusAreas[6] = loadSegment("g", 1, 1, true, 6);
    bonusAreas[7] = loadSegment("h", 1, 1, true, 7);
    bonusAreas[8] = loadSegment("i", 1, 1, true, 8);
    bonusAreas[9] = loadSegment("j", 4, 2, true, 9); // 4-2 warp zone
    bonusAreas[10] = loadSegment("k", 1, 1, true, 10);
    bonusAreas[11] = loadSegment("end", 1, 1, true, 11);
    bonusAreas[12] = loadSegment("8-4-4", 8, 4, true, 12); // 8-4 underwater
    bonusAreas[13] = loadSegment("transition", 1, 1, true, 13);
  }

  private Segment loadSegment(String name, int world, int subworld,
      boolean isBonusArea, int bonusAreaIndex) throws Throwable {

    ArrayList<int[]> freeEntryPoints = new ArrayList<int[]>();
    ArrayList<int[]> pipeEntryPoints = new ArrayList<int[]>();

    Segment segment = new Segment();
    segment.world = world;
    segment.subworld = subworld;
    segment.isBonusArea = isBonusArea;
    segment.bonusAreaIndex = bonusAreaIndex;

    ArrayList<String> lines = new ArrayList<String>();
    ClassLoader classLoader = Main.class.getClassLoader();
    BufferedReader br = new BufferedReader(new InputStreamReader(
        classLoader.getResourceAsStream("levels/" + name + ".txt")));

    segment.stoneIndex = Integer.parseInt(br.readLine().trim().substring(1));

    String items = br.readLine().trim().substring(1);
    int itemIndex = 0;

    int[] platformWidths = new int[0];
    int platformWidthsIndex = 0;
    String platformString = br.readLine().trim().substring(1);
    if (platformString.length() > 0) {
      String[] platformWidthsStrings = platformString.split(",");
      platformWidths = new int[platformWidthsStrings.length];
      for(int i = 0; i < platformWidthsStrings.length; i++) {
        platformWidths[i] = Integer.parseInt(platformWidthsStrings[i]);
      }
    }

    segment.timeLimit = Integer.parseInt(br.readLine().trim().substring(1));

    String line = null;
    while((line = br.readLine()) != null) {
      line = line.trim();
      if (line.length() > 0) {
        lines.add(line);
      }
    }

    int width = segment.width = (lines.size() / 15) * 16;
    int paddedWidth = width + 16;
    segment.displayMap = new int[15][paddedWidth];
    segment.physicsMap = new int[15][paddedWidth];
    segment.enemiesMap = new int[15][paddedWidth];
    segment.valuesMap = new int[15][paddedWidth];
    segment.suction = new int[paddedWidth];

    char[][] cs = new char[17][width + 2];
    for(int x = width + 1; x >= 0; x--) {
      cs[0][x] = '.';
      cs[16][x] = '.';
    }
    Iterator<String> i = lines.iterator();
    for(int X = 0; X < width; X += 16) {
      for(int y = 0; y < 15; y++) {
        line = i.next();
        for(int x = 0; x < 16; x++) {
          cs[y + 1][x + 1 + X] = line.charAt(x);
        }
      }
    }

    int pipeExitIndex = 0;
    for(int x = 0; x < width; x++) {
      for(int y = 0; y < 15; y++) {
        segment.enemiesMap[y][x] = EnemyTypes.NONE;
        switch(cs[y + 1][x + 1]) {
          case '|':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.hasHalfWayPoint = true;
            segment.halfWayPointX = x << 5;
            segment.halfWayPointY = y << 5;
            break;
          case '7':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.DOOR;
            break;
          case 'x':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.HOVERING_KOOPA_PARATROOPA;
            break;
          case '/':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.BULLET_BILLS_SPAWNING_START;
            break;
          case '\\':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.BULLET_BILLS_SPAWNING_END;
            break;
          case 'e':
            segment.displayMap[y][x] = BlockTypes.TURTLE_CANNON;
            segment.physicsMap[y][x] = BlockTypes.SOLID;
            segment.enemiesMap[y][x] = EnemyTypes.TURTLE_CANNON;
            break;
          case 'E':
            segment.displayMap[y][x] = BlockTypes.TURTLE_CANNON_PILLAR;
            segment.physicsMap[y][x] = BlockTypes.SOLID;
            break;
          case 'M':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.MAZE;
            break;
          case 'm':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.WRONG_WAY;
            break;
          case 'l':
            if (cs[y + 1][x] == 'u') {
              segment.displayMap[y][x] = BlockTypes.LEFT_UP_PIPE;
            } else if (cs[y + 1][x] == 'd') {
              segment.displayMap[y][x] = BlockTypes.LEFT_DOWN_PIPE;
            } else {
              segment.displayMap[y][x] = BlockTypes.LEFT_PIPE;
            }
            segment.physicsMap[y][x] = BlockTypes.SOLID;
            break;
          case 'L':
            segment.displayMap[y][x] = BlockTypes.LEFT_PIPE_CAP;
            segment.physicsMap[y][x] = BlockTypes.SOLID;
            break;
          case 'r':
            if (cs[y + 1][x + 2] == 'u') {
              segment.displayMap[y][x] = BlockTypes.RIGHT_UP_PIPE;
            } else if (cs[y + 1][x + 2] == 'd') {
              segment.displayMap[y][x] = BlockTypes.RIGHT_DOWN_PIPE;
            } else {
              segment.displayMap[y][x] = BlockTypes.RIGHT_PIPE;
            }
            segment.physicsMap[y][x] = BlockTypes.SOLID;
            break;
          case 'R':
            segment.displayMap[y][x] = BlockTypes.RIGHT_PIPE_CAP;
            segment.physicsMap[y][x] = BlockTypes.SOLID;
            break;
          case 'u':
            segment.displayMap[y][x] = BlockTypes.UP_PIPE;
            segment.physicsMap[y][x] = BlockTypes.SOLID;
            break;
          case 'U':
            segment.displayMap[y][x] = BlockTypes.UP_PIPE_CAP;
            segment.physicsMap[y][x] = BlockTypes.SOLID;
            break;
          case 'd':
            segment.displayMap[y][x] = BlockTypes.DOWN_PIPE;
            segment.physicsMap[y][x] = BlockTypes.SOLID;
            break;
          case 'D':
            segment.displayMap[y][x] = BlockTypes.DOWN_PIPE_CAP;
            segment.physicsMap[y][x] = BlockTypes.SOLID;
            break;
          case '.':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            break;
          case 'B':
            segment.displayMap[y][x] = BlockTypes.BRICK;
            segment.physicsMap[y][x] = BlockTypes.BRICK;
            break;
          case 'b':
            segment.displayMap[y][x] = BlockTypes.BRICK;
            segment.physicsMap[y][x] = BlockTypes.QUESTIONMARK;
            itemIndex = addItem(items, itemIndex, segment, x, y);
            break;
          case 'X':
            segment.displayMap[y][x] = BlockTypes.SOLID;
            segment.physicsMap[y][x] = BlockTypes.SOLID;
            break;
          case '?':
            segment.displayMap[y][x] = BlockTypes.QUESTIONMARK;
            segment.physicsMap[y][x] = BlockTypes.QUESTIONMARK;
            itemIndex = addItem(items, itemIndex, segment, x, y);
            break;
          case '!': // hidden ?
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.HIDDEN;
            itemIndex = addItem(items, itemIndex, segment, x, y);
            break;
          case 'I': // invisible
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.SOLID;
            break;
          case 'C':
            segment.displayMap[y][x] = BlockTypes.BRICK;
            segment.physicsMap[y][x] = BlockTypes.COINS;
            break;
          case 'c':
            segment.displayMap[y][x] = BlockTypes.COIN;
            segment.physicsMap[y][x] = BlockTypes.COIN;
            break;
          case 'p':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.PIRANHA_PLANT;
            break;
          case 'g':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.GOOMBA_1;
            break;
          case 'G':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.GOOMBA_2;
            break;
          case 'h':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.GOOMBA_3;
            break;
          case 'H':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.GOOMBA_PAIR;
            break;
          case 'j':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.GOOMBA_2_2;
            break;
          case 'k':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.GREEN_KOOPA_TROOPA_1;
            break;
          case 'K':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.RED_KOOPA_TROOPA;
            break;
          case 'J':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.JUMPING_BOARD;
            break;
          case 'y':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.GREEN_KOOPA_PARATROOPA;
            break;
          case 'Y':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.RED_KOOPA_PARATROOPA;
            break;
          case 'o':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.GREEN_KOOPA_TROOPA_2;
            break;
          case 'O':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.GREEN_KOOPA_TROOPA_3;
            break;
          case 'a':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.BUZZY_BEETLE;
            break;
          case 'v':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.BUZZY_BEETLE_DOUBLE;
            break;
          case 'f':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.BUZZY_BEETLE_TRIPLE;
            break;
          case 'n':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.RED_KOOPA_TROOPA_2;
            break;
          case 'Q':
            segment.displayMap[y][x] = BlockTypes.SOLID;
            segment.physicsMap[y][x] = BlockTypes.SOLID;
            segment.enemiesMap[y][x] = EnemyTypes.FIREBAR_LONG_CLOCKWISE;
            break;
          case 'w':
            if (segment.stoneIndex == PlayingMode.STONE_WATER) {
              segment.displayMap[y][x] = BlockTypes.EMPTY;
              segment.physicsMap[y][x] = BlockTypes.EMPTY;
            } else {
              segment.displayMap[y][x] = BlockTypes.SOLID;
              segment.physicsMap[y][x] = BlockTypes.SOLID;
            }
            segment.enemiesMap[y][x] = EnemyTypes.FIREBAR_COUNTERCLOCKWISE;
            break;
          case 'W':
            if (segment.stoneIndex == PlayingMode.STONE_WATER) {
              segment.displayMap[y][x] = BlockTypes.EMPTY;
              segment.physicsMap[y][x] = BlockTypes.EMPTY;
            } else {
              segment.displayMap[y][x] = BlockTypes.SOLID;
              segment.physicsMap[y][x] = BlockTypes.SOLID;
            }
            segment.enemiesMap[y][x] = EnemyTypes.FIREBAR_CLOCKWISE;
            break;
          case '_':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.ELEVATOR_DOWN;
            segment.valuesMap[y][x] = platformWidths[platformWidthsIndex++];
            break;
          case '-':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.ELEVATOR_UP;
            segment.valuesMap[y][x] = platformWidths[platformWidthsIndex++];
            break;
          case '(': 
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.RECIPROCATING_TOP;
            segment.valuesMap[y][x] = platformWidths[platformWidthsIndex++];
            break;
          case ')':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.RECIPROCATING_BOTTOM;
            break;
          case '[':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.RECIPROCATING_LEFT;
            segment.valuesMap[y][x] = platformWidths[platformWidthsIndex++];
            break;
          case ']':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.RECIPROCATING_RIGHT;
            break;
          case '"':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.SKY_LIFT;
            segment.valuesMap[y][x] = platformWidths[platformWidthsIndex++];
            break;
          case '=':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.DROP_PLATFORM;
            segment.valuesMap[y][x] = platformWidths[platformWidthsIndex++];
            break;
          case ':':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.SCALE_START;
            segment.valuesMap[y][x] = platformWidths[platformWidthsIndex++];
            break;
          case ';':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.SCALE_END;
            break;
          case '~':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.SPITTING_FIRE;
            break;
          case '%':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.CHEEP_CHEEP_SPAWNER;
            break;
          case '9':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.BLOOBER;
            break;
          case '`':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.TOAD;
            break;
          case 'Z':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.BOWSER;
            break;
          case 'F':
            segment.displayMap[2][x] = BlockTypes.FLAG_POLE_KNOB;
            segment.physicsMap[2][x] = BlockTypes.EMPTY;
            segment.displayMap[y][x] = BlockTypes.SOLID;
            segment.physicsMap[y][x] = BlockTypes.SOLID;
            segment.flagPoleX = (x << 5) + 15;
            segment.enemiesMap[y][x] = EnemyTypes.FLAG;
            break;
          case 'A':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.AXE;
            segment.maxCameraX = (x - 13) << 5;
            break;
          case '<':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.FLYING_CHEEP_CHEEPS_ON;
            break;
          case '>':
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.FLYING_CHEEP_CHEEPS_OFF;
            break;
          case '^': // pipe exit point
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            if (cs[y + 1][x] == 'D') {
              segment.physicsMap[y][x] = BlockTypes.ENTER_LEFT;
            } else if (cs[y + 1][x + 2] == 'D') {
              segment.physicsMap[y][x] = BlockTypes.ENTER_RIGHT;
            } else if (cs[y][x + 1] == 'L') {
              segment.physicsMap[y][x] = BlockTypes.ENTER_UP;
            } else if (cs[y + 2][x + 1] == 'L') {
              segment.physicsMap[y][x] = BlockTypes.ENTER_DOWN;
            }
            segment.valuesMap[y][x] = pipeExitIndex++;
            break;
          case '*': { // free entry point
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            int[] freeEntryPoint = new int[2];
            freeEntryPoint[0] = x << 5;
            freeEntryPoint[1] = y << 5;
            freeEntryPoints.add(freeEntryPoint);
            break;
          }
          case '$': { // pipe entry point
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            int[] pipeEntryPoint = new int[3];
            pipeEntryPoint[0] = x << 5;
            pipeEntryPoint[1] = y << 5;
            if (cs[y + 1][x] == 'D') {
              pipeEntryPoint[2] = Mario.PIPING_RIGHT;
            } else if (cs[y + 1][x + 2] == 'D') {
              pipeEntryPoint[2] = Mario.PIPING_LEFT;
            } else if (cs[y][x + 1] == 'L') {
              pipeEntryPoint[0] += 16;
              pipeEntryPoint[2] = Mario.PIPING_DOWN;
            } else if (cs[y + 2][x + 1] == 'L') {
              pipeEntryPoint[0] += 16;
              pipeEntryPoint[2] = Mario.PIPING_UP;
            }
            pipeEntryPoints.add(pipeEntryPoint);
            break;
          }
          case '1': // warp zone message
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.WARP_ZONE;
            if (world == 1 && subworld == 2) {
              segment.valuesMap[y][x] = 0;
            } else if (world == 4 && subworld == 2) {
              if (isBonusArea) {
                segment.valuesMap[y][x] = 1;
              } else {
                segment.valuesMap[y][x] = 2;
              }
            }
            break;
          case '2': // hammer brother
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.HAMMER_BROTHER;
            break;
          case 'S': // podoboo (fire ball)
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.PODOBOO;
            break;
          case '{': // Lakitu
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.LAKITU_START;
            break;
          case '}': // Lakitu
            segment.displayMap[y][x] = BlockTypes.EMPTY;
            segment.physicsMap[y][x] = BlockTypes.EMPTY;
            segment.enemiesMap[y][x] = EnemyTypes.LAKITU_END;
            break;
          default:
            segment.displayMap[y][x] = BlockTypes.SOLID;
            segment.physicsMap[y][x] = BlockTypes.SOLID;
            break;
        }
      }

      segment.freeEntryPoints = freeEntryPoints.toArray(new int[0][0]);
      segment.pipeEntryPoints = pipeEntryPoints.toArray(new int[0][0]);
    }

    if (segment.stoneIndex == PlayingMode.STONE_WATER) {
      for(int x = 0; x < segment.width; x++) {
        if (segment.physicsMap[14][x] <= BlockTypes.HIDDEN) {
          int end = x + 1;
          while(segment.physicsMap[14][end] <= BlockTypes.HIDDEN
              && end < segment.width) {
            end++;
          }
          int middle = (x + end) >> 1;
          for(int j = x; j < end; j++) {
            if (j < middle) {
              segment.suction[j] = SuctionTypes.RIGHT;
            } else {
              segment.suction[j] = SuctionTypes.LEFT;
            }
          }
          x = end - 1;
        } else {
          segment.suction[x] = SuctionTypes.NONE;
        }
      }
    }

    return segment;
  }

  private int addItem(String items, int itemIndex,
      Segment segment, int x, int y) {
    switch(items.charAt(itemIndex)) {
      case 'c':
        segment.valuesMap[y][x] = ItemTypes.COIN;
        break;
      case 'm':
        segment.valuesMap[y][x] = ItemTypes.MUSHROOM;
        break;
      case '1':
        segment.valuesMap[y][x] = ItemTypes.EXTRA_LIFE;
        break;
      case 's':
        segment.valuesMap[y][x] = ItemTypes.STARMAN;
        break;
      case 'v':
        segment.valuesMap[y][x] = ItemTypes.VINE;
        break;
    }
    return itemIndex + 1;
  }

  public void warpToWorld(int world) {
    passedHalfwayPoint = false;
    score = playingMode.score;
    coins = playingMode.coins;
    lives = playingMode.lives;
    this.world = world;
    subworld = 1;
    setGameMode(new WorldTitleMode(score, coins, lives, world, subworld));
  }

  public void subworldCompleted() {
    passedHalfwayPoint = false;
    score = playingMode.score;
    coins = playingMode.coins;
    lives = playingMode.lives;
    subworld++;
    if (subworld == 5) {
      world++;
      subworld = 1;
    }
    if (world == 9) {
      setGameMode(new EndingMode());
    } else {
      setGameMode(new WorldTitleMode(score, coins, lives, world, subworld));
    }
  }

  public void passedHalfwayPoint() {
    passedHalfwayPoint = true;
  }

  public void marioDied() {
    score = playingMode.score;
    coins = playingMode.coins;
    lives = playingMode.lives - 1;
    playingMode.mario.hasFlower = false;
    playingMode.mario.big = false;

    if (playingMode.timer <= 0) {
      setGameMode(new TimeUpMode());
    } else {
      finishedTimeUpMode();
    }
  }

  public void finishedCredits() {
    score = 0;
    coins = 0;
    lives = 0;
    playingMode.mario.hasFlower = false;
    playingMode.mario.big = false;
    world = 1;
    subworld = 1;
    passedHalfwayPoint = false;
    setGameMode(new TitleScreenMode(false));
  }

  public void finishedTimeUpMode() {
    if (lives == 0) {
      setGameMode(new TitleScreenMode(true));
    } else {
      setGameMode(new WorldTitleMode(score, coins, lives, world, subworld));
    }
  }

  public void finishedTransitionScene() {
    setGameMode(new PlayingMode(
        worlds[world - 1][subworld - 1], true, 0, false));
  }

  public void worldTitleModeComplete() {
    if (playingMode != null
        && (playingMode.segment.world != world
            || playingMode.segment.subworld != subworld)
        && (worlds[world - 1][subworld - 1].stoneIndex
            == PlayingMode.STONE_UNDERGROUND
            || worlds[world - 1][subworld - 1].stoneIndex
                == PlayingMode.STONE_WATER)) {
      bonusAreas[13].world = world;
      bonusAreas[13].subworld = subworld;
      setGameMode(new PlayingMode(
          bonusAreas[13], true, 0, false));
    } else {
      setGameMode(new PlayingMode(
          worlds[world - 1][subworld - 1], true, 0, false));
    }
  }

  public void startDemo() {
    hardMode2 = hardMode;
    hardMode = false;
    world = 1;
    random = new Random(123456789);
    passedHalfwayPoint = false;
    playingMode = null;
    inputPlayer.resetInput();
    subworld = inputPlayer.reset() + 1;
    lives = 3;
    score = 0;
    coins = 0;
    PlayingMode mode = new PlayingMode(
        worlds[world - 1][subworld - 1], true, 0, false);
    mode.demoMode = true;
    setGameMode(mode);
  }

  public void finishedDemo() {
    hardMode = hardMode2;
    finishedCredits();
  }

  public void startGame() {
    world = 1;
    continueGame();
  }

  public void continueGame() {
    random = new Random();
    passedHalfwayPoint = false;
    playingMode = null;
    subworld = 1;
    lives = 3;
    score = 0;
    coins = 0;
    setGameMode(new WorldTitleMode(score, coins, lives, world, subworld));
  }

  public void enterPipe(int world, int subworld, boolean isBonusArea,
      int bonusAreaIndex, int pipeIndex) {

    if (isBonusArea) {
      if (bonusAreaIndex == 13) {
        finishedTransitionScene();
        return;
      }
      switch(world) {
        case 1:
          switch(subworld) {
            case 1:
              setGameMode(new PlayingMode(worlds[0][0], false, 0, true));
              break;
            case 2:
              setGameMode(new PlayingMode(worlds[0][1], false, 0, true));
              break;
          }
          break;
        case 2:
          switch(subworld) {
            case 1:
              switch(pipeIndex) {
                case 0:
                  setGameMode(new PlayingMode(worlds[1][0], false, 0, true));
                  break;
                case -1:
                  setGameMode(new PlayingMode(worlds[1][0], true, 1, true));
                  break;
              }
              break;
          }
          break;
        case 3:
          switch(subworld) {
            case 1:
              switch(pipeIndex) {
                case 0:
                  setGameMode(new PlayingMode(worlds[2][0], false, 0, true));
                  break;
                case -1:
                  setGameMode(new PlayingMode(worlds[2][0], true, 1, true));
                  break;
              }
              break;
          }
          break;
        case 4:
          switch(subworld) {
            case 1:
              setGameMode(new PlayingMode(worlds[3][0], false, 0, true));
              break;
            case 2:
              if (bonusAreaIndex == 9) {
                switch(pipeIndex) {
                  case 0:
                    if (hardMode) {
                      bonusAreas[11].world = 4;
                      bonusAreas[11].subworld = 2;
                      setGameMode(new PlayingMode(
                          bonusAreas[11], false, 0, true));
                    } else {
                      warpToWorld(8);
                    }
                    break;
                  case 1:
                    if (hardMode) {
                      bonusAreas[11].world = 4;
                      bonusAreas[11].subworld = 2;
                      setGameMode(new PlayingMode(
                          bonusAreas[11], false, 0, true));
                    } else {
                      warpToWorld(7);
                    }
                    break;
                  case 2:
                    if (hardMode) {
                      bonusAreas[11].world = 4;
                      bonusAreas[11].subworld = 2;
                      setGameMode(new PlayingMode(
                          bonusAreas[11], false, 0, true));
                    } else {
                      warpToWorld(6);
                    }
                    break;
                }
              } else {
                setGameMode(new PlayingMode(worlds[3][1], false, 0, true));
              }
              break;
          }
          break;
        case 5:
          switch(subworld) {
            case 1:
              setGameMode(new PlayingMode(worlds[4][0], false, 0, true));
              break;
            case 2:
              switch(pipeIndex) {
                case 0:
                  setGameMode(new PlayingMode(worlds[4][1], false, 0, true));
                  break;
                case -1:
                  setGameMode(new PlayingMode(worlds[4][1], true, 1, true));
                  break;
              }
              break;
          }
          break;
        case 6:
          switch(subworld) {
            case 2:
              switch(bonusAreaIndex) {
                case 4:
                  setGameMode(new PlayingMode(worlds[5][1], false, 0, true));
                  break;
                case 6:
                  setGameMode(new PlayingMode(worlds[5][1], false, 1, true));
                  break;
                case 3:
                  setGameMode(new PlayingMode(worlds[5][1], false, 2, true));
                  break;
                case 10:
                  setGameMode(new PlayingMode(worlds[5][1], true, 1, true));
                  break;
              }
              break;
          }
          break;
        case 7:
          switch(subworld) {
            case 1:
              setGameMode(new PlayingMode(worlds[6][0], false, 0, true));
              break;
          }
          break;
        case 8:
          switch(subworld) {
            case 1:
              setGameMode(new PlayingMode(worlds[7][0], false, 0, true));
              break;
            case 2:
              setGameMode(new PlayingMode(worlds[7][1], false, 0, true));
              break;
            case 4:
              setGameMode(new PlayingMode(worlds[7][3], false, 3, true));
              break;
          }
          break;
      }
    } else {
      switch(world) {
        case 1:
          switch(subworld) {
            case 1:
              bonusAreas[0].world = 1;
              bonusAreas[0].subworld = 1;
              setGameMode(new PlayingMode(bonusAreas[0], true, 0, true));
              break;
            case 2:
              switch(pipeIndex) {
                case 0:
                  bonusAreas[1].world = 1;
                  bonusAreas[1].subworld = 2;
                  setGameMode(new PlayingMode(bonusAreas[1], true, 0, true));
                  break;
                case 1:
                  bonusAreas[11].world = 1;
                  bonusAreas[11].subworld = 2;
                  setGameMode(new PlayingMode(bonusAreas[11], false, 0, true));                  
                  break;
                case 2:
                  if (hardMode) {
                    bonusAreas[11].world = 1;
                    bonusAreas[11].subworld = 2;
                    setGameMode(new PlayingMode(
                        bonusAreas[11], false, 0, true));
                  } else {
                    warpToWorld(4);
                  }
                  break;
                case 3:
                  if (hardMode) {
                    bonusAreas[11].world = 1;
                    bonusAreas[11].subworld = 2;
                    setGameMode(new PlayingMode(
                        bonusAreas[11], false, 0, true));
                  } else {
                    warpToWorld(3);
                  }
                  break;
                case 4:
                  if (hardMode) {
                    bonusAreas[11].world = 1;
                    bonusAreas[11].subworld = 2;
                    setGameMode(new PlayingMode(
                        bonusAreas[11], false, 0, true));
                  } else {
                    warpToWorld(2);
                  }
                  break;
              }
              break;
          }
          break;
        case 2:
          switch(subworld) {
            case 1:
              switch(pipeIndex) {
                case 0:
                  bonusAreas[0].world = 2;
                  bonusAreas[0].subworld = 1;
                  setGameMode(new PlayingMode(bonusAreas[0], true, 0, true));
                  break;
                case -1:
                  bonusAreas[7].world = 2;
                  bonusAreas[7].subworld = 1;
                  setGameMode(new PlayingMode(bonusAreas[7], true, 0, true));
                  break;
              }
              break;
            case 2:
              bonusAreas[11].world = 2;
              bonusAreas[11].subworld = 2;
              setGameMode(new PlayingMode(bonusAreas[11], false, 0, true));
              break;
          }
          break;
        case 3:
          switch(subworld) {
            case 1:
              switch(pipeIndex) {
                case 0:
                  bonusAreas[2].world = 3;
                  bonusAreas[2].subworld = 1;
                  setGameMode(new PlayingMode(bonusAreas[2], true, 0, true));
                  break;
                case -1:
                  bonusAreas[8].world = 3;
                  bonusAreas[8].subworld = 1;
                  setGameMode(new PlayingMode(bonusAreas[8], true, 0, true));
                  break;
              }
              break;
          }
          break;
        case 4:
          switch(subworld) {
            case 1:
              bonusAreas[3].world = 4;
              bonusAreas[3].subworld = 1;
              setGameMode(new PlayingMode(bonusAreas[3], true, 0, true));
              break;
            case 2:
              switch(pipeIndex) {
                case -1:
                  bonusAreas[9].world = 4;
                  bonusAreas[9].subworld = 2;
                  setGameMode(new PlayingMode(bonusAreas[9], true, 0, true));
                  break;
                case 0:
                  bonusAreas[4].world = 4;
                  bonusAreas[4].subworld = 2;
                  setGameMode(new PlayingMode(bonusAreas[4], true, 0, true));
                  break;
                case 1:
                  bonusAreas[11].world = 4;
                  bonusAreas[11].subworld = 2;
                  setGameMode(new PlayingMode(bonusAreas[11], false, 0, true));
                  break;
                case 2:
                  if (hardMode) {
                    bonusAreas[11].world = 4;
                    bonusAreas[11].subworld = 2;
                    setGameMode(new PlayingMode(
                        bonusAreas[11], false, 0, true));
                    break;
                  } else {
                    warpToWorld(5);
                  }
                  break;
              }
              break;
          }
          break;
        case 5:
          switch(subworld) {
            case 1:
              bonusAreas[4].world = 5;
              bonusAreas[4].subworld = 1;
              setGameMode(new PlayingMode(bonusAreas[4], true, 0, true));
              break;
            case 2:
              switch(pipeIndex) {
                case 0:
                  bonusAreas[5].world = 5;
                  bonusAreas[5].subworld = 2;
                  setGameMode(new PlayingMode(bonusAreas[5], true, 0, true));
                  break;
                case -1:
                  bonusAreas[7].world = 5;
                  bonusAreas[7].subworld = 2;
                  setGameMode(new PlayingMode(bonusAreas[7], true, 0, true));
                  break;
              }
              break;
          }
          break;
        case 6:
          switch(subworld) {
            case 2:
              switch(pipeIndex) {
                case 0:
                  bonusAreas[4].world = 6;
                  bonusAreas[4].subworld = 2;
                  setGameMode(new PlayingMode(bonusAreas[4], true, 0, true));
                  break;
                case 1:
                  bonusAreas[6].world = 6;
                  bonusAreas[6].subworld = 2;
                  setGameMode(new PlayingMode(bonusAreas[6], true, 0, true));
                  break;
                case 2:
                  bonusAreas[3].world = 6;
                  bonusAreas[3].subworld = 2;
                  setGameMode(new PlayingMode(bonusAreas[3], true, 0, true));
                  break;
                case -1:
                  bonusAreas[10].world = 6;
                  bonusAreas[10].subworld = 2;
                  setGameMode(new PlayingMode(bonusAreas[10], true, 0, true));
                  break;
              }
              break;
          }
          break;
        case 7:
          switch(subworld) {
            case 1:
              bonusAreas[0].world = 7;
              bonusAreas[0].subworld = 1;
              setGameMode(new PlayingMode(bonusAreas[0], true, 0, true));
              break;
            case 2:
              bonusAreas[11].world = 7;
              bonusAreas[11].subworld = 2;
              setGameMode(new PlayingMode(bonusAreas[11], false, 0, true));
              break;
          }
          break;
        case 8:
          switch(subworld) {
            case 1:
              bonusAreas[1].world = 8;
              bonusAreas[1].subworld = 1;
              setGameMode(new PlayingMode(bonusAreas[1], true, 0, true));
              break;
            case 2:
              bonusAreas[4].world = 8;
              bonusAreas[4].subworld = 2;
              setGameMode(new PlayingMode(bonusAreas[4], true, 0, true));
              break;
            case 4:
              switch(pipeIndex) {
                case 0:
                case 2:
                case 4:
                case 6:
                  setGameMode(new PlayingMode(worlds[7][3], false, 0, true));
                  break;
                case 1:
                  setGameMode(new PlayingMode(worlds[7][3], false, 1, true));
                  break;
                case 3:
                  setGameMode(new PlayingMode(worlds[7][3], false, 2, true));
                  break;
                case 5:
                  setGameMode(new PlayingMode(bonusAreas[12], false, 0, true));
                  break;
              }
              break;
          }
          break;
      }
    }
  }
  
  public static void main(String[] args) {
    JFrame frame = new JFrame("Worst Mario Game Ever");
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    Main applet = new Main();
    applet.init();
    applet.setPreferredSize(new Dimension(512, 480));    
    frame.add(applet, BorderLayout.CENTER);
    frame.setResizable(false);
    frame.pack();
    frame.setLocationRelativeTo(null);
    frame.setVisible(true);
  }
}
