package mario;

import java.awt.*;

public class Mario extends WorldObject {

  public static final float ACCELERATION = 0.25f;
  public static final float SWIM_ACCELERATION = 0.125f;
  public static final float WALK_SPEED = 2f;
  public static final float RUN_SPEED = 5f;
  public static final float SWIM_SPEED = 3f;
  public static final float HIGH_JUMP_SPEED = 0.5f * (WALK_SPEED + RUN_SPEED);
  public static final int LOW_JUMP_COUNT = 10;
  public static final int HIGH_JUMP_COUNT = 14;

  public static final int JUMP_STATE_SUPPORTED = 0;
  public static final int JUMP_STATE_JUMP_UP = 1;
  public static final int JUMP_STATE_JUMP = 2;

  public static final int PIPING_LEFT = 0;
  public static final int PIPING_RIGHT = 1;
  public static final int PIPING_UP = 2;
  public static final int PIPING_DOWN = 3;

  public IInput input;
  public boolean big = false;
  public boolean hasFlower = false;
  public boolean kneeling = false;
  public int jumpState = JUMP_STATE_JUMP;
  public int jumpUpCount;
  public int maxJumpUpCount;
  public WorldObject enemySupport;
  public StaticKoopaShell shellSupport;
  public boolean acquiringItem;
  public boolean transforming;
  public boolean shrinking;
  public int transformCount;
  public boolean facingLeft;
  public boolean lastIsRun;
  public int starCount;
  public int stunned;
  public boolean piping;
  public int pipingType;
  public int pipingCount;
  public int pipingX;
  public int pipingY;
  public int pipingIndex;
  public boolean pipingExit;
  public boolean onJumpingBoard;
  public boolean onVine;
  public boolean vining;
  public boolean swimming;
  public Color color;
  public boolean demoMode;

  public Mario() {
    bx1 = 0;
    by1 = 0;
    bx2 = 31;
    by2 = 31;
    mx1 = 0;
    my1 = 0;
    mx2 = 31;
    my2 = 31;
    big = false;
  }

  @Override
  public void update() {

    if (!input.hasNext() || (demoMode && input.isStart())) {
      main.finishedDemo();
      return;
    }

    boolean isUp = input.isUp();
    boolean isDown = input.isDown();
    boolean isLeft = input.isLeft();
    boolean isRight = input.isRight();
    boolean isRun = input.isRun();
    boolean isJump = input.isJump();

    if (x == playingMode.flagPoleX - 32) {
      playingMode.hitFlagPole();
      return;
    }

    if (remove) {
      if (y > 1500) {
        main.marioDied();
      } else {
        vy += G;
        fy += vy;
        y = (int)fy;
      }
      return;
    }

    if (vining) {
      if (y <= 80) {
        translateY(-2, true);
        if (y + by2 < 0) {
          main.enterPipe(playingMode.world, playingMode.subworld,
              playingMode.isBonusArea, playingMode.bonusAreaIndex, -1);
        }
      }
      return;
    }

    if (piping) {
      if (pipingCount < 64) {
        pipingCount++;
        switch(pipingType) {
          case PIPING_UP:
            fy = --y;
            break;
          case PIPING_DOWN:
            fy = ++y;
            break;
          case PIPING_LEFT:
            fx = --x;
            break;
          case PIPING_RIGHT:
            fx = ++x;
            break;
        }
      } else {
        if (pipingExit) {
          main.enterPipe(playingMode.world, playingMode.subworld,
              playingMode.isBonusArea, playingMode.bonusAreaIndex,
              pipingIndex);
        } else {
          piping = false;
          pipingCount = 0;
        }
      }
      return;
    }

    if (starCount > 0) {
      starCount--;
    }

    if (transforming) {
      if (++transformCount == 60) {
        transforming = false;
        acquiringItem = false;
        if (shrinking) {
          shrinking = false;
          big = false;
          hasFlower = false;
          y += 32;
          fy = y;
          bx1 = 0;
          by1 = 0;
          bx2 = 31;
          by2 = 31;
          mx1 = 0;
          my1 = 0;
          mx2 = 31;
          my2 = 31;
          stunned = 150;
        }
      } else {
        return;
      }
    }

    if (swimming) {
      int suctionType = playingMode.suction[((x + 16) >> 5)];
      if (suctionType != SuctionTypes.NONE) {
        G = 0.25f;
        if (suctionType == SuctionTypes.LEFT) {
          if (x > playingMode.cameraX) {
            translateX(-0.75f, false);
          }
        } else {
          translateX(0.75f, false);
        }
      } else {
        G = 0.1f;
      }
    }

    enemySupport = null;
    shellSupport = null;
    boolean supported = false;
    if (onVine) {
      vy = 0;
      jumpState = JUMP_STATE_SUPPORTED;
      supported = true;
      if (isUp) {
        translateY(-2, true);
        if (y <= 32) {
          vining = true;
          return;
        }
      } else if (isDown) {
        translateY(2, true);
      }
    } else {
      supported = applyGravity(true);
    }
    if (supported && enemySupport == null) {
      playingMode.resetStompPoints();
      if (isDown && getPhysicsBlock(x + bx1, y + by2)
          == BlockTypes.ENTER_DOWN) {
        exitViaPipe(PIPING_DOWN, getValuesBlock(x + bx1, y + by2));
        return;
      }
    }

    scanForCoins();

    if (big) {
      if (supported && !onVine) {
        kneeling = isDown;
        if (kneeling) {
          by1 = 32;          
        } else {
          by1 = 0;
        }
      }
    } else {
      kneeling = false;
    }

    if (swimming) {

      if (y < 48) {
        vy = 0;
        fy = y = 48;
      }

      switch(jumpState) {
        case JUMP_STATE_SUPPORTED:
        case JUMP_STATE_JUMP:
          if (isJump) {
            jumpState = JUMP_STATE_JUMP_UP;
            vy = -3f;
          }
          break;
        case JUMP_STATE_JUMP_UP:
          if (!isJump) {
            jumpState = JUMP_STATE_JUMP;
          }
          break;
      }

      if (!kneeling) {
        if (isLeft) {
          if (supported) {
            if (vx > -WALK_SPEED) {
              vx -= SWIM_ACCELERATION;
            } else {
              vx = -WALK_SPEED;
            }
          } else {
            if (vx > -SWIM_SPEED) {
              vx -= SWIM_ACCELERATION;
            } else {
              vx = -SWIM_SPEED;
            }
          }
        } else if (isRight) {
          if (supported) {
            if (vx < WALK_SPEED) {
              vx += SWIM_ACCELERATION;
            } else {
              vx = WALK_SPEED;
            }
          } else {
            if (vx < SWIM_SPEED) {
              vx += SWIM_ACCELERATION;
            } else {
              vx = SWIM_SPEED;
            }
          }
        }
      }
      if (!isLeft && !isRight) {
        if (vx < -SWIM_ACCELERATION) {
          vx += SWIM_ACCELERATION;
        } else if (vx > SWIM_ACCELERATION) {
          vx -= SWIM_ACCELERATION;
        } else {
          vx = 0;
        }
      }
    } else {
      switch(jumpState) {
        case JUMP_STATE_SUPPORTED:
          if (supported && !onVine) {
            if (isJump) {
              playingMode.resetStompPoints();
              if (onJumpingBoard) {
                jumpState = JUMP_STATE_JUMP;
                vy = -20f;
              } else {
                jumpState = JUMP_STATE_JUMP_UP;
                vy = -8f;
              }
              applyGravity(true);
              jumpUpCount = 0;
              maxJumpUpCount = Math.abs(vx) >= HIGH_JUMP_SPEED
                  ? HIGH_JUMP_COUNT : LOW_JUMP_COUNT;
            }
          } else {
            jumpState = JUMP_STATE_JUMP;
          }
          break;
        case JUMP_STATE_JUMP_UP:
          if (isJump && !onVine) {
            if (++jumpUpCount > maxJumpUpCount) {
              jumpState = JUMP_STATE_JUMP;
            } else {
              vy = -8f;
            }
          } else {
            jumpState = JUMP_STATE_JUMP;
          }
          break;
        case JUMP_STATE_JUMP:
          if (supported && !isJump) {
            jumpState = JUMP_STATE_SUPPORTED;
          }
          break;
      }

      if (!(kneeling && supported)) {
        if (isLeft) {
          if (isRun) {
            if (vx > -RUN_SPEED) {
              vx -= ACCELERATION;
            } else {
              vx = -RUN_SPEED;
            }
          } else {
            if (vx > -WALK_SPEED) {
              vx -= ACCELERATION;
            } else {
              vx = -WALK_SPEED;
            }
          }
        } else if (isRight) {
          if (isRun) {
            if (vx < RUN_SPEED) {
              vx += ACCELERATION;
            } else {
              vx = RUN_SPEED;
            }
          } else {
            if (vx < WALK_SPEED) {
              vx += ACCELERATION;
            } else {
              vx = WALK_SPEED;
            }
          }
        }
      }
      if ((!isLeft && !isRight) || kneeling) {
        if (vx < -ACCELERATION) {
          vx += ACCELERATION;
        } else if (vx > ACCELERATION) {
          vx -= ACCELERATION;
        } else {
          vx = 0;
        }
      }
    }

    if (vx != 0) {      
      float targetX = fx + vx;      
      if (targetX < playingMode.cameraX) {
        targetX = playingMode.cameraX;
        vx = 0;
      }
      if (!translateX(vx, true)) {
        vx = 0;        
        if (supported && !kneeling && (x & 31) == 0 && (y & 31) == 0) {
          if (isRight) {
            if (getPhysicsBlock(x, big ? y + 32 : y)
                  == BlockTypes.ENTER_RIGHT) {
              exitViaPipe(PIPING_RIGHT, getValuesBlock(x, big ? y + 32 : y));
              return;
            }
          } else if (isLeft) {
            if (getPhysicsBlock(x, big ? y + 32 : y)
                  == BlockTypes.ENTER_LEFT) {
              exitViaPipe(PIPING_LEFT, getValuesBlock(x, big ? y + 32 : y));
              return;
            }
          }
        }
      } 
    }

    if (stunned > 0) {
      stunned--;
    } else {
      for(int i = playingMode.enemies.size() - 1; i >= 0; i--) {
        WorldObject enemy = playingMode.enemies.get(i);
        if (!enemy.remove && enemy.hurtsMario()) {
          if (starCount == 0) {
            die();
            return;
          } else {
            enemy.starred(vx < 0);
          }
        }
      }
    } 

    if (enemySupport != null && !enemySupport.remove) {
      enemySupport.crush();
      vy = -7f;
      jumpState = JUMP_STATE_JUMP;
    }

    for(int i = playingMode.shells.size() - 1; i >= 0; i--) {
      StaticKoopaShell shell = playingMode.shells.get(i);
      if (!shell.remove) {
        if (intersects(shell)) {
          shell.remove = true;
          playingMode.addPoints(shell, shell.timeOut < 60 ? 1000 
              : shell.timeOut < 2 * 60 ? 500 : 400);
          playingMode.enemies.add(new MovingKoopaShell(
              shell.x, shell.y, x > shell.x, shell.type));
        }
      }
    }

    if (shellSupport != null && !shellSupport.remove) {
      shellSupport.remove = true;
      playingMode.addStompPoints(this);
      playingMode.enemies.add(new MovingKoopaShell(
          shellSupport.x, shellSupport.y,
          x > shellSupport.x, shellSupport.type));
      vy = -7f;
      jumpState = JUMP_STATE_JUMP;
    }

    int block = getPhysicsBlock(x, y + by1 + 15);
    if (block > BlockTypes.HIDDEN) {
      x++;
      fx = x;
    } else {
      block = getPhysicsBlock(x + 31, y + by1 + 15);
      if (block > BlockTypes.HIDDEN) {
        x++;
        fx = x;
      }
    }

    if (x > playingMode.maxCameraX + 480) {
      fx = x = playingMode.maxCameraX + 480;
    }

    if (isLeft) {
      facingLeft = true;
    } else if (isRight) {
      facingLeft = false;
    }
    if (!lastIsRun && isRun) {
      if (hasFlower && playingMode.fireballCount < 2) {        
        if (facingLeft) {
          playingMode.others.add(new Fireball(x - 16, y + 16, true));
        } else {
          playingMode.others.add(new Fireball(x + 32, y + 16, false));
        }
      }
    }
    lastIsRun = isRun;

    if (y > 480) {
      playingMode.fellInPit();
      return;
    }
  }

  private void scanForCoins() {
    int x1 = (x + bx1) >> 5;
    int x2 = (x + bx2) >> 5;
    int y1 = (y + by1) >> 5;
    int y2 = (y + by2) >> 5;

    for(int i = y1; i <= y2; i++) {
      for(int j = x1; j <= x2; j++) {        
        if (getPhysicsMap(j, i) == BlockTypes.COIN) {
          int X = j << 5;
          int Y = i << 5;
          if (intersects(X + 5, Y + 2, X + 20, Y + 28, true)) {
            playingMode.physicsMap[i][j] = BlockTypes.EMPTY;
            playingMode.displayMap[i][j] = BlockTypes.EMPTY;
            playingMode.score += 200;
            playingMode.collectCoin(this);
          }
        }
      }
    }
  }

  public void timeUp() {
    if (!remove) {
      starCount = 0;
      remove = true;
      G = 0.58f;
      vy = -10f;
    }
  }

  public void die() {
    if (!remove) {
      if (stunned == 0 && starCount == 0) {
        if (big || hasFlower) {
          shrink();
        } else {
          starCount = 0;
          remove = true;
          G = 0.58f;
          vy = -10f;
        }
      }
    }
  }

  public void shrink() {
    if (big || hasFlower) {
      acquiringItem = true;
      transformCount = 0;
      shrinking = true;
      transforming = true;
      transformCount = 0;
      starCount = 0;
      hasFlower = false;
    }    
  }

  public void exitViaPipe(int pipingType, int pipingIndex) {
    stunned = 0;
    pipingX = x;
    pipingY = y;
    starCount = 0;
    piping = true;
    pipingExit = true;
    pipingCount = big ? 0 : 32;
    if (pipingType == PIPING_LEFT || pipingType == PIPING_RIGHT) {
      pipingCount = 32;
    }
    this.pipingType = pipingType;
    this.pipingIndex = pipingIndex;
  }

  @Override
  public boolean isSolid(int rx1, int ry1, int rx2, int ry2, int direction) {
    if (direction == DirectionTypes.DOWN) {
      if (starCount == 0) {
        for(int i = playingMode.shells.size() - 1; i >= 0; i--) {
          StaticKoopaShell shell = playingMode.shells.get(i);
          if (!shell.remove && shell.intersects(rx1, ry1, rx2, ry2, true)) {
            shellSupport = shell;
            return true;
          }
        }
        for(int i = playingMode.enemies.size() - 1; i >= 0; i--) {
          WorldObject enemy = playingMode.enemies.get(i);
          if (enemy.supports && enemy.intersects(rx1, ry1, rx2, ry2, false)) {
            enemySupport = enemy;
            return true;
          }
        }
      }
    }
    if (rx2 >= playingMode.flagPoleX) {
      return true;
    }
    return super.isSolid(rx1, ry1, rx2, ry2, direction);
  }

  public void collectStar() {
    starCount = 15 * 60;
    stunned = 0;
  }

  public void collectMushroom() {
    if (!big) {
      acquiringItem = true;
      transformCount = 0;
      transforming = true;
      big = true;
      y -= 32;
      fy = y;
      bx1 = 0;
      by1 = 0;
      bx2 = 31;
      by2 = 63;
      mx1 = 0;
      my1 = 0;
      mx2 = 31;
      my2 = 63;
      stunned = 0;
    }
  }

  public void collectFlower() {
    if (!hasFlower) {
      if (big) {
        acquiringItem = true;
        hasFlower = true;
        transformCount = 0;
        transforming = true;
        stunned = 0;
      } else {
        collectMushroom();
      }
    }
  }

  @Override
  public boolean applyGravity(boolean useBorder) {
    vy += G;
    if (vy >= 0) {
      boolean movingFast = ((vx <= 0.0F) ? -vx : vx) >= HIGH_JUMP_SPEED;
      if (movingFast) {
        mx2 = bx2 = 32;
      }
      boolean hasSupport = isSupported(useBorder);
      if (movingFast) {
        mx2 = bx2 = 31;
      }
      if (hasSupport) {
        vy = 0;
        return true;
      } else {
        return !translateY(vy, useBorder);
      }
    } else {
      if (!translateY(vy, useBorder)) {
        int x32 = x & 0x1F;
        int ya = y + by1 - 1;
        if (x32 < 6) {
          if (getPhysicsBlock(x, ya) < BlockTypes.HIDDEN
              && getPhysicsBlock(x + 31, ya) > BlockTypes.HIDDEN
              && x > playingMode.cameraX + 1) {
            translateX(-1, useBorder);
            vy -= G;
            if (jumpState == JUMP_STATE_JUMP_UP && jumpUpCount > 0) {
              jumpUpCount--;
            }
            return false;
          }
        } else if (x32 > 25) {
          if (getPhysicsBlock(x + 31, ya) < BlockTypes.HIDDEN
              && getPhysicsBlock(x, ya) > BlockTypes.HIDDEN
              && x < playingMode.cameraX + 511) {
            translateX(1, useBorder);
            vy -= G;
            if (jumpState == JUMP_STATE_JUMP_UP && jumpUpCount > 0) {
              jumpUpCount--;
            }
            return false;
          }
        }
        vy = -vy;
        jumpState = JUMP_STATE_JUMP;
        if (!hitBlock(x + 16, ya)) {
          if (!hitBlock(x, ya)) {
            hitBlock(x + 31, ya);
          }
        }
      }
      return false;
    }
  }

  private boolean hitBlock(int x, int y) {
    switch(getPhysicsBlock(x, y)) {
      case BlockTypes.BRICK:
        if (big) {
          removeBlock(x, y);
          int X = x & 0xFFFFFFE0;
          int Y = y & 0xFFFFFFE0;
          playingMode.others.add(new Brick(X, Y, -4, -10));
          playingMode.others.add(new Brick(X + 16, Y, 4, -10));
          playingMode.others.add(new Brick(X, Y + 16, -4.5f, -7));
          playingMode.others.add(new Brick(X + 16, Y + 16, 4.5f, -7));
          playingMode.platforms.add(new BrickPlatform(x, y, true));
          playingMode.score += 50;
        } else {
          playingMode.platforms.add(new BrickPlatform(x, y));
        }
        return true;
      case BlockTypes.QUESTIONMARK:
        playingMode.platforms.add(new QuestionmarkPlatform(x, y));
        return true;
      case BlockTypes.COINS:
        playingMode.platforms.add(new CoinPlatform(x, y));
        return true;
      case BlockTypes.HIDDEN:
        playingMode.platforms.add(new QuestionmarkPlatform(x, y));
        return true;
    }
    return false;
  }

  public boolean isPlaying() {
    return !(acquiringItem || remove || piping || vining);
  }

  @Override
  public void render() {
    if (stunned > 0 && ((stunned>>1)&1) == 0) {
      return;
    }
    if (starCount > 0) {
      if (starCount > 180) {
        g.setColor(Palette.GAINING_FLOWERS[(starCount>>2)&7]);
      } else {
        g.setColor(Palette.GAINING_FLOWERS[(starCount>>3)&7]);
      }
    } else {
      g.setColor(hasFlower ? Palette.MARIO_WITH_FLOWER : color);
    }
    if (piping) {
      if (big) {
        g.setClip(pipingX - playingMode.cameraX, pipingY, 32, 64);
      } else {
        g.setClip(pipingX - playingMode.cameraX, pipingY, 32, 32);
      }
    }
    g.translate(x - playingMode.cameraX, y);
    if (this.acquiringItem) {
      if (hasFlower) {
        g.setColor(Palette.GAINING_FLOWERS[(transformCount>>2)&7]);
        g.drawOval(2, 0, 27, 23);
        g.drawLine(15, 24, 15, 44);
        g.drawLine(16, 24, 16, 44);
        g.drawLine(15, 44, 0, 63);
        g.drawLine(16, 44, 31, 63);
        g.drawLine(15, 27, 0, 41);
        g.drawLine(16, 27, 31, 41);
      } else {
        int count = shrinking ? 59 - transformCount : transformCount;
        if (count < 30) {
          if ((count&4) == 0) {
            g.translate(0, 16);
            g.drawOval(2, 0, 27, 17);
            g.drawLine(15, 18, 15, 33);
            g.drawLine(16, 18, 16, 33);
            g.drawLine(15, 32, 0, 47);
            g.drawLine(16, 32, 31, 47);
            g.drawLine(15, 20, 0, 30);
            g.drawLine(16, 20, 31, 30);
            g.translate(0, -16);
          } else {
            g.translate(0, 32);
            g.drawOval(2, 0, 27, 11);
            g.drawLine(15, 12, 15, 22);
            g.drawLine(16, 12, 16, 22);
            g.drawLine(15, 22, 0, 31);
            g.drawLine(16, 22, 31, 31);
            g.drawLine(15, 13, 0, 20);
            g.drawLine(16, 13, 31, 20);
            g.translate(0, -32);
          }
        } else {
          if ((count&4) == 0) {
            g.translate(0, 16);
            g.drawOval(2, 0, 27, 17);
            g.drawLine(15, 18, 15, 33);
            g.drawLine(16, 18, 16, 33);
            g.drawLine(15, 32, 0, 47);
            g.drawLine(16, 32, 31, 47);
            g.drawLine(15, 20, 0, 30);
            g.drawLine(16, 20, 31, 30);
            g.translate(0, -16);
          } else {
            g.drawOval(2, 0, 27, 23);
            g.drawLine(15, 24, 15, 44);
            g.drawLine(16, 24, 16, 44);
            g.drawLine(15, 44, 0, 63);
            g.drawLine(16, 44, 31, 63);
            g.drawLine(15, 27, 0, 41);
            g.drawLine(16, 27, 31, 41);
          }
        }
      }
    } else if (big && !kneeling) {
      g.drawOval(2, 0, 27, 23);
      g.drawLine(15, 24, 15, 44);
      g.drawLine(16, 24, 16, 44);
      g.drawLine(15, 44, 0, 63);
      g.drawLine(16, 44, 31, 63);
      g.drawLine(15, 27, 0, 41);
      g.drawLine(16, 27, 31, 41);
    } else {
      if (kneeling) {
        g.translate(0, 32);
      }
      g.drawOval(2, 0, 27, 11);
      g.drawLine(15, 12, 15, 22);
      g.drawLine(16, 12, 16, 22);
      g.drawLine(15, 22, 0, 31);
      g.drawLine(16, 22, 31, 31);
      g.drawLine(15, 13, 0, 20);
      g.drawLine(16, 13, 31, 20);
      if (kneeling) {
        g.translate(0, -32);
      }
    }
    g.translate(playingMode.cameraX - x, -y);
    if (piping) {
      g.setClip(null);
    }
  }

}
