package mario;

import java.awt.*;

public class MovingKoopaShell extends WorldObject {

  public static final float SPEED = 6;

  public int type;
  public int shellSequenceIndex;
  public Color color;

  public MovingKoopaShell(int x, int y, boolean left, int type) {
    this.fx = this.x = x;
    this.fy = this.y = y;
    this.type = type;
    this.vx = left ? -SPEED : SPEED;
    switch(type) {
      case KoopaTroopaTypes.GREEN:
        color = Palette.GREEN_KOOPA_TROOPA;
        break;
      case KoopaTroopaTypes.RED:
        color = Palette.RED_KOOPA_TROOPA;
        break;
      case KoopaTroopaTypes.BUZZY_BEETLE:
        color = Palette.BUZZY_BEETLE;
        break;
    }
  }

  public void update() {

    for(int i = playingMode.platforms.size() - 1; i >= 0; i--) {
      AbstractPlatform platform = playingMode.platforms.get(i);
      if (platform.deadly && platform.intersects(this)) {
        remove = true;
        playingMode.shells.add(new StaticKoopaShell(x, y,
            platform.x < x ? 4 : -4, -9, type));
        playingMode.addPoints(this, 100);
        return;
      }
    }

    applyGravity(true);
    if (vx < 0) {
      if (!translateX(vx, false)) {
        vx = Math.abs(vx);
      }
    } else {
      if (!translateX(vx, false)) {
        vx = -Math.abs(vx);
      }
    }

    WorldObject enemy = playingMode.intersectsEnemy(this);
    if (enemy != null) {
      enemy.shelled(vx < 0);
      if (shellSequenceIndex >= PlayingMode.SHELL_SEQUENCE.length) {
        playingMode.addExtraLife(enemy);
      } else {
        playingMode.addPoints(enemy,
            PlayingMode.SHELL_SEQUENCE[shellSequenceIndex++]);
      }
    }
    StaticKoopaShell shell = playingMode.intersectsStaticKoopaShell(this);
    if (shell != null) {
      shell.shelled(vx < 0);
      if (shellSequenceIndex >= PlayingMode.SHELL_SEQUENCE.length) {
        playingMode.addExtraLife(enemy);
      } else {
        playingMode.addPoints(shell,
            PlayingMode.SHELL_SEQUENCE[shellSequenceIndex++]);
      }
    }

    if (x < playingMode.cameraX - 512
        || x > playingMode.cameraX + 1024 || y > 480) {
      remove = true;
      return;
    }
  }

  @Override
  public boolean blast(int px, int py, boolean left) {
    if (type != KoopaTroopaTypes.BUZZY_BEETLE && withinMargin(px, py)) {
      playingMode.others.add(
          new UpsideDownDeadKoopaShell(x, y, left ? -4 : 4, type));
      playingMode.addPoints(this, 200);
      remove = true;
      return true;
    }
    return false;
  }

  @Override
  public void starred(boolean left) {
    shelled(left);
    playingMode.addPoints(this, 200);
  }

  @Override
  public void shelled(boolean left) {
    playingMode.others.add(
        new UpsideDownDeadKoopaShell(x, y, left ? -4 : 4, type));
    remove = true;
  }

  @Override
  public boolean hurtsMario() {
    return !remove
        && ((vx < 0 && playingMode.mario.x <= x)
            || (vx > 0 && playingMode.mario.x >= x))
        && intersects(playingMode.mario, true);
  }

  @Override
  public void crush() {
    remove = true;
    playingMode.shells.add(new StaticKoopaShell(x, y, 0, 0, type));
    playingMode.addStompPoints(this);
  }

  public void render() {
    g.translate(x - playingMode.cameraX, y + 4);
    g.setColor(color);
    g.drawArc(2, 0, 27, 30, 0, 180);
    g.setColor(Color.WHITE);
    g.drawLine(0, 16, 5, 16);
    g.drawLine(26, 16, 31, 16);
    g.drawArc(5, 7, 21, 20, 0, -180);
    g.translate(playingMode.cameraX - x, -y - 4);
  }
}
