package mario;

public class Mushroom extends WorldObject {

  public boolean big;
  public int rising = 32;

  public Mushroom(int mapX, int mapY, boolean big) {
    bx1 = 6;
    by1 = 0;
    bx2 = 17;
    by2 = 31;

    fx = x = mapX << 5;
    fy = y = mapY << 5;
    vx = 1;

    this.big = big;
  }

  public void update() {

    if (remove) {
      return;
    }

    if (rising > 0) {
      rising--;
      fy--;
      y = (int)fy;
    } else {      
      for(int i = playingMode.platforms.size() - 1; i >= 0; i--) {
        AbstractPlatform platform = playingMode.platforms.get(i);
        if (platform.deadly && platform.intersects(this)) {
          vx = platform.x < x ? 1 : -1;
          vy = -9f;
          while(platform.intersects(this)) {            
            y--;
            fy = y;
          }
          return;
        }
      }

      applyGravity(true);

      if (!translateX(vx, false)) {
        vx = -vx;
      }

      if (x < playingMode.cameraX - 256
          || x > playingMode.cameraX + 576 || y > 480) {
        remove = true;
        return;
      }

      if (playingMode.mario.intersects(this)) {
        consume();
      }
    }
  }

  private void consume() {
    remove = true;
    if (big) {
      playingMode.addPoints(this, 1000);
      playingMode.mario.collectMushroom();
    } else {
      playingMode.addExtraLife(this);
    }
  }

  public void render() {
    g.translate(x - playingMode.cameraX, y);
    if (rising > 0) {
      g.setClip(0, 0, 32, 32 - rising);
    }
    g.setColor(Palette.MUSHROOM_BOTTOM);
    g.drawRect(8, 21, 15, 10);
    g.setColor(Palette.MUSHROOM_TOP);
    g.drawArc(0, 0, 31, 25, 0, 225);
    g.drawArc(0, 0, 31, 25, 0, -45);
    g.drawLine(5, 21, 26, 21);
    if (big) {
      g.setColor(Palette.MUSHROOM_BIG_TEXT);
      Text.drawString(4, 10, "BIG", 6, 2);
    } else {
      g.setColor(Palette.MUSHROOM_1UP_TEXT);
      Text.drawString(4, 10, "1UP", 6, 2);
    }
    if (rising > 0) {
      g.setClip(null);
    }
    g.translate(playingMode.cameraX - x, -y);
  }
  
}
