package mario;

public class PiranhaPlant extends WorldObject {

  public static final int PAUSE_TIME = 2 * 60;

  public static final int STATE_BELOW = 0;
  public static final int STATE_MOVING_UP = 1;
  public static final int STATE_ABOVE = 2;
  public static final int STATE_MOVING_DOWN = 3;

  public int state = STATE_BELOW;
  public int count = PAUSE_TIME - 2;
  public int clipY;

  public PiranhaPlant(int x, int y, boolean mapCoordinates) {
    if (mapCoordinates) {
      x <<= 5;
      y <<= 5;
    }
    fx = this.x = x;
    fy = this.y = y;
    clipY = y - 46;

    mx1 = bx1 = 0;
    my1 = by1 = 0;
    mx2 = bx2 = 31;
    my2 = by2 = 43;

    supports = false;
  }

  @Override
  public void update() {
    switch(state) {
      case STATE_BELOW:
        if (Math.abs(playingMode.mario.x - x) > 64) {
          if (count < PAUSE_TIME) {
            count++;
          } else {
            count = 0;
            state = STATE_MOVING_UP;
          }
        }
        break;
      case STATE_MOVING_UP:
        if (count < 45) {
          count++;
          y--;
          fy = y;
        } else {
          count = 0;
          state = STATE_ABOVE;
        }
        break;
      case STATE_ABOVE:
        if (count < PAUSE_TIME) {
          count++;
        } else {
          count = 0;
          state = STATE_MOVING_DOWN;
        }
        break;
      case STATE_MOVING_DOWN:
        if (count < 45) {
          count++;
          y++;
          fy = y;
        } else {
          count = 0;
          state = STATE_BELOW;
        }
        break;
    }
  }

  @Override
  public boolean blast(int px, int py, boolean left) {
    if (withinMargin(px, py)) {
      playingMode.addPoints(this, 200);
      remove = true;
      return true;
    }
    return false;
  }

  @Override
  public void starred(boolean left) {
    shelled(left);    
  }

  @Override
  public void shelled(boolean left) {
    playingMode.addPoints(this, 200);
    remove = true;
  }

  @Override
  public boolean hurtsMario() {
    switch(state) {
      case STATE_BELOW:
        return false;
      case STATE_MOVING_UP:
        if (count < 22) {
          return false;
        }
        break;
      case STATE_MOVING_DOWN:
        if (count >= 22) {
          return false;
        }
        break;
    }
    return !remove && intersects(playingMode.mario, true);
  }

  @Override
  public void render() {
    g.setClip(x - playingMode.cameraX, clipY, 32, 45);
    g.translate(x - playingMode.cameraX, y);
    g.setColor(Palette.PIRANHA_PLANT_HEAD);
    g.drawArc(0, -3, 31, 32, 120, 300);
    g.drawLine(7, 0, 15, 25);
    g.drawLine(24, 0, 16, 25);
    g.setColor(Palette.PIRANHA_PLANT_STEM);
    g.fillRect(14, 30, 4, 14);
    g.drawLine(0, 30, 13, 43);
    g.drawLine(31, 30, 18, 43);
    g.translate(playingMode.cameraX - x, -y);
    g.setClip(null);
  }

}
