package mario;

import java.awt.*;
import java.util.*;

public class PlayingMode implements IMode {

  public static final int[] STOMP_SEQUENCE
      = { 100, 200, 400, 500, 800, 1000, 2000, 4000, 5000, 8000 };
  public static final int[] SHELL_SEQUENCE
      = { 500, 800, 1000, 2000, 4000, 5000, 8000 };

  public static final int FLAG_POLE_STATE_NONE = 0;
  public static final int FLAG_POLE_STATE_SLIDING = 1;
  public static final int FLAG_POLE_STATE_REVERSING = 2;
  public static final int FLAG_POLE_STATE_WALKING = 3;
  public static final int FLAG_POLE_STATE_POINTS = 4;
  public static final int FLAG_POLE_STATE_FIREWORKS = 5;
  public static final int FLAG_POLE_STATE_FIREWORK_DONE = 6;
  public static final int FLAG_POLE_STATE_FINAL_DELAY = 7;

  public static final int AXE_STATE_NONE = 0;
  public static final int AXE_STATE_STONES_BREAKING = 1;
  public static final int AXE_STATE_BOWSER_DROPPING = 2;
  public static final int AXE_STATE_MARIO_WALKING = 3;
  public static final int AXE_STATE_MESSAGE_1 = 4;
  public static final int AXE_STATE_MESSAGE_2 = 5;

  public static final int STONE_BROWN = 0;
  public static final int STONE_UNDERGROUND = 1;
  public static final int STONE_CASTLE = 2;
  public static final int STONE_CLOUD = 3;
  public static final int STONE_WATER = 4;
  public static final int STONE_MUSHROOM = 5;

  public Main main;
  public Graphics g;
  public Mario mario;

  public int[][] displayMap;
  public int[][] physicsMap;
  public int[][] enemiesMap;
  public int[][] valuesMap;
  public int[] suction;
  public int mapWidth;
  public int cameraX;
  public int maxCameraX;
  public ArrayList<AbstractPlatform> platforms 
      = new ArrayList<AbstractPlatform>();
  public ArrayList<WorldObject> enemies
      = new ArrayList<WorldObject>();
  public ArrayList<WorldObject> others
      = new ArrayList<WorldObject>();
  public ArrayList<WorldObject> items
      = new ArrayList<WorldObject>();
  public ArrayList<StaticKoopaShell> shells
      = new ArrayList<StaticKoopaShell>();
  public int enemiesMapX = 0;
  public int timer;
  public int timerIncrementor;
  public Color[] questionmarkColors = new Color[64];
  public int questionmarkColorsIndex;
  public int score;
  public int stompSequenceIndex;
  public int coins;
  public int lives;
  public int world;
  public int subworld;
  public boolean isBonusArea;
  public int bonusAreaIndex;
  public int fireballCount;
  public int flagPoleX;
  public int flagPoleState = FLAG_POLE_STATE_NONE;
  public int fireworksCount;
  public String flagPoints;
  public Flag flag;
  public Random random;
  public int stoneIndex;
  public Segment segment;
  public boolean freeEntry;
  public int entryIndex;
  public boolean spittingFire;
  public int spitFireDelay;
  public Bowser bowser;
  public int axeState = AXE_STATE_NONE;
  public int axeBlocksRemaining;
  public int axeBlockMapX;
  public int axeDelay;
  public boolean flyingCheepCheepSpawning;
  public int availableFlyingCheepCheeps = 3;
  public int flyingCheepCheepDelay;
  public Lakitu lakitu;
  public Maze maze;
  public boolean spawningBulletBills;
  public int spawningBulletBillsDelay;
  public boolean hardMode;
  public boolean showTimer = true;
  public boolean startAtHalfWayPoint;
  public String name;
  public int halfWayX = Integer.MAX_VALUE;
  public boolean passedHalfwayPoint;
  public IInput input;
  public boolean pauseCleared;
  public boolean paused;
  public int flagPoleDelay;
  public boolean demoMode;

  public PlayingMode(Segment segment, boolean freeEntry, int entryIndex,
      boolean copyFields) {

    main = Main.main;
    g = main.g;
    random = main.random;

    world = segment.world;
    subworld = segment.subworld;
    isBonusArea = segment.isBonusArea;
    bonusAreaIndex = segment.bonusAreaIndex;
    this.segment = segment;
    this.freeEntry = freeEntry;
    this.entryIndex = entryIndex;
    stoneIndex = segment.stoneIndex;

    if (segment.hasHalfWayPoint) {
      halfWayX = segment.halfWayPointX;
    }

    if (Main.playingMode != null) {
      mario = Main.playingMode.mario;
    }
    if (copyFields) {
      if (Main.playingMode != null) {
        lives = Main.playingMode.lives;
        score = Main.playingMode.score;
        coins = Main.playingMode.coins;
        timer = Main.playingMode.timer;        
      }
    } else {
      timer = segment.timeLimit;
      lives = main.lives;
      score = main.score;
      coins = main.coins;
    }

    mapWidth = segment.width;
    displayMap = copyMap(segment.displayMap, segment.width);
    physicsMap = copyMap(segment.physicsMap, segment.width);
    enemiesMap = copyMap(segment.enemiesMap, segment.width);
    valuesMap = copyMap(segment.valuesMap, segment.width);
    suction = copyMap(segment.suction, segment.width);
    flagPoleX = segment.flagPoleX;
    maxCameraX = Math.min((segment.width - 16) << 5, segment.maxCameraX);
    if (freeEntry && entryIndex == 0) {
      startAtHalfWayPoint = main.passedHalfwayPoint;
    }

    for(int i = 0; i < questionmarkColors.length; i++) {
      double v = Math.sin(Math.PI * i / (double)questionmarkColors.length);
      questionmarkColors[i] = Palette.QUESTIONMARKS[0];
      if (v < 0.3) {
        questionmarkColors[i] = Palette.QUESTIONMARKS[2];
      } else if (v < 0.6) {
        questionmarkColors[i] = Palette.QUESTIONMARKS[1];
      }
    }
  }

  @Override
  public void init() {

    boolean vineEntry = false;
    Main.playingMode = this;
    hardMode = main.hardMode;

    if (mario != null) {
      Mario tempMario = mario;
      mario = new Mario();
      mario.big = tempMario.big;
      mario.hasFlower = tempMario.hasFlower;
      if (mario.big) {
        mario.my2 = mario.by2 = 63;
      }
    } else {
      mario = new Mario();
    }
    if (segment.isBonusArea && segment.bonusAreaIndex == 13) {
      input = mario.input = new TransitionInput();
      showTimer = false;
    } else {
      input = mario.input = demoMode ? main.inputPlayer : new KeyboardInput();
      mario.demoMode = demoMode;
    }
    if (stoneIndex == STONE_WATER) {
      mario.swimming = true;
    }
    if (freeEntry) {
      if (startAtHalfWayPoint && segment.hasHalfWayPoint) {
        mario.fx = mario.x = segment.halfWayPointX;
        mario.fy = mario.y = segment.halfWayPointY;
      } else {
        mario.fx = mario.x = segment.freeEntryPoints[entryIndex][0];
        mario.fy = mario.y = segment.freeEntryPoints[entryIndex][1];
      }
      if (mario.y == 448) {
        vineEntry = true;
      }
      if (mario.big) {
        mario.y -= 32;
        mario.fy = (int)mario.y;        
      }
    } else {
      mario.piping = true;
      mario.pipingCount = mario.big ? 0 : 32;
      mario.fx = mario.x = segment.pipeEntryPoints[entryIndex][0];
      mario.fy = mario.y = segment.pipeEntryPoints[entryIndex][1];
      mario.pipingType = segment.pipeEntryPoints[entryIndex][2];
      switch(mario.pipingType) {
        case Mario.PIPING_DOWN:
          mario.y -= mario.big ? 64 : 32;
          mario.fy = mario.y;
          break;
        case Mario.PIPING_UP:
          mario.y += 32;
          mario.fy = mario.y;
          mario.pipingX = mario.x;
          mario.pipingY = mario.big ? mario.y - 64 : mario.y - 32;
          break;
        case Mario.PIPING_LEFT:
          mario.x += 32;
          mario.fx = mario.x;
          break;
        case Mario.PIPING_RIGHT:
          mario.x -= 32;
          mario.fx = mario.x;
          break;
      }
    }

    if (vineEntry) {
      mario.vining = true;
      mario.y += mario.big ? 64 : 32;
      mario.fy = mario.y;
      others.add(new Vine(mario.x, mario.y, false, true));
    }

    if (main.luigiMode) {
      mario.color = Palette.LUIGI;
      name = "LUIGI";
    } else {
      mario.color = Palette.MARIO;
      name = "MARIO";
    }

    int nextCameraX = mario.x - 240;
    if (nextCameraX > maxCameraX) {
      nextCameraX = maxCameraX;
    }
    if (nextCameraX > cameraX) {
      cameraX = nextCameraX;
    }

    enemiesMapX = Math.max(cameraX >> 5, 0);
  }

  private int[] copyMap(int[] source, int width) {
    width += 16;
    int[] destination = new int[width];
    System.arraycopy(source, 0, destination, 0, width);
    return destination;
  }

  private int[][] copyMap(int[][] source, int width) {
    width += 16;
    int[][] destination = new int[15][width];
    for(int y = 0; y < 15; y++) {
      System.arraycopy(source[y], 0, destination[y], 0, width);
    }
    return destination;
  }

  public void update() {

    boolean isPause = input.isPause();

    if (pauseCleared) {
      if (isPause) {
        pauseCleared = false;
        paused = !paused;
      }
    } else {
      if (!isPause) {
        pauseCleared = true;
      } else {
        return;
      }
    }

    if (paused) {
      return;
    }

    if (++questionmarkColorsIndex == 64) {
      questionmarkColorsIndex = 0;
    }

    for(int targetEnemiesMapX = (cameraX >> 5) + 17;
        enemiesMapX < targetEnemiesMapX; enemiesMapX++) {
      for(int i = 0; i < 15; i++) {
        switch(enemiesMap[i][enemiesMapX]) {
          case EnemyTypes.GOOMBA_1:
            if (hardMode) {
              enemies.add(new KoopaTroopa(enemiesMapX, i, true,
                  KoopaTroopaTypes.BUZZY_BEETLE));
            } else {
              enemies.add(new Goomba(enemiesMapX, i, true));
            }
            break;
          case EnemyTypes.GOOMBA_2:
            if (hardMode) {
              enemies.add(new KoopaTroopa(enemiesMapX, i, true,
                  KoopaTroopaTypes.BUZZY_BEETLE));
              enemies.add(new KoopaTroopa(enemiesMapX + 2, i, true,
                  KoopaTroopaTypes.BUZZY_BEETLE));
            } else {
              enemies.add(new Goomba(enemiesMapX, i, true));
              enemies.add(new Goomba(enemiesMapX + 2, i, true));
            }
            break;
          case EnemyTypes.GOOMBA_2_2:
            if (hardMode) {
              enemies.add(new KoopaTroopa(enemiesMapX, i, true,
                  KoopaTroopaTypes.BUZZY_BEETLE));
              enemies.add(new KoopaTroopa(enemiesMapX + 3, i, true,
                  KoopaTroopaTypes.BUZZY_BEETLE));
            } else {
              enemies.add(new Goomba(enemiesMapX, i, true));
              enemies.add(new Goomba(enemiesMapX + 3, i, true));
            }
            break;
          case EnemyTypes.GOOMBA_3:
            if (hardMode) {
              enemies.add(new KoopaTroopa(enemiesMapX, i, true,
                  KoopaTroopaTypes.BUZZY_BEETLE));
              enemies.add(new KoopaTroopa(enemiesMapX + 2, i, true,
                  KoopaTroopaTypes.BUZZY_BEETLE));
              enemies.add(new KoopaTroopa(enemiesMapX + 4, i, true,
                  KoopaTroopaTypes.BUZZY_BEETLE));
            } else {
              enemies.add(new Goomba(enemiesMapX, i, true));
              enemies.add(new Goomba(enemiesMapX + 2, i, true));
              enemies.add(new Goomba(enemiesMapX + 4, i, true));
            }
            break;
          case EnemyTypes.GOOMBA_PAIR:
            if (hardMode) {
              enemies.add(new KoopaTroopa(enemiesMapX << 5, i << 5, false,
                  KoopaTroopaTypes.BUZZY_BEETLE));
              enemies.add(new KoopaTroopa((enemiesMapX << 5) + 33, i << 5,
                  false, KoopaTroopaTypes.BUZZY_BEETLE));
            } else {
              enemies.add(new Goomba(enemiesMapX << 5, i << 5, false));
              enemies.add(new Goomba((enemiesMapX << 5) + 33, i << 5, false));
            }
            break;
          case EnemyTypes.BUZZY_BEETLE:
            enemies.add(new KoopaTroopa(enemiesMapX, i, true,
                KoopaTroopaTypes.BUZZY_BEETLE));
            break;
          case EnemyTypes.BUZZY_BEETLE_DOUBLE:
            enemies.add(new KoopaTroopa(enemiesMapX, i, true,
                KoopaTroopaTypes.BUZZY_BEETLE));
            enemies.add(new KoopaTroopa(enemiesMapX + 2, i, true,
                KoopaTroopaTypes.BUZZY_BEETLE));
            break;
          case EnemyTypes.BUZZY_BEETLE_TRIPLE:
            enemies.add(new KoopaTroopa(enemiesMapX << 5, i << 5, false,
                KoopaTroopaTypes.BUZZY_BEETLE));
            enemies.add(new KoopaTroopa((enemiesMapX << 5) + 33, i << 5, false,
                KoopaTroopaTypes.BUZZY_BEETLE));
            enemies.add(new KoopaTroopa((enemiesMapX << 5) + 66, i << 5, false,
                KoopaTroopaTypes.BUZZY_BEETLE));
            break;
          case EnemyTypes.GREEN_KOOPA_TROOPA_1:
            enemies.add(new KoopaTroopa(enemiesMapX, i, true, 
                KoopaTroopaTypes.GREEN));
            break;
          case EnemyTypes.GREEN_KOOPA_TROOPA_2:
            enemies.add(new KoopaTroopa(enemiesMapX, i, true, 
                KoopaTroopaTypes.GREEN));
            enemies.add(new KoopaTroopa(enemiesMapX + 2, i, true, 
                KoopaTroopaTypes.GREEN));
            break;
          case EnemyTypes.GREEN_KOOPA_TROOPA_3:
            enemies.add(new KoopaTroopa(enemiesMapX, i, true, 
                KoopaTroopaTypes.GREEN));
            enemies.add(new KoopaTroopa(enemiesMapX + 2, i, true, 
                KoopaTroopaTypes.GREEN));
            enemies.add(new KoopaTroopa(enemiesMapX + 4, i, true, 
                KoopaTroopaTypes.GREEN));
            break;
          case EnemyTypes.RED_KOOPA_TROOPA_2:
            enemies.add(new KoopaTroopa(enemiesMapX, i, true, 
                KoopaTroopaTypes.RED));
            enemies.add(new KoopaTroopa(enemiesMapX + 2, i, true, 
                KoopaTroopaTypes.RED));
            break;
          case EnemyTypes.RED_KOOPA_TROOPA:
            enemies.add(new KoopaTroopa(enemiesMapX, i, true, 
                KoopaTroopaTypes.RED));
            break;
          case EnemyTypes.GREEN_KOOPA_PARATROOPA:
            enemies.add(new GreenKoopaParatroopa(enemiesMapX, i, true));
            break;
          case EnemyTypes.RED_KOOPA_PARATROOPA:
            enemies.add(new RedKoopaParatroopa(enemiesMapX, i, true));
            break;
          case EnemyTypes.HOVERING_KOOPA_PARATROOPA:
            enemies.add(new HoveringKoopaParatroopa(enemiesMapX, i, true));
            break;
          case EnemyTypes.PIRANHA_PLANT:
            enemies.add(new PiranhaPlant((enemiesMapX << 5) - 16,
                (i << 5) + 33, false));
            break;
          case EnemyTypes.FIREBAR_CLOCKWISE:
            items.add(new Firebar(enemiesMapX, i, true, true, 6));
            break;
          case EnemyTypes.FIREBAR_COUNTERCLOCKWISE:
            items.add(new Firebar(enemiesMapX, i, true, false, 6));
            break;
          case EnemyTypes.FIREBAR_LONG_CLOCKWISE:
            items.add(new Firebar(enemiesMapX, i, true, true, 12));
            break;
          case EnemyTypes.FLAG:
            others.add(new Flag(enemiesMapX));
            others.add(new Door(enemiesMapX + 6));
            break;
          case EnemyTypes.ELEVATOR_DOWN: {
            int width = valuesMap[i][enemiesMapX];
            if (hardMode && width > 2) {
              width--;
            }
            platforms.add(new ElevatorPlatform(
                enemiesMapX, i, true, 
                width << 5, 16, 1.5f));
            break;
          }
          case EnemyTypes.ELEVATOR_UP: {
            int width = valuesMap[i][enemiesMapX];
            if (hardMode && width > 2) {
              width--;
            }
            platforms.add(new ElevatorPlatform(
                enemiesMapX, i, true, 
                width << 5, 16, -1.5f));
            break;
          }
          case EnemyTypes.SKY_LIFT: {
            int width = valuesMap[i][enemiesMapX];
            if (hardMode && width > 2) {
              width--;
            }
            platforms.add(new SkyLiftPlatform(
                enemiesMapX, i, true,
                width << 5, 16));
            break;
          }
          case EnemyTypes.DROP_PLATFORM: {
            int width = valuesMap[i][enemiesMapX];
            if (hardMode && width > 2) {
              width--;
            }
            platforms.add(new DropPlatform(
                enemiesMapX, i, true,
                width << 5, 16));
            break;
          }
          case EnemyTypes.RECIPROCATING_TOP: {
            int width = valuesMap[i][enemiesMapX];
            if (hardMode && width > 2) {
              width--;
            }
            int j = i + 1;
            for(; j < 15; j++) {
              if (enemiesMap[j][enemiesMapX]
                  == EnemyTypes.RECIPROCATING_BOTTOM) {
                break;
              }
            }
            platforms.add(new ReciprocatingPlatform(i, j, enemiesMapX, true,
                false, width << 5, 16));
            break;
          }
          case EnemyTypes.RECIPROCATING_LEFT: {
            int width = valuesMap[i][enemiesMapX];
            if (hardMode && width > 2) {
              width--;
            }
            int j = enemiesMapX + 1;
            for(; j < enemiesMapX + 16; j++) {
              if (enemiesMap[i][j] == EnemyTypes.RECIPROCATING_RIGHT) {
                break;
              }
            }
            platforms.add(new ReciprocatingPlatform(enemiesMapX, j, i, true,
                true, width << 5, 16));
            break;
          }
          case EnemyTypes.JUMPING_BOARD:
            platforms.add(new JumpingBoard(enemiesMapX, i, true));
            break;
          case EnemyTypes.AXE:
            items.add(new Axe(enemiesMapX, i, true));
            break;
          case EnemyTypes.SPITTING_FIRE:
            spittingFire = true;
            break;
          case EnemyTypes.BOWSER:
            spittingFire = false;
            enemies.add(new Bowser(enemiesMapX, i, true));
            break;
          case EnemyTypes.TOAD:
            if (world == 8) {
              others.add(new Toad(enemiesMapX, i, true));
            } else {
              others.add(new Toad(enemiesMapX, i, false));
            }
            break;
          case EnemyTypes.CHEEP_CHEEP_SPAWNER:
            others.add(new CheepCheepSpawner());
            break;
          case EnemyTypes.BLOOBER:
            enemies.add(new Bloober(enemiesMapX, i, true));
            break;
          case EnemyTypes.FLYING_CHEEP_CHEEPS_ON:
            flyingCheepCheepSpawning = true;
            break;
          case EnemyTypes.FLYING_CHEEP_CHEEPS_OFF:
            flyingCheepCheepSpawning = false;
            break;
          case EnemyTypes.WARP_ZONE:
            if (!hardMode) {
              others.add(new WarpZoneText(
                  enemiesMapX, i, true, valuesMap[i][enemiesMapX]));
            }
            break;
          case EnemyTypes.PODOBOO:
            enemies.add(new Podoboo(enemiesMapX));
            break;
          case EnemyTypes.HAMMER_BROTHER:
            enemies.add(new HammerBrother(enemiesMapX, i, true));
            break;
          case EnemyTypes.SCALE_START:
            others.add(new Scale(enemiesMapX, i));
            break;
          case EnemyTypes.LAKITU_START:
            if (lakitu == null) {
              lakitu = new Lakitu();
              enemies.add(lakitu);
            }
            break;
          case EnemyTypes.LAKITU_END:
            if (lakitu != null) {
              lakitu.exit();
            }
            break;
          case EnemyTypes.MAZE:
            if (maze == null) {
              maze = new Maze(world);
              others.add(maze);
            }
            break;
          case EnemyTypes.WRONG_WAY:
            items.add(new WrongWay(enemiesMapX, i, true));
            break;
          case EnemyTypes.TURTLE_CANNON:
            items.add(new TurtleCannon(enemiesMapX, i, true));
            break;
          case EnemyTypes.BULLET_BILLS_SPAWNING_START:
            spawningBulletBills = true;
            break;
          case EnemyTypes.BULLET_BILLS_SPAWNING_END:
            spawningBulletBills = false;
            break;
          case EnemyTypes.DOOR:
            others.add(new Door(enemiesMapX));
            break;
        }
      }
    }

    if (spittingFire) {
      if (spitFireDelay <= 0) {
        spitFireDelay = 240;
        if (random.nextInt(6) == 0) {
          spitFireDelay >>= 1;
        }
        enemies.add(new SpitFire(cameraX + 512, 
          8 + ((random.nextInt(144) + 160) & 0xFFFFFFE0)));
      } else {
        spitFireDelay--;
      }
    }

    if (spawningBulletBills) {
      if (spawningBulletBillsDelay <= 0) {
        spawningBulletBillsDelay = 240;
        enemies.add(new BulletBill(cameraX + 512,
            (2 + random.nextInt(11)) << 5, false,
            -TurtleCannon.BULLET_SPEED, false, 0, 0));
      } else {
        spawningBulletBillsDelay--;
      }
    }

    mario.onVine = false;
    for(int i = others.size() - 1; i >= 0; i--) {
      WorldObject other = others.get(i);
      if (other.remove) {
        others.remove(i);
      } else {
        other.update();
        if (other.remove) {
          others.remove(i);
        }
      }
    }

    if (mario.isPlaying()) {
      if (flagPoleState == FLAG_POLE_STATE_NONE 
          && axeState == AXE_STATE_NONE
          && ++timerIncrementor > 25) {
        timerIncrementor = 0;
        timer--;
        if (timer <= 0) {
          timer = 0;
          mario.timeUp();
        }
      }
      
      for(int i = items.size() - 1; i >= 0; i--) {
        WorldObject item = items.get(i);
        if (item.remove) {
          items.remove(i);
        } else {
          item.update();
          if (item.remove) {
            items.remove(i);
          }
        }
      }

      for(int i = enemies.size() - 1; i >= 0; i--) {
        WorldObject enemy = enemies.get(i);
        if (enemy.remove) {
          enemies.remove(i);
        } else {
          enemy.update();
          if (enemy.remove) {
            enemies.remove(i);
          }
        }
      }

      for(int i = shells.size() - 1; i >= 0; i--) {
        WorldObject shell = shells.get(i);
        if (shell.remove) {
          shells.remove(i);
        } else {
          shell.update();
          if (shell.remove) {
            shells.remove(i);
          }
        }
      }
    }

    if (flagPoleState == FLAG_POLE_STATE_NONE) {
      if (axeState == AXE_STATE_NONE) {
        mario.update();
      } else {
        handleAxe();
      }
    } else {
      handleFlagPole();
    }

    if (flyingCheepCheepSpawning && availableFlyingCheepCheeps > 0) {
      if (flyingCheepCheepDelay > 0) {
        flyingCheepCheepDelay--;
      } else {
        availableFlyingCheepCheeps--;
        flyingCheepCheepDelay = 15 + random.nextInt(60);
        enemies.add(new FlyingCheepCheep());
      }
    }

    if (mario.isPlaying()) {
      mario.onJumpingBoard = false;
      for(int i = platforms.size() - 1; i >= 0; i--) {
        AbstractPlatform platform = platforms.get(i);
        if (platform.remove) {
          platforms.remove(i);
        } else {
          platform.update();
          if (platform.remove) {
            platforms.remove(i);
          }
        }
      }
    }

    if (mario.x < cameraX) {
      mario.fx = mario.x = cameraX;
    }

    if (mario.x >= halfWayX && !passedHalfwayPoint) {
      passedHalfwayPoint = true;
      main.passedHalfwayPoint();
    }

    if (axeState == AXE_STATE_NONE) {
      int nextCameraX = mario.x - 240;
      if (nextCameraX > maxCameraX) {
        nextCameraX = maxCameraX;
      }
      if (nextCameraX > cameraX) {
        cameraX = nextCameraX;
      }
    }
  }

  public void resetStompPoints() {
    stompSequenceIndex = 0;
  }

  public void addExtraLife(WorldObject object) {
    others.add(new Points(object, "1UP"));
    if (lives < 100) {
      lives++;
    }
  }

  public void addPoints(WorldObject object, int points) {
    others.add(new Points(object, points));
  }

  public void addStompPoints(WorldObject object) {
    if (stompSequenceIndex >= STOMP_SEQUENCE.length) {
      addExtraLife(object);
    } else {
      others.add(new Points(object, STOMP_SEQUENCE[stompSequenceIndex++]));
    }
  }

  private void drawStatusBar() {
    g.setColor(Color.WHITE);
    Text.drawString(48, 32, name);
    Text.drawNumber(48, 48, score, 6);
    Text.drawChar(192, 48, '~');
    Text.drawNumber(208, 48, coins, 2);
    Text.drawString(288, 32, "WORLD");
    Text.drawNumber(304, 48, world, 1);
    Text.drawChar(320, 48, '-');
    Text.drawNumber(336, 48, subworld, 1);
    Text.drawString(400, 32, "TIME");
    if (showTimer) {
      Text.drawNumber(416, 48, timer, 3);
    }

    g.setColor(questionmarkColors[questionmarkColorsIndex]);
    g.drawOval(180, 48, 8, 14);
  }

  public void fellInPit() {
    if (stoneIndex == STONE_CLOUD) {
      main.enterPipe(world, subworld, isBonusArea, bonusAreaIndex, -1);
    } else {
      mario.remove = true;
    }
  }

  private void removeAll(ArrayList<? extends WorldObject> worldObjects) {
    for(int i = worldObjects.size() - 1; i >= 0; i--) {
      worldObjects.get(i).remove = true;
    }
  }


  public void hitAxe(int mapX, int mapY) {

    mario.stunned = 0;

    physicsMap[mapY][mapX + 1] = BlockTypes.EMPTY;
    physicsMap[mapY - 1][mapX + 1] = BlockTypes.EMPTY;
    physicsMap[mapY - 2][mapX + 1] = BlockTypes.EMPTY;
    physicsMap[mapY][mapX + 2] = BlockTypes.EMPTY;
    physicsMap[mapY - 1][mapX + 2] = BlockTypes.EMPTY;
    physicsMap[mapY - 2][mapX + 2] = BlockTypes.EMPTY; 

    boolean removeBowser = bowser.remove;

    removeAll(enemies);
    removeAll(items);
    removeAll(others);
    removeAll(platforms);
    removeAll(shells);

    if (removeBowser) {
      axeState = AXE_STATE_MARIO_WALKING;
      maxCameraX += 512;
    } else {
      axeState = AXE_STATE_STONES_BREAKING;
      bowser.remove = false;
      axeBlocksRemaining = 13;
      axeBlockMapX = mapX - 1;  
    }
  }

  private void handleAxe() {
    switch(axeState) {
      case AXE_STATE_STONES_BREAKING:
        mario.applyGravity(true);
        if (axeBlocksRemaining > 0) {         
          if (axeDelay <= 0) {
            axeBlocksRemaining--;
            displayMap[10][axeBlockMapX] = BlockTypes.EMPTY;
            physicsMap[10][axeBlockMapX] = BlockTypes.EMPTY;
            int X = axeBlockMapX << 5;
            others.add(new Brick(X, 320, -4, -10));
            others.add(new Brick(X + 16, 320, 4, -10));
            others.add(new Brick(X, 336, -4.5f, -7));
            others.add(new Brick(X + 16, 336, 4.5f, -7));
            axeBlockMapX--;
            axeDelay = 8;
          } else {
            axeDelay--;
          }
        } else {
          axeState = AXE_STATE_BOWSER_DROPPING;
        }
        break;
      case AXE_STATE_BOWSER_DROPPING:
        bowser.applyGravity(true);
        mario.applyGravity(true);
        if (bowser.y > 500) {
          axeState = AXE_STATE_MARIO_WALKING;
          maxCameraX += 512;
        }
        break;
      case AXE_STATE_MARIO_WALKING:
        mario.applyGravity(true);
        mario.translateX(2, true);
        if (cameraX < maxCameraX) {
          cameraX += 2;
        } else {
          axeDelay = 2 * 60;
          axeState = AXE_STATE_MESSAGE_1;
        }
        break;
      case AXE_STATE_MESSAGE_1:
        if (--axeDelay == 0) {
          axeDelay = 4 * 60;
          axeState = AXE_STATE_MESSAGE_2;
        }
        break;
      case AXE_STATE_MESSAGE_2:
        if (--axeDelay == 0) {
          main.subworldCompleted();
        }
        break;
    }
  }

  public void hitFlagPole() {

    fireworksCount = timer % 10;
    if (fireworksCount != 1 && fireworksCount != 3 && fireworksCount != 6) {
      fireworksCount = 0;
    }

    if (mario.y < 4 * 32) {
      score += 5000;
      flagPoints = "5000";
    } else if (mario.y < 7 * 32) {
      score += 2000;
      flagPoints = "2000";
    } else if (mario.y < 9 * 32) {
      score += 800;
      flagPoints = "800";
    } else if (mario.y < 11 * 32) {
      score += 400;
      flagPoints = "400";
    } else {
      score += 100;
      flagPoints = "100";
    }

    flagPoleState = FLAG_POLE_STATE_SLIDING;
    mario.starCount = 0;
    mario.vy = 0;
    mario.vx = 0;
    for(int i = enemies.size() - 1; i >= 0; i--) {
      WorldObject enemy = enemies.get(i);
      if (!enemy.remove) {
        enemy.starred(false);
      }
    }
    for(int i = items.size() - 1; i >= 0; i--) {
      WorldObject item = items.get(i);
      item.remove = true;
    }
    for(int i = shells.size() - 1; i >= 0; i--) {
      StaticKoopaShell shell = shells.get(i);
      shell.remove = true;
    }
  }

  public void handleFlagPole() {
    switch(flagPoleState) {
      case FLAG_POLE_STATE_SLIDING:
        int marioMaxY = mario.big ? 320 : 352;
        for(int i = 0; i < 3 && mario.y < marioMaxY; i++) {
          mario.fy = ++mario.y;
        }
        for(int i = 0; i < 4 && flag.y < 342; i++) {          
          flag.fy = ++flag.y;
        }
        if (flag.y == 342) {
          flagPoleState = FLAG_POLE_STATE_REVERSING;
        }
        break;
      case FLAG_POLE_STATE_REVERSING:
        if (mario.x < flagPoleX + 2) {
          mario.fx = ++mario.x;
        } else {
          flagPoleState = FLAG_POLE_STATE_WALKING;
        }
        break;
      case FLAG_POLE_STATE_WALKING: {
        if (mario.x < flagPoleX + 177) {
          mario.x += 2;
          mario.fx = mario.x;
        } else {
          flagPoleState = FLAG_POLE_STATE_POINTS;
        }
        mario.vy += mario.G;
        mario.fy += mario.vy;
        mario.y = (int)mario.fy;
        int maxY = mario.x < flagPoleX + 16 ? 
            (mario.big ? 320 : 352) : (mario.big ? 352 : 384);
        if (mario.y >= maxY) {
          mario.vy = 0;
          mario.fy = mario.y = maxY;
        }
        break;
      }
      case FLAG_POLE_STATE_POINTS:
        if (timer > 0) {
          timer--;
          score += 50;
        } else {
          flagPoleState = FLAG_POLE_STATE_FIREWORK_DONE;
        }
        break;
      case FLAG_POLE_STATE_FIREWORK_DONE:
        if (fireworksCount > 0) {
          fireworksCount--;
          flagPoleState = FLAG_POLE_STATE_FIREWORKS;
          score += 500;
          others.add(new Firework(mario.x + 16
              + ((fireworksCount & 1) == 0 ? 1 : -1) * random.nextInt(112),
              96 + random.nextInt(128)));
        } else {
          flagPoleState = FLAG_POLE_STATE_FINAL_DELAY;
        }
        break;
      case FLAG_POLE_STATE_FINAL_DELAY:
        if (++flagPoleDelay == 60) {
          main.subworldCompleted();
        }
        break;
    }
  }

  public void collectCoin(WorldObject object) {
    coins++;
    if (coins >= 100) {
      coins = 0;
      addExtraLife(object);
    }
  }

  public void render() {

    g.setColor(Color.BLACK);
    g.fillRect(0, 0, 512, 480);

    int ax = cameraX >> 5;
    int bx = cameraX & 31;

    g.translate(-bx, 0);
    for(int y = 0; y < 15; y++) {
      for(int x = 0; x < 17; x++) {
        g.translate(x << 5, y << 5);
        switch(displayMap[y][ax + x]) {
          case BlockTypes.BRICK:
            g.setColor(Palette.STONES[stoneIndex]);
            g.drawRect(0, 0, 31, 31);
            break;
          case BlockTypes.SOLID:
            g.setColor(Palette.STONES[stoneIndex]);
            g.drawRect(0, 0, 31, 31);
            g.drawLine(1, 1, 30, 30);
            g.drawLine(30, 1, 1, 30);
            break;
          case BlockTypes.LEFT_PIPE_CAP:
            g.setColor(Palette.PIPE);
            g.drawLine(0, 0, 31, 0);
            g.drawLine(0, 0, 0, 31);
            g.drawLine(0, 31, 31, 31);
            break;
          case BlockTypes.LEFT_PIPE:
            g.setColor(Palette.PIPE);
            g.drawLine(3, 0, 3, 31);
            break;
          case BlockTypes.RIGHT_PIPE_CAP:
            g.setColor(Palette.PIPE);
            g.drawLine(0, 0, 31, 0);
            g.drawLine(31, 0, 31, 31);
            g.drawLine(0, 31, 31, 31);
            break;
          case BlockTypes.RIGHT_PIPE:
            g.setColor(Palette.PIPE);
            g.drawLine(28, 0, 28, 31);
            break;
          case BlockTypes.UP_PIPE_CAP:
            g.setColor(Palette.PIPE);
            g.drawLine(0, 0, 0, 31);
            g.drawLine(0, 0, 31, 0);
            g.drawLine(31, 0, 31, 31);
            break;
          case BlockTypes.UP_PIPE:
            g.setColor(Palette.PIPE);
            g.drawLine(0, 3, 31, 3);
            break;
          case BlockTypes.DOWN_PIPE_CAP:
            g.setColor(Palette.PIPE);
            g.drawLine(0, 0, 0, 31);
            g.drawLine(0, 31, 31, 31);
            g.drawLine(31, 0, 31, 31);
            break;
          case BlockTypes.DOWN_PIPE:
            g.setColor(Palette.PIPE);
            g.drawLine(0, 28, 31, 28);
            break;
          case BlockTypes.LEFT_UP_PIPE:
            g.setColor(Palette.PIPE);
            g.drawLine(3, 0, 3, 31);
            g.drawLine(0, 3, 3, 3);
            break;
          case BlockTypes.LEFT_DOWN_PIPE:
            g.setColor(Palette.PIPE);
            g.drawLine(3, 0, 3, 31);
            g.drawLine(0, 28, 3, 28);
            break;
          case BlockTypes.RIGHT_UP_PIPE:
            g.setColor(Palette.PIPE);
            g.drawLine(28, 0, 28, 31);
            g.drawLine(28, 3, 31, 3);
            break;
          case BlockTypes.RIGHT_DOWN_PIPE:
            g.setColor(Palette.PIPE);
            g.drawLine(28, 0, 28, 31);
            g.drawLine(28, 28, 31, 28);
            break;
          case BlockTypes.QUESTIONMARK:
            g.setColor(questionmarkColors[questionmarkColorsIndex]);
            g.drawRect(0, 0, 31, 31);
            Text.drawChar(8, 8, '?');
            break;
          case BlockTypes.FLAG_POLE_KNOB:
            g.setColor(Palette.FLAG_POLE);
            g.drawLine(15, 32, 15, 320);
            g.drawLine(16, 32, 16, 320);
            g.setColor(Palette.FLAG_POLE_KNOB);
            g.drawOval(8, 16, 15, 15);
            break;
          case BlockTypes.COIN:
            g.setColor(questionmarkColors[questionmarkColorsIndex]);
            g.drawOval(5, 2, 19, 27);
            break;
          case BlockTypes.TURTLE_CANNON:
            g.setColor(Palette.TURTLE_CANNON);
            g.drawRect(0, 0, 5, 31);
            g.drawRect(26, 0, 5, 31);
            g.drawLine(6, 2, 25, 2);
            g.drawLine(6, 29, 25, 29);
            break;
          case BlockTypes.TURTLE_CANNON_PILLAR:
            g.setColor(Palette.TURTLE_CANNON);
            g.drawLine(2, 0, 2, 31);
            g.drawLine(29, 0, 29, 31);
            break;
        }
        g.translate(-(x << 5), -(y << 5));
      }
    }
    if (stoneIndex == STONE_WATER) {
      g.setColor(Palette.WATER);
      for(int x = 0; x < 17; x++) {
        if (displayMap[2][ax + x] == BlockTypes.EMPTY) {
          int X = x << 5;
          g.drawLine(X, 64, X + 5, 74);
          g.drawLine(X + 5, 74, X + 15, 64);
          g.drawLine(X + 15, 64, X + 20, 74);
          g.drawLine(X + 20, 74, X + 31, 64);
        }
      }
    }
    g.translate(bx, 0);

    for(int i = others.size() - 1; i >= 0; i--) {
      WorldObject other = others.get(i);
      other.render();
    }

    for(int i = items.size() - 1; i >= 0; i--) {
      WorldObject item = items.get(i);
      item.render();
    }

    for(int i = platforms.size() - 1; i >= 0; i--) {
      AbstractPlatform platform = platforms.get(i);
      platform.render();
    }

    for(int i = enemies.size() - 1; i >= 0; i--) {
      WorldObject enemy = enemies.get(i);
      if (!enemy.remove) {
        enemy.render();
      }
    }

    for(int i = shells.size() - 1; i >= 0; i--) {
      WorldObject shell = shells.get(i);
      shell.render();
    }

    if (flagPoleState < FLAG_POLE_STATE_POINTS) {
      mario.render();
    }

    if (axeState == AXE_STATE_MESSAGE_1) {
      g.setColor(Color.WHITE);
      Text.drawString(128, 160,
          main.luigiMode ? "THANK YOU LUIGI!" : "THANK YOU MARIO!");
    } else if (axeState == AXE_STATE_MESSAGE_2) {
      g.setColor(Color.WHITE);
      Text.drawString(128, 160, 
          main.luigiMode ? "THANK YOU LUIGI!" : "THANK YOU MARIO!");
      if (world == 8) {
        Text.drawString(112, 224,  "YOUR QUEST IS OVER.");
        if (hardMode) {
          Text.drawString(80, 256, "YOU ARE A SUPER PLAYER!");
        } else {
          Text.drawString(48, 256, "YOU HAVE BEATEN EASY MODE.");
        }
      } else {
        Text.drawString(80, 224, "BUT OUR PRINCESS IS IN");
        Text.drawString(80, 256, "ANOTHER CASTLE!");
      }
    }

    drawStatusBar();

    if (paused) {
      g.setColor(Color.BLACK);
      g.fillRect(206, 230, 98, 18);
      g.setColor(Color.WHITE);
      Text.drawString(208, 232, "PAUSED");
    }
  }

  public WorldObject intersectsEnemy(WorldObject self) {
    for(int i = enemies.size() - 1; i >= 0; i--) {
      WorldObject enemy = enemies.get(i);
      if (enemy != self && enemy.intersects(self)) {
        return enemy;
      }
    }
    return null;
  }

  public StaticKoopaShell intersectsStaticKoopaShell(WorldObject self) {
    for(int i = shells.size() - 1; i >= 0; i--) {
      StaticKoopaShell shell = shells.get(i);
      if (shell != self && shell.intersects(self)) {
        return shell;
      }
    }
    return null;  
  }

  public WorldObject withEnemyMargin(WorldObject self, int x, int y) {
    for(int i = shells.size() - 1; i >= 0; i--) {
      StaticKoopaShell shell = shells.get(i);
      if (shell.withinMargin(x, y)) {
        return shell;
      }
    }
    for(int i = enemies.size() - 1; i >= 0; i--) {
      WorldObject enemy = enemies.get(i);
      if (enemy != self && enemy.withinMargin(x, y)) {
        return enemy;
      }
    }
    return null;
  }

  public void flyingCheepCheepDied() {
    availableFlyingCheepCheeps++;
  }
}
