package mario;

public class QuestionmarkPlatform extends AbstractPlatform {

  public static final int STEPS = 16;
  public static final float INCREMENT = (float)(Math.PI / STEPS);
  public static final float RADIUS = 8;

  public int mapX;
  public int mapY;
  public float t;
  public int steps = STEPS;
  public int originY;

  public QuestionmarkPlatform(int px, int py) {
    this.mapX = px >> 5;
    this.mapY = py >> 5;
    playingMode.physicsMap[mapY][mapX] = BlockTypes.EMPTY;
    playingMode.displayMap[mapY][mapX] = BlockTypes.EMPTY;
    x = mapX << 5;
    originY = y = mapY << 5;
    deadly = true;
    if (playingMode.valuesMap[mapY][mapX] == ItemTypes.COIN) {
      playingMode.others.add(new FlyingCoin(x, y - 32));
    }
  }

  @Override
  public void move() {
    if (--steps == 0) {
      playingMode.physicsMap[mapY][mapX] = BlockTypes.SOLID;
      playingMode.displayMap[mapY][mapX] = BlockTypes.SOLID;      
      remove = true;
      switch(playingMode.valuesMap[mapY][mapX]) {
        case ItemTypes.MUSHROOM:
          if (playingMode.mario.big) {
            playingMode.items.add(new Flower(mapX, mapY));
          } else {
            playingMode.items.add(new Mushroom(mapX, mapY, true));
          }
          break;
        case ItemTypes.EXTRA_LIFE:
          playingMode.items.add(new Mushroom(mapX, mapY, false));
          break;
        case ItemTypes.STARMAN:
          playingMode.items.add(new Starman(mapX, mapY));
          break;
        case ItemTypes.VINE:
          playingMode.others.add(new Vine(mapX, mapY, true, false));
          break;
      }
    } else {
      y = (int)(originY - RADIUS * Math.sin(t));
      t += INCREMENT;
    }
  }

  @Override
  public void render() {
    g.translate((int)x - playingMode.cameraX, (int)y);
    g.setColor(Palette.STONES[playingMode.stoneIndex]);
    g.drawRect(0, 0, 31, 31);
    g.drawLine(1, 1, 30, 30);
    g.drawLine(30, 1, 1, 30);
    g.translate(playingMode.cameraX - (int)x, -(int)y);
  }
}
