package mario;

import java.awt.*;

public class RedKoopaParatroopa extends WorldObject {

  public static final float ANGLE_INCREMENT = (float)(2.0 * Math.PI / (6 * 60));
  public static final float RADIUS = 3 * 32;

  public float centerY;
  public float angle;

  public RedKoopaParatroopa(
      int x, int y, boolean mapCoordinates) {

    if (mapCoordinates) {
      x <<= 5;
      y <<= 5;
    }

    this.fx = this.x = x;
    this.fy = this.y = y;

    centerY = y;

    fy = centerY + RADIUS * (float)Math.sin(angle);
    y = (int)fy;
  }

  public void update() {

    vy = (centerY + RADIUS * (float)Math.sin(angle)) - fy;
    if (!moveY(vy)) {
      return;
    }

    angle += ANGLE_INCREMENT;

    if (x < playingMode.cameraX - 64) {
      remove = true;
    }
  }

  @Override
  public boolean blast(int px, int py, boolean left) {
    if (withinMargin(px, py)) {
      playingMode.others.add(new UpsideDownDeadKoopaTroopa(
          x, y, left ? -4 : 4, KoopaTroopaTypes.RED));
      playingMode.addPoints(this, 200);
      remove = true;
      return true;
    }
    return false;
  }

  @Override
  public void starred(boolean left) {
    shelled(left);
    playingMode.addPoints(this, 200);
  }

  @Override
  public void shelled(boolean left) {
    playingMode.others.add(new UpsideDownDeadKoopaTroopa(
        x, y, left ? -4 : 4, KoopaTroopaTypes.RED));
    playingMode.addPoints(this, 200);
    remove = true;
  }

  @Override
  public void crush() {
    remove = true;
    playingMode.enemies.add(new KoopaTroopa(x, y, false, KoopaTroopaTypes.RED));
    playingMode.addStompPoints(this);
  }

  @Override
  public boolean hurtsMario() {
    return !remove && intersects(playingMode.mario, true);
  }

  public void render() {
    g.translate(x - playingMode.cameraX, y);
    g.setColor(Palette.KOOPA_TROOPA_LEG);
    g.drawLine(3, 17, 3, 31);
    g.drawLine(28, 17, 28, 31);
    g.drawLine(3, 31, 12, 31);
    g.drawLine(12, 27, 12, 31);
    g.drawLine(19, 27, 19, 31);
    g.drawLine(19, 31, 28, 31);
    g.setColor(Palette.RED_KOOPA_TROOPA);
    g.drawArc(2, 0, 27, 30, 0, 180);
    g.setColor(Color.WHITE);
    g.drawLine(0, 16, 5, 16);
    g.drawLine(26, 16, 31, 16);
    g.drawArc(5, 7, 21, 20, 0, -180);
    g.drawLine(0, 0, 8, 8);
    g.drawLine(31, 0, 23, 8);
    g.translate(playingMode.cameraX - x, -y);
  }
}
