package mario;

public class Scale extends WorldObject {

  public ScalePlatform scalePlatform1;
  public ScalePlatform scalePlatform2;
  public int halfWidth;
  public int horizontalWidth;

  public Scale(int mapX, int mapY) {

    int width = playingMode.valuesMap[mapY][mapX];
    if (playingMode.hardMode && width > 2) {
      width--;
    }
    width <<= 5;
    int x1 = 0;
    int x2 = 0;
    int y1 = 0;
    int y2 = 0;

    halfWidth = width >> 1;

    for(int i = mapX + 1; i < mapX + 16; i++) {
      if (playingMode.enemiesMap[mapY][i] == EnemyTypes.SCALE_END) {
        x1 = i;
        break;
      }
    }
    for(int i = x1 + 1; i < mapX + 16; i++) {
      if (playingMode.enemiesMap[mapY][i] == EnemyTypes.SCALE_END) {
        x2 = i;
        break;
      }
    }
    for(int i = mapY + 1; i < 15; i++) {
      if (playingMode.enemiesMap[i][x1] == EnemyTypes.SCALE_END) {
        y1 = i;
        break;
      }
    }
    for(int i = mapY + 1; i < 15; i++) {
      if (playingMode.enemiesMap[i][x2] == EnemyTypes.SCALE_END) {
        y2 = i;
        break;
      }
    }

    scalePlatform1 = new ScalePlatform(
        16 + (x1 << 5) - (width >> 1), y1 << 5, width, 16, (mapY << 5) + 32);
    scalePlatform2 = new ScalePlatform(
        16 + (x2 << 5) - (width >> 1), y2 << 5, width, 16, (mapY << 5) + 32);
    scalePlatform1.otherScalePlatform = scalePlatform2;
    scalePlatform2.otherScalePlatform = scalePlatform1;

    playingMode.platforms.add(scalePlatform1);
    playingMode.platforms.add(scalePlatform2);

    horizontalWidth = ((x2 - x1) << 5) - 11;
    this.fx = this.x = x1 << 5;
    this.fy = this.y = mapY << 5;
  }

  @Override
  public void update() {
    if (x + horizontalWidth + scalePlatform2.width < playingMode.cameraX) {
      remove = true;
    }
  }

  @Override
  public void render() {

    g.translate(x - playingMode.cameraX, y);

    g.setColor(Palette.SCALE_STRING);
    g.fillArc(13, 7, 22, 22, 90, 90);
    g.fillArc(7 + horizontalWidth, 7, 22, 22, 0, 90);

    g.setColor(Palette.SCALE_WHEEL);
    g.fillOval(17, 11, 14, 14);
    g.fillOval(11 + horizontalWidth, 11, 14, 14);

    g.setColor(Palette.SCALE_STRING);
    
    g.translate(playingMode.cameraX - x, -y);

    g.translate(scalePlatform1.x - playingMode.cameraX, 0);
    g.fillRect(halfWidth - 2, y + 16, 4,
        (scalePlatform1.broken ? scalePlatform1.brokeY : scalePlatform1.y) 
            - y - 16);
    g.translate(playingMode.cameraX - scalePlatform1.x, 0);

    g.translate(scalePlatform2.x - playingMode.cameraX, 0);
    g.fillRect(halfWidth - 2, y + 16, 4,
        (scalePlatform2.broken ? scalePlatform2.brokeY : scalePlatform2.y) 
            - y - 16);
    g.translate(playingMode.cameraX - scalePlatform2.x, 0);

    g.translate(x - playingMode.cameraX, y);
    g.fillRect(22, 8, horizontalWidth, 4);
    g.translate(playingMode.cameraX - x, -y);
  }
}
