package mario;

import java.awt.*;

public class Spiny extends WorldObject {

  public Lakitu lakitu;

  public Spiny(int x, int y, boolean mapCoordinates, Lakitu lakitu) {
    if (mapCoordinates) {
      x <<= 5;
      y <<= 5;
    }
    this.fx = this.x = x;
    this.fy = this.y = y;

    vy = -5f;

    supports = false;
    this.lakitu = lakitu;
  }

  public void update() {

    for(int i = playingMode.platforms.size() - 1; i >= 0; i--) {
      AbstractPlatform platform = playingMode.platforms.get(i);
      if (platform.deadly && platform.intersects(this)) {
        die();
        playingMode.others.add(new UpsideDownDeadSpiny(x, y,
            platform.x < x ? 2 : -2, -9));
        return;
      }
    }

    if (vx == 0) {
      G = 0.2f;
    } else {
      G = 0.58f;
    }

    boolean supported = applyGravity(true);

    if (vx == 0 && supported) {
      if (x < playingMode.mario.x) {
        vx = 1f;
      } else {
        vx = -1f;
      }
    }

    if (vx < 0) {
      if (playingMode.withEnemyMargin(this, (int)(fx + vx), y + 15)
          != null || !translateX(vx, false)) {
        vx = Math.abs(vx);
      }
    } else if (vx > 0) {
      if (playingMode.withEnemyMargin(this, 31 + (int)(fx + vx), y + 15)
          != null || !translateX(vx, false)) {
        vx = -Math.abs(vx);
      }
    }

    if (x + 48 < playingMode.cameraX || y > 480
        || x > playingMode.cameraX + 544) {
      die();
    }
  }

  @Override
  public boolean blast(int px, int py, boolean left) {
    if (withinMargin(px, py)) {
      playingMode.others.add(
          new UpsideDownDeadSpiny(x, y, left ? -4 : 4, -8));
      playingMode.addPoints(this, 200);
      die();
      return true;
    }
    return false;
  }

  @Override
  public void starred(boolean left) {
    shelled(left);
    playingMode.addPoints(this, 200);
  }

  @Override
  public void shelled(boolean left) {
    playingMode.others.add(
        new UpsideDownDeadSpiny(x, y, left ? -4 : 4, -8));
    playingMode.addPoints(this, 200);
    die();
  }

  @Override
  public boolean hurtsMario() {
    return !remove && intersects(playingMode.mario, true);
  }

  private void die() {
    remove = true;
    lakitu.spinyDied();
  }

  public void render() {
    g.translate(x - playingMode.cameraX, y);
    g.setColor(Palette.KOOPA_TROOPA_LEG);
    g.drawLine(3, 17, 3, 31);
    g.drawLine(28, 17, 28, 31);
    g.drawLine(3, 31, 12, 31);
    g.drawLine(12, 27, 12, 31);
    g.drawLine(19, 27, 19, 31);
    g.drawLine(19, 31, 28, 31);
    g.setColor(Palette.SPINY_SPIKES);
    g.drawLine(16, 0, 11, 6);
    g.drawLine(4, 10, 0, 8);
    g.drawLine(0, 8, 2, 15);
    g.drawLine(4, 0, 4, 10);
    g.drawLine(4, 0, 11, 6);
    g.drawLine(27, 10, 31, 8);
    g.drawLine(31, 8, 29, 15);
    g.drawLine(27, 0, 27, 10);
    g.drawLine(27, 0, 20, 6);
    g.drawLine(16, 0, 20, 6);
    g.setColor(Palette.SPINY_SHELL);
    g.drawArc(2, 7, 27, 15, 0, 180);
    g.setColor(Color.WHITE);
    g.drawLine(0, 16, 5, 16);
    g.drawLine(26, 16, 31, 16);
    g.drawArc(5, 7, 21, 20, 0, -180);
    g.translate(playingMode.cameraX - x, -y);
  }

}

