package mario;

import java.awt.*;

public class StaticKoopaShell extends WorldObject {

  public int timeOut = 5 * 60;
  public Color color;
  public int type;

  public StaticKoopaShell(int x, int y, float vx, float vy, int type) {
    this.fx = this.x = x;
    this.fy = this.y = y;
    this.vx = vx;
    this.vy = vy;
    this.type = type;

    switch(type) {
      case KoopaTroopaTypes.GREEN:
        color = Palette.GREEN_KOOPA_TROOPA;
        break;
      case KoopaTroopaTypes.RED:
        color = Palette.RED_KOOPA_TROOPA;
        break;
      case KoopaTroopaTypes.BUZZY_BEETLE:
        color = Palette.BUZZY_BEETLE;
        break;
    }

    mx1 = bx1 = 0;
    mx2 = bx2 = 31;
    my1 = by1 = 4;
    my2 = by2 = 31;
  }

  public void update() {

    for(int i = playingMode.platforms.size() - 1; i >= 0; i--) {
      AbstractPlatform platform = playingMode.platforms.get(i);
      if (platform.deadly && platform.intersects(this)) {
        vx = platform.x < x ? 4 : -4;
        vy = -9f;
        while(platform.intersects(this)) {
          y--;
          fy = y;
        }
        timeOut = 5 * 60;
        return;
      }
    }

    if (applyGravity(true)) {
      vx = 0;
    }

    if (vx != 0 && !translateX(vx, false)) {
      vx = -vx;
    }

    if (--timeOut == 0) {
      remove = true;
      playingMode.enemies.add(new KoopaTroopa(x, y, false, type));
      return;
    }

    if (x < playingMode.cameraX - 64
        || x > playingMode.cameraX + 576 || y > 480) {
      remove = true;
      return;
    }
  }

  @Override
  public boolean blast(int px, int py, boolean left) {
    if (type != KoopaTroopaTypes.BUZZY_BEETLE && withinMargin(px, py)) {
      playingMode.others.add(
          new UpsideDownDeadKoopaShell(x, y, left ? -4 : 4, type));
      playingMode.addPoints(this, 200);
      remove = true;
      return true;
    }
    return false;
  }

  @Override
  public void starred(boolean left) {
    shelled(left);
    playingMode.addPoints(this, 200);
  }

  @Override
  public void shelled(boolean left) {
    playingMode.others.add(
        new UpsideDownDeadKoopaShell(x, y, left ? -4 : 4, type));
    remove = true;
  }

  public void render() {
    g.translate(x - playingMode.cameraX, y + 4);
    if (timeOut < 2 * 60 && ((timeOut >> 3) & 1) == 0) {
      g.setColor(Palette.KOOPA_TROOPA_LEG);
      g.drawLine(3, 17, 3, 27);
      g.drawLine(28, 17, 28, 27);
      g.drawLine(3, 27, 12, 27);
      g.drawLine(12, 27, 12, 27);
      g.drawLine(19, 27, 19, 27);
      g.drawLine(19, 27, 28, 27);
    }
    g.setColor(color);
    g.drawArc(2, 0, 27, 30, 0, 180);
    g.setColor(Color.WHITE);
    g.drawLine(0, 16, 5, 16);
    g.drawLine(26, 16, 31, 16);
    g.drawArc(5, 7, 21, 20, 0, -180);
    g.translate(playingMode.cameraX - x, -y - 4);
  }
}
