package mario;

import java.awt.*;

public class Text {

  private static final int DEFAULT_SIZE = 13;
  private static final int DEFAULT_PADDING = 3;

  private static int[][] glyphs = new int[256][];
  private static int size;
  private static int s1;
  private static int s2;
  private static int s3;
  private static int t1;
  private static int t2;

  // | | | | |   ----5----     -10- -11-    \26   27/
  // | | | | |               12|\  |  /|16   \     /                    33
  // | | | | |   ----6----     13 14 15|      \   /           31  32   ---
  // | | | | |                 |  \|/  |       \ /              \/   35| |36
  // 0 1 2 3 4   ----7----     -17- -18-        \               /\     ---
  // | | | | |                 |  /|\  |       / \                      34
  // | | | | |   ----8----     19 20 21|      /   \          | 28
  // | | | | |               22|/  |  \|23   /     \
  // | | | | |   ----9----     -24- -25-    /       \        . 29   |30

  // \37  38/
  //  \    /
  //   \  /
  //    \/

  static {
    updateSize(DEFAULT_SIZE);

    glyphs['A'] = glyphs['a'] = new int[] { 0, 5, 7, 4 };
    glyphs['B'] = glyphs['b'] = new int[] { 5, 9, 2, 4, 18 };
    glyphs['C'] = glyphs['c'] = new int[] { 5, 0, 9};
    glyphs['D'] = glyphs['d'] = new int[] { 5, 9, 2, 4 };
    glyphs['E'] = glyphs['e'] = new int[] { 5, 17, 9, 0 };
    glyphs['F'] = glyphs['f'] = new int[] { 5, 0, 17 };
    glyphs['G'] = glyphs['g'] = new int[] { 0, 5, 9, 18, 23 };
    glyphs['H'] = glyphs['h'] = new int[] { 0, 4, 7 };
    glyphs['I'] = glyphs['i'] = new int[] { 5, 9, 2 };
    glyphs['J'] = glyphs['j'] = new int[] { 22, 9, 4 };
    glyphs['K'] = glyphs['k'] = new int[] { 0, 17, 15, 21 };
    glyphs['L'] = glyphs['l'] = new int[] { 0, 9 };
    glyphs['M'] = glyphs['m'] = new int[] { 0, 4, 13, 15 };
    glyphs['N'] = glyphs['n'] = new int[] { 0, 26, 4 };
    glyphs['O'] = glyphs['o'] = new int[] { 0, 4, 5, 9 };
    glyphs['P'] = glyphs['p'] = new int[] { 0, 5, 7, 16 };
    glyphs['Q'] = glyphs['q'] = new int[] { 0, 4, 5, 9, 21 };
    glyphs['R'] = glyphs['r'] = new int[] { 0, 5, 7, 16, 21 };
    glyphs['S'] = glyphs['s'] = new int[] { 5, 7, 9, 12, 23 };
    glyphs['T'] = glyphs['t'] = new int[] { 5, 2 };
    glyphs['U'] = glyphs['u'] = new int[] { 0, 4, 9 };
    glyphs['V'] = glyphs['v'] = new int[] { 37, 38 };
    glyphs['W'] = glyphs['w'] = new int[] { 0, 4, 19, 21 };
    glyphs['X'] = glyphs['x'] = new int[] { 26, 27 };
    glyphs['Y'] = glyphs['y'] = new int[] { 13, 15, 20 };
    glyphs['Z'] = glyphs['z'] = new int[] { 5, 9, 27 };
    glyphs['0'] = new int[] { 0, 4, 5, 9, 27 };
    glyphs['1'] = new int[] { 2 };
    glyphs['2'] = new int[] { 5, 7, 9, 16, 22 };
    glyphs['3'] = new int[] { 4, 5, 7, 9 };
    glyphs['4'] = new int[] { 4, 12, 7 };
    glyphs['5'] = new int[] { 5, 7, 9, 12, 23 };
    glyphs['6'] = new int[] { 5, 7, 9, 0, 23 };
    glyphs['7'] = new int[] { 5, 4 };
    glyphs['8'] = new int[] { 5, 7, 9, 0, 4 };
    glyphs['9'] = new int[] { 5, 7, 9, 4, 12 };
    glyphs[' '] = new int[] { };
    glyphs['<'] = new int[] { 15, 21 };
    glyphs['>'] = new int[] { 13, 19 };
    glyphs['*'] = new int[] { 2, 7, 26, 27 };
    glyphs['!'] = new int[] { 14, 28, 29 };
    glyphs['?'] = new int[] { 5, 16, 18, 28, 29 };
    glyphs['.'] = new int[] { 29 };
    glyphs[','] = new int[] { 30 };
    glyphs['-'] = new int[] { 7 };
    glyphs['='] = new int[] { 6, 8 };
    glyphs['+'] = new int[] { 2, 7 };
    glyphs['_'] = new int[] { 9 };
    glyphs['|'] = new int[] { 2 };
    glyphs['['] = new int[] { 2, 11, 25 };
    glyphs[']'] = new int[] { 2, 10, 24 };
    glyphs['/'] = new int[] { 27 };
    glyphs['\\'] = new int[] { 26 };
    glyphs['~'] = new int[] { 31, 32 };
    glyphs['@'] = new int[] { 0, 4, 5, 9, 33, 34, 35 };
  }

  private static void drawLine(Graphics g, int lineIndex) {
    switch(lineIndex) {
      case 0:
        g.drawLine(0, 0, 0, size);
        break;
      case 1:
        g.drawLine(s1, 0, s1, size);
        break;
      case 2:
        g.drawLine(s2, 0, s2, size);
        break;
      case 3:
        g.drawLine(s3, 0, s3, size);
        break;
      case 4:
        g.drawLine(size, 0, size, size);
        break;
      case 5:
        g.drawLine(0, 0, size, 0);
        break;
      case 6:
        g.drawLine(0, s1, size, s1);
        break;
      case 7:
        g.drawLine(0, s2, size, s2);
        break;
      case 8:
        g.drawLine(0, s3, size, s3);
        break;
      case 9:
        g.drawLine(0, size, size, size);
        break;
      case 10:
        g.drawLine(0, 0, s2, 0);
        break;
      case 11:
        g.drawLine(s2, 0, size, 0);
        break;
      case 12:
        g.drawLine(0, 0, 0, s2);
        break;
      case 13:
        g.drawLine(0, 0, s2, s2);
        break;
      case 14:
        g.drawLine(s2, 0, s2, s2);
        break;
      case 15:
        g.drawLine(size, 0, s2, s2);
        break;
      case 16:
        g.drawLine(size, 0, size, s2);
        break;
      case 17:
        g.drawLine(0, s2, s2, s2);
        break;
      case 18:
        g.drawLine(s2, s2, size, s2);
        break;
      case 19:
        g.drawLine(s2, s2, 0, size);
        break;
      case 20:
        g.drawLine(s2, s2, s2, size);
        break;
      case 21:
        g.drawLine(s2, s2, size, size);
        break;
      case 22:
        g.drawLine(0, s2, 0, size);
        break;
      case 23:
        g.drawLine(size, s2, size, size);
        break;
      case 24:
        g.drawLine(0, size, s2, size);
        break;
      case 25:
        g.drawLine(s2, size, size, size);
        break;
      case 26:
        g.drawLine(0, 0, size, size);
        break;
      case 27:
        g.drawLine(size, 0, 0, size);
        break;
      case 28:
        g.drawLine(s2, s2, s2, s3);
        break;
      case 29:
        g.drawLine(s2, size, s2, size);
        break;
      case 30:
        g.drawLine(s2, s3, s2, size);
        break;
      case 31:
        g.drawLine(s1, s2, s3, size);
        break;
      case 32:
        g.drawLine(s3, s2, s1, size);
        break;
      case 33:
        g.drawLine(t1, t1, t2, t1);
        break;
      case 34:
        g.drawLine(t1, t2, t2, t2);
        break;
      case 35:
        g.drawLine(t1, t1, t1, t2);
        break;
      case 36:
        g.drawLine(t2, t1, t2, t2);
        break;
      case 37:
        g.drawLine(0, 0, s2, size);
        break;
      case 38:
        g.drawLine(size, 0, s2, size);
        break;
    }
  }

  public static void drawString(int x, int y, String string) {
    drawString(x, y, string, DEFAULT_SIZE, DEFAULT_PADDING);
  }

  public static void drawChar(int x, int y, char c) {
    drawChar(x, y, c, DEFAULT_SIZE);
  }

  public static void drawString(int x, int y, String string,           
      int size, int padding) {
    updateSize(size);
    Graphics g = Main.main.g;
    for(int i = 0; i < string.length(); i++, x += size + padding) {
      drawChar(x, y, string.charAt(i), g);
    }
  }

  private static void drawChar(int x, int y, char c, Graphics g) {
    int[] lines = glyphs[c];
    if (lines != null) {
      g.translate(x, y);
      for(int i = lines.length - 1; i >= 0; i--) {
        drawLine(g, lines[i]);
      }
      g.translate(-x, -y);
    }
  }
  
  public static void drawChar(int x, int y, char c, int size) {
    updateSize(size);
    drawChar(x, y, c, Main.main.g);
  }

  public static void drawNumber(int x, int y, int value, int digits) {
    drawNumber(x, y, value, digits, DEFAULT_SIZE, DEFAULT_PADDING);
  }

  public static void drawNumber(int x, int y, int value, int digits,
      int size, int padding) {
    updateSize(size);
    Graphics g = Main.main.g;
    if (value < 0) {
      value = 0;
    }
    x += (digits - 1) * (size + padding);
    for(int i = 0; i < digits; i++, x -= size + padding, value /= 10) {
      drawChar(x, y, (char)('0' + (value % 10)), g);
    }
  }

  public static void drawNumberHiddenLeading(
      int x, int y, int value, int digits) {
    drawNumberHiddenLeading(x, y, value, digits, DEFAULT_SIZE, DEFAULT_PADDING);
  }

  public static void drawNumberHiddenLeading(
      int x, int y, int value, int digits,
      int size, int padding) {
    updateSize(size);
    Graphics g = Main.main.g;
    if (value < 0) {
      value = 0;
    }
    x += (digits - 1) * (size + padding);
    for(int i = 0; i < digits && value > 0; i++, x -= size + padding, value /= 10) {
      drawChar(x, y, (char)('0' + (value % 10)), g);
    }
  }

  private static void updateSize(int size) {
    if (size != Text.size) {
      Text.size = size;
      s1 = size >> 2;
      s2 = size >> 1;
      s3 = s1 + s2;
      t1 = size / 3;
      t2 = t1 << 1;
    }
  }
}
