package mario;

import java.awt.*;
import java.awt.event.*;

public class TitleScreenMode implements IMode {

  public static final int DEMO_DELAY = 15 * 60;

  public static final int MENU_MAIN = 0;
  public static final int MENU_OPTIONS = 1;
  public static final int MENU_INPUT = 2;
  public static final int MENU_DIFFICULTY = 3;
  public static final int MENU_PLAYER = 4;
  public static final int MENU_READING_INPUT = 5;
  public static final int MENU_GAME_OVER = 6;

  public static final int MAIN_START = 0;
  public static final int MAIN_OPTIONS = 1;

  public static final int OPTIONS_INPUT = 0;
  public static final int OPTIONS_DIFFICULTY = 1;
  public static final int OPTIONS_PLAYER = 2;

  public static final int INPUT_CHANGE = 0;
  public static final int INPUT_MAIN_MENU = 1;

  public static final int DIFFICULTY_EASY = 0;
  public static final int DIFFICULTY_HARD = 1;

  public static final int PLAYER_MARIO = 0;
  public static final int PLAYER_LUIGI = 1;

  public static final int READING_INPUT_UP = 0;
  public static final int READING_INPUT_DOWN = 1;
  public static final int READING_INPUT_LEFT = 2;
  public static final int READING_INPUT_RIGHT = 3;
  public static final int READING_INPUT_RUN = 4;
  public static final int READING_INPUT_JUMP = 5;
  public static final int READING_INPUT_PAUSE = 6;
  public static final int READING_INPUT_SELECT = 7;

  public static final int GAME_OVER_CONTINUE = 0;
  public static final int GAME_OVER_END = 1;

  public static final String[] READING_INPUT_MESSAGES = {
    "PRESS UP",
    "PRESS DOWN",
    "PRESS LEFT",
    "PRESS RIGHT",
    "PRESS RUN",
    "PRESS JUMP",
    "PRESS PAUSE",
    "PRESS START",
  };
  public static final String[] INPUTS_EQUAL = {
    "UP    = UP",
    "DOWN  = DOWN",
    "LEFT  = LEFT",
    "RIGHT = RIGHT",
    "RUN   = D",
    "JUMP  = F",
    "PAUSE = P",
    "START = ENTER",
  };
  
  public Main main;
  public Graphics g;
  public int menuY = 300;
  public int optionsCount = 2;
  public int optionIndex = 0;
  public int selectedY = 300;
  public IInput input;
  public int menu = MENU_MAIN;
  public boolean keyHeld;
  public int readingInputIndex;
  public boolean startCleared;
  public int demoDelay;

  public TitleScreenMode(boolean gameOver) {
    main = Main.main;
    g = main.g;

    if (gameOver) {
      menu = MENU_GAME_OVER;
      menuY = selectedY = 256;
    }
  }

  public void init() {
    input = new KeyboardInput();
  }

  public void update() {

    if (!startCleared) {
      if (!input.isStart()) {
        startCleared = true;
      } else {
        return;
      }
    }

    if (keyHeld) {
      demoDelay = 0;
      if (main.K[main.lastKeyPressed]) {
        return;
      } else {
        keyHeld = false;
        main.lastKeyPressed = 0;
      }
    }

    if (menu == MENU_MAIN) {
      if (++demoDelay >= DEMO_DELAY) {
        main.startDemo();
      }
    } else {
      demoDelay = 0;
    }

    if (menu == MENU_READING_INPUT) {
      if (main.lastKeyPressed > 0) {
        keyHeld = true;
        switch(readingInputIndex) {
          case READING_INPUT_UP:
            main.upKeyCode = main.lastKeyPressed;
            INPUTS_EQUAL[READING_INPUT_UP] = "UP    = "
                + KeyEvent.getKeyText(main.upKeyCode).toUpperCase();
            break;
          case READING_INPUT_DOWN:
            main.downKeyCode = main.lastKeyPressed;
            INPUTS_EQUAL[READING_INPUT_DOWN] = "DOWN  = "
                + KeyEvent.getKeyText(main.downKeyCode).toUpperCase();
            break;
          case READING_INPUT_LEFT:
            main.leftKeyCode = main.lastKeyPressed;
            INPUTS_EQUAL[READING_INPUT_LEFT] = "LEFT  = "
                + KeyEvent.getKeyText(main.leftKeyCode).toUpperCase();
            break;
          case READING_INPUT_RIGHT:
            main.rightKeyCode = main.lastKeyPressed;
            INPUTS_EQUAL[READING_INPUT_RIGHT] = "RIGHT = "
                + KeyEvent.getKeyText(main.rightKeyCode).toUpperCase();
            break;
          case READING_INPUT_RUN:
            main.runKeyCode = main.lastKeyPressed;
            INPUTS_EQUAL[READING_INPUT_RUN] = "RUN   = "
                + KeyEvent.getKeyText(main.runKeyCode).toUpperCase();
            break;
          case READING_INPUT_JUMP:
            main.jumpKeyCode = main.lastKeyPressed;
            INPUTS_EQUAL[READING_INPUT_JUMP] = "JUMP  = "
                + KeyEvent.getKeyText(main.jumpKeyCode).toUpperCase();
            break;
          case READING_INPUT_PAUSE:
            main.pauseKeyCode = main.lastKeyPressed;
            INPUTS_EQUAL[READING_INPUT_PAUSE] = "PAUSE = "
                + KeyEvent.getKeyText(main.pauseKeyCode).toUpperCase();
            break;
          case READING_INPUT_SELECT:
            main.startKeyCode = main.lastKeyPressed;
            INPUTS_EQUAL[READING_INPUT_SELECT] = "START = "
                + KeyEvent.getKeyText(main.startKeyCode).toUpperCase();
            break;
        }
        if (++readingInputIndex > READING_INPUT_SELECT) {
          menu = MENU_INPUT;
          optionsCount = 2;
          optionIndex = 0;
          selectedY = menuY = 398;
          input = new KeyboardInput();
        }
      }
    } else {     
      if (input.isUp()) {
        keyHeld = true;
        if (optionIndex > 0) {
          optionIndex--;
        }
        selectedY = menuY + (optionIndex << 5);
      } else if (input.isDown()) {
        keyHeld = true;
        if (optionIndex < optionsCount - 1) {
          optionIndex++;
        }
        selectedY = menuY + (optionIndex << 5);
      } else if (input.isStart()) {
        keyHeld = true;
        switch(menu) {
          case MENU_MAIN:
            switch(optionIndex) {
              case MAIN_START:
                main.startGame();
                break;
              case MAIN_OPTIONS:
                menu = MENU_OPTIONS;
                optionsCount = 3;
                optionIndex = 0;
                selectedY = menuY = 300;
                break;
            }
            break;
          case MENU_OPTIONS:
            switch(optionIndex) {
              case OPTIONS_INPUT:
                menu = MENU_INPUT;
                optionsCount = 2;
                optionIndex = 0;
                selectedY = menuY = 398;
                break;
              case OPTIONS_DIFFICULTY:
                menu = MENU_DIFFICULTY;
                optionsCount = 2;
                optionIndex = main.hardMode ? 1 : 0;
                menuY = 300;
                selectedY = menuY + (optionIndex << 5);
                break;
              case OPTIONS_PLAYER:
                menu = MENU_PLAYER;
                optionsCount = 2;
                optionIndex = main.luigiMode ? 1 : 0;
                menuY = 300;
                selectedY = menuY + (optionIndex << 5);
                break;
            }
            break;
          case MENU_INPUT:
            switch(optionIndex) {
              case INPUT_CHANGE:
                menu = MENU_READING_INPUT;
                readingInputIndex = 0;
                break;
              case INPUT_MAIN_MENU:
                menu = MENU_MAIN;
                optionsCount = 2;
                optionIndex = 0;
                selectedY = menuY = 300;
                break;
            }
            break;
          case MENU_DIFFICULTY:
            switch(optionIndex) {
              case DIFFICULTY_EASY:
                main.hardMode = false;
                menu = MENU_MAIN;
                optionsCount = 2;
                optionIndex = 0;
                selectedY = menuY = 300;
                break;
              case DIFFICULTY_HARD:
                main.hardMode = true;
                menu = MENU_MAIN;
                optionsCount = 2;
                optionIndex = 0;
                selectedY = menuY = 300;
                break;
            }
            break;
          case MENU_PLAYER:
            switch(optionIndex) {
              case PLAYER_MARIO:
                main.luigiMode = false;
                menu = MENU_MAIN;
                optionsCount = 2;
                optionIndex = 0;
                selectedY = menuY = 300;
                break;
              case PLAYER_LUIGI:
                main.luigiMode = true;
                menu = MENU_MAIN;
                optionsCount = 2;
                optionIndex = 0;
                selectedY = menuY = 300;
                break;
            }
            break;
          case MENU_GAME_OVER:
            switch(optionIndex) {
              case GAME_OVER_CONTINUE:
                main.continueGame();
                break;
              case GAME_OVER_END:
                menu = MENU_MAIN;
                optionsCount = 2;
                optionIndex = 0;
                selectedY = menuY = 300;
                break;
            }
            break;
        }
      }
    }
  }

  public void render() {
    g.setColor(Color.BLACK);
    g.fillRect(0, 0, 512, 480);

    if (menu != MENU_GAME_OVER) {
      g.setColor(Palette.TITLE);
      Text.drawString(78, 80, "WORST", 32, 4);
      Text.drawString(78, 128, "MARIO GAME", 32, 4);
      Text.drawString(294, 176, "EVER", 32, 4);

      g.setColor(Palette.TITLE_BOX);
      g.drawRect(68, 68, 376, 152);
    }

    switch(menu) {
      case MENU_MAIN:
        renderMainMenu();
        break;
      case MENU_OPTIONS:
        renderOptionsMenu();
        break;
      case MENU_INPUT:
        renderInputMenu();
        break;
      case MENU_DIFFICULTY:
        renderDifficultyMenu();
        break;
      case MENU_PLAYER:
        renderPlayerMenu();
        break;
      case MENU_READING_INPUT:
        renderReadingInput();
        break;
      case MENU_GAME_OVER:
        renderGameOverMenu();
        break;
    }
  }

  public void renderMainMenu() {
    g.setColor(Color.WHITE);
    Text.drawString(224, 300, "START");
    Text.drawString(224, 332, "OPTIONS");
    Text.drawChar(200, selectedY, '>');
    Text.drawString(81, 448, "@ 2010 MEATFIGHTER.COM");
  }

  public void renderOptionsMenu() {
    g.setColor(Color.WHITE);
    Text.drawString(224, 300, "INPUT");
    Text.drawString(224, 332, "DIFFICULTY");
    Text.drawString(224, 364, "PLAYER");
    Text.drawChar(200, selectedY, '>');
    Text.drawString(81, 448, "@ 2010 MEATFIGHTER.COM");
  }

  public void renderInputMenu() {
    g.setColor(Palette.TITLE);
    Text.drawString(146, 250, INPUTS_EQUAL[READING_INPUT_DOWN]);
    Text.drawString(146, 286, INPUTS_EQUAL[READING_INPUT_RIGHT]);
    Text.drawString(146, 322, INPUTS_EQUAL[READING_INPUT_JUMP]);
    Text.drawString(146, 358, INPUTS_EQUAL[READING_INPUT_SELECT]);

    g.setColor(Color.WHITE);
    Text.drawString(146, 232, INPUTS_EQUAL[READING_INPUT_UP]);
    Text.drawString(146, 268, INPUTS_EQUAL[READING_INPUT_LEFT]);
    Text.drawString(146, 304, INPUTS_EQUAL[READING_INPUT_RUN]);
    Text.drawString(146, 340, INPUTS_EQUAL[READING_INPUT_PAUSE]);
    Text.drawString(224, 398, "CHANGE");
    Text.drawString(224, 430, "MAIN MENU");

    Text.drawChar(200, selectedY, '>');
  }

  public void renderDifficultyMenu() {
    g.setColor(Color.WHITE);
    Text.drawString(224, 300, "EASY");
    Text.drawString(224, 332, "HARD");
    Text.drawChar(200, selectedY, '>');
    Text.drawString(81, 448, "@ 2010 MEATFIGHTER.COM");
  }

  public void renderPlayerMenu() {
    g.setColor(Color.WHITE);
    Text.drawString(224, 300, "MARIO");
    Text.drawString(224, 332, "LUIGI");
    Text.drawChar(200, selectedY, '>');
    Text.drawString(81, 448, "@ 2010 MEATFIGHTER.COM");
  }

  public void renderReadingInput() {
    g.setColor(Color.WHITE);
    Text.drawString(200, 300, READING_INPUT_MESSAGES[readingInputIndex]);
    Text.drawString(81, 448, "@ 2010 MEATFIGHTER.COM");
  }

  public void renderGameOverMenu() {
    g.setColor(Color.WHITE);
    Text.drawString(192, 176, "GAME OVER");
    Text.drawString(208, 256, "CONTINUE");
    Text.drawString(208, 288, "END");
    Text.drawChar(176, selectedY, '>');
  }
}
