package mario;

import java.awt.*;
import java.util.*;

public abstract class WorldObject implements IObject {

  public float G = 0.58f;

  public PlayingMode playingMode;
  public Main main;
  public Graphics g;
  public Random random;
  public int x;
  public int y;
  public float fx;
  public float fy;
  public float vx;
  public float vy;
  public int bx1;
  public int by1;
  public int bx2;
  public int by2;
  public int mx1;
  public int my1;
  public int mx2;
  public int my2; 
  public boolean remove;
  public boolean supports = true;

  public WorldObject() {
    main = Main.main;
    g = main.g;
    playingMode = Main.playingMode;
    random = main.random;
    bx1 = mx1 = 0;
    by1 = my1 = 0;
    bx2 = mx2 = 31;
    by2 = my2 = 31;
  }

  public void crush() {
  }

  public boolean blast(int px, int py, boolean left) {
    return false;
  }

  public void starred(boolean left) {
  }

  public void shelled(boolean left) {
  }

  public boolean applyGravity(boolean useBorder) {
    vy += G;
    if (vy >= 0) {
      if (isSupported(useBorder)) {
        vy = 0;
        return true;
      } else {
        return !translateY(vy, useBorder);
      }
    } else {
      if (!translateY(vy, useBorder)) {
        vy = -vy;
      }
      return false;
    }
  }

  public boolean supportedBy(WorldObject object) {
    return intersects(
        x + mx1,
        y + my2 + 1,
        x + mx2,
        y + my2 + 1,
        object.x + object.mx1,
        object.y + object.my1,
        object.x + object.mx2,
        object.y + object.my2);
  }

  public boolean isSupported(boolean useBorder) {

    int mapY = 0;
    int mapX1 = 0;
    int mapX2 = 0;
    int rx1 = 0;
    int ry1 = 0;
    int rx2 = 0;
    int ry2 = 0;

    if (useBorder) {
      rx1 = x + bx1;
      rx2 = x + bx2;
      ry1 = y + by2 + 1;
    } else {
      rx1 = x + mx1;
      rx2 = x + mx2;
      ry1 = y + my2 + 1;
    }
    ry2 = ry1;
    mapX1 = rx1 >> 5;
    mapX2 = rx2 >> 5;
    mapY = ry2 >> 5;

    if (isSolid(rx1, ry1, rx2, ry2, DirectionTypes.NONE)) {
      return true;
    }

    for(int i = mapX1; i <= mapX2; i++) {
      if (isMapSolid(i, mapY, DirectionTypes.NONE)) {
        int sx = i << 5;
        int sy = mapY << 5;
        return intersects(rx1, ry1, rx2, ry2, sx, sy, sx + 31, sy + 31);
      }
    }

    return false;
  }

  public boolean translateX(float dx, boolean useBorder) {

    float fx2 = fx + dx;
    int x2 = (int)fx2;

    int mapX = 0;
    int mapY1 = 0;
    int mapY2 = 0;
    int rx1 = 0;
    int ry1 = 0;
    int rx2 = 0;
    int ry2 = 0;
    
    if (useBorder) {
      ry1 = y + by1;
      ry2 = y + by2;
    } else {
      ry1 = y + my1;
      ry2 = y + my2;
    }
    mapY1 = ry1 >> 5;
    mapY2 = ry2 >> 5;

    if (x > x2) {
      for(int i = x - 1; i >= x2; i--) {
        if (useBorder) {
          rx1 = i + bx1;
          rx2 = i + bx2;
        } else {
          rx1 = i + mx1;
          rx2 = i + mx2; 
        }
        if (isSolid(rx1, ry1, rx2, ry2, DirectionTypes.LEFT)) {
          x = i + 1;
          fx = x;
          return false;
        }
        mapX = rx1 >> 5;
        for(int j = mapY1; j <= mapY2; j++) {
          if (isMapSolid(mapX, j, DirectionTypes.LEFT)) {
            int sx = mapX << 5;
            int sy = j << 5;
            if (intersects(rx1, ry1, rx2, ry2, sx, sy, sx + 31, sy + 31)) {
              x = i + 1;
              fx = x;
              return false;
            }
          }
        }
      }
    } else if (x < x2) {
      for(int i = x + 1; i <= x2; i++) {
        if (useBorder) {
          rx1 = i + bx1;
          rx2 = i + bx2;
        } else {
          rx1 = i + mx1;
          rx2 = i + mx2;
        }
        if (isSolid(rx1, ry1, rx2, ry2, DirectionTypes.RIGHT)) {
          x = i - 1;
          fx = x;
          return false;
        }
        mapX = rx2 >> 5;
        for(int j = mapY1; j <= mapY2; j++) {
          if (isMapSolid(mapX, j, DirectionTypes.RIGHT)) {
            int sx = mapX << 5;
            int sy = j << 5;
            if (intersects(rx1, ry1, rx2, ry2, sx, sy, sx + 31, sy + 31)) {
              x = i - 1;
              fx = x;
              return false;
            }
          }
        }
      }
    }

    fx = fx2;
    x = x2;

    return true;
  }

  public boolean translateY(float dy, boolean useBorder) {

    float fy2 = fy + dy;
    int y2 = (int)fy2;

    int mapY = 0;
    int mapX1 = 0;
    int mapX2 = 0;
    int rx1 = 0;
    int ry1 = 0;
    int rx2 = 0;
    int ry2 = 0;

    if (useBorder) {
      rx1 = x + bx1;
      rx2 = x + bx2;
    } else {
      rx1 = x + mx1;
      rx2 = x + mx2;
    }
    mapX1 = rx1 >> 5;
    mapX2 = rx2 >> 5;

    if (y > y2) {
      for(int i = y - 1; i >= y2; i--) {
        if (useBorder) {
          ry1 = i + by1;
          ry2 = i + by2;
        } else {
          ry1 = i + my1;
          ry2 = i + my2;
        }
        if (isSolid(rx1, ry1, rx2, ry2, DirectionTypes.UP)) {
          y = i + 1;
          fy = y;
          return false;
        }
        mapY = ry1 >> 5;
        for(int j = mapX1; j <= mapX2; j++) {
          if (isMapSolid(j, mapY, DirectionTypes.UP)) {
            int sx = j << 5;
            int sy = mapY << 5;
            if (intersects(rx1, ry1, rx2, ry2, sx, sy, sx + 31, sy + 31)) {
              y = i + 1;
              fy = y;
              return false;
            }
          }
        }
      }
    } else if (y < y2) {
      for(int i = y + 1; i <= y2; i++) {
        if (useBorder) {
          ry1 = i + by1;
          ry2 = i + by2;
        } else {
          ry1 = i + my1;
          ry2 = i + my2;
        }
        if (isSolid(rx1, ry1, rx2, ry2, DirectionTypes.DOWN)) {
          y = i - 1;
          fy = y;
          return false;
        }
        mapY = ry2 >> 5;
        for(int j = mapX1; j <= mapX2; j++) {
          if (isMapSolid(j, mapY, DirectionTypes.DOWN)) {
            int sx = j << 5;
            int sy = mapY << 5;
            if (intersects(rx1, ry1, rx2, ry2, sx, sy, sx + 31, sy + 31)) {
              y = i - 1;
              fy = y;
              return false;
            }            
          }
        }
      }
    }

    fy = fy2;
    y = y2;

    return true;
  }

  public boolean isSolid(int rx1, int ry1, int rx2, int ry2, int direction) { 
    for(int i = playingMode.platforms.size() - 1; i >= 0; i--) {
      AbstractPlatform platform = playingMode.platforms.get(i);
      if (platform.intersects(rx1, ry1, rx2, ry2, false)) {
        return true;
      }
    }
    return false;
  }

  public boolean isPointSolid(int x, int y, int direction) {
    return isMapSolid(x >> 5, y >> 5, direction);
  }

  public boolean isMapSolid(int mapX, int mapY, int direction) {
    if (mapY < 0 || mapY > 14) {
      return false;
    }
    if (mapX < 0) {
      return mapY > 14;
    }
    if (mapX >= playingMode.mapWidth) {
      return true;
    }
    int block = playingMode.physicsMap[mapY][mapX];
    if (direction == DirectionTypes.UP) {
      return block >= BlockTypes.HIDDEN;
    } else {
      return block > BlockTypes.HIDDEN;
    }
  }

  public void removeBlock(int x, int y) {
    if (x >= 0 && y >= 0 && y < 480) {
      int Y = y >> 5;
      int X = x >> 5;
      playingMode.displayMap[Y][X] = BlockTypes.EMPTY;
      playingMode.physicsMap[Y][X] = BlockTypes.EMPTY;
    }
  }

  public void setPhysicsBlock(int x, int y, int block) {
    if (x >= 0 && y >= 0 && y < 480) {
      playingMode.physicsMap[y >> 5][x >> 5] = block;
    }
  }

  public int getPhysicsMap(int mapX, int mapY) {
    if (mapY < 0 || mapY > 14 || mapX < 0) {
      return BlockTypes.EMPTY;
    }    
    return playingMode.physicsMap[mapY][mapX];
  }

  public int getValuesBlock(int x, int y) {
    if (x < 0 || y < 0 || y >= 480) {
      return BlockTypes.EMPTY;
    }
    return playingMode.valuesMap[y >> 5][x >> 5];
  }

  public int getPhysicsBlock(int x, int y) {
    if (x < 0 || y < 0 || y >= 480) {
      return BlockTypes.EMPTY;
    }
    return playingMode.physicsMap[y >> 5][x >> 5];
  }

  public static boolean contains(
      int x1, int y1, int x2, int y2, int px, int py) {
    return px >= x1 && py >= y1 && px <= x2 && py <= y2;
  }

  public boolean contains(int px, int py) {
    return px >= x + bx1 && py >= y + by1 && px <= x + bx2 && py <= y + by2
        && !remove;
  }

  public boolean withinMargin(int px, int py) {
    return px >= x + mx1 && py >= y + my1 && px <= x + mx2 && py <= y + my2
        && !remove;
  }

  public static boolean intersects(
      int ax1, int ay1, int ax2, int ay2,
      int bx1, int by1, int bx2, int by2) {
    return ax2 >= bx1 && ax1 <= bx2 && ay2 >= by1 && ay1 <= by2;
  }

  public boolean intersects(WorldObject object, boolean useBorder) {
    if (remove || object.remove) {
      return false;
    }
    if (useBorder) {
      return intersects(object.x + object.bx1,
          object.y + object.by1, object.x + object.bx2,
          object.y + object.by2, x + bx1, y + by1, x + bx2, y + by2);
    } else {
      return intersects(object.x + object.mx1,
          object.y + object.my1, object.x + object.mx2,
          object.y + object.my2, x + mx1, y + my1, x + mx2, y + my2);
    }
  }

  public boolean intersects(
      int rx1, int ry1, int rx2, int ry2, boolean useBorder) {
    if (remove) {
      return false;
    }
    if (useBorder) {
      return intersects(rx1, ry1, rx2, ry2, x + bx1, y + by1, x + bx2, y + by2);
    } else {
      return intersects(rx1, ry1, rx2, ry2, x + mx1, y + my1, x + mx2, y + my2);
    }
  }

  public boolean intersects(WorldObject object) {
    if (remove || object.remove) {
      return false;
    }
    return intersects(
        x + mx1,
        y + my1,
        x + mx2,
        y + my2,
        object.x + object.mx1,
        object.y + object.my1,
        object.x + object.mx2,
        object.y + object.my2);
  }

  public boolean hurtsMario() {
    return false;
  }

  public boolean moveY(float dy) {

    if (playingMode.mario.supportedBy(this)) {
      return false;
    }

    float fy2 = fy + dy;
    int y2 = (int)fy2;

    if (y > y2) {
      for(y--; y >= y2; y--) {
        if (playingMode.mario.supportedBy(this)) {
          return false;
        }
      }
    }

    y = y2;
    fy = fy2;
    return true;
  }
}
